@@ -903,7 +903,7 @@ int line;
 	name_or_number(unit, tmpbuf);
 	sprintf(spbuf, "%c%d %-16s ",
 		unitchars[unit->type], side_number(unit->side), tmpbuf);
-	if (unit->act && unit->act->acp > 0) {
+	if (has_acp_left(unit)) {
 	    tprintf(spbuf, "%3d", unit->act->acp);
 	}
 	if (unit->plan && unit->plan->asleep) {
