@@ -7,6 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+char *next_ai_type(char *typename);
 #include "kernel.h"
 /* (should do a better separation of kernel in set_side_ai) */
 #include "kpublic.h"
@@ -36,6 +37,25 @@ init_ai_types(void)
     all_ai_ops[2] = &iplayer_ops;
 }
 
+char *
+next_ai_type(char *typename)
+{
+  int i;
+  char *name;
+
+  if (typename == NULL)
+    return all_ai_ops[1]->name;
+  for (i = 1; i < numaitypes; ++i) {
+    name = (all_ai_ops[i])->name;
+    if (strcmp(name, typename) == 0) {
+      if (i == (numaitypes - 1))
+	return NULL;
+      else
+	return (all_ai_ops[i+1])->name;
+    }
+  }
+}
+
 /* Given a side that wants to have an AI running its units, find and
    record the actual AI type that will be used, and run its
    initialization method. */
@@ -357,10 +377,7 @@ GoalDefn goaldefns[] = {
 /* General handling of goals. */
 
 Goal *
-create_goal(type, side, tf)
-GoalType type;
-Side *side;
-int tf;
+create_goal(GoalType type, Side *side, int tf)
 {
     Goal *goal = (Goal *) xmalloc(sizeof(Goal));
 
@@ -371,8 +388,7 @@ int tf;
 }
 
 int
-lookup_goal_type(name)
-char *name;
+lookup_goal_type(char *name)
 {
     int i;
 
@@ -383,15 +399,13 @@ char *name;
 }
 
 int
-cell_unknown(x, y)
-int x, y;
+cell_unknown(int x, int y)
 {
     return !(tmpside->see_all || terrain_view(tmpside, x, y) != UNSEEN);
 }
 
 int
-enemies_present(x, y)
-int x, y;
+enemies_present(int x, int y)
 {
     if (units_visible(tmpside, x, y)) {
     	return (unit_at(x, y) != NULL && !trusted_side(tmpside, unit_at(x, y)->side));
@@ -405,11 +419,9 @@ int x, y;
 /* Test a goal to see if it is true for side, as specified. */
 
 int
-goal_truth(side, goal)
-Side *side;
-Goal *goal;
+goal_truth(Side *side, Goal *goal)
 {
-    int x, y;
+    int x, y, m, n;
     Side *side2 = NULL;
 
     if (goal == NULL) return 0;
@@ -457,7 +469,12 @@ Goal *goal;
       case GOAL_HAS_UNIT_TYPE_NEAR:
 	return 0;
       case GOAL_HAS_MATERIAL_TYPE:
-	return 0;
+	m = goal->args[0];
+	n = goal->args[1];
+	if (side->treasury[m] >= n)
+	  return 100;
+	else
+	  return -100;
       default:
 	case_panic("goal type", goal->type);
 	return 0;
@@ -470,8 +487,7 @@ Goal *goal;
 char *goalbuf = NULL;
 
 char *
-goal_desig(goal)
-Goal *goal;
+goal_desig(Goal *goal)
 {
     int numargs, i, arg;
     char *argtypes;
@@ -816,7 +832,7 @@ colonizing_worth(int u)
       return 0;
     for_all_unit_types(u2) {
 	if (could_create(u, u2)
-	    && u_advanced(u2)) {
+	    && (u_advanced(u2) || could_create(u2, u))) {
 	    /* It is more valuable if it can build bigger cities. */
 	    colvalue += u_reach(u2) + 1;
 	}
