@@ -8,7 +8,8 @@ any later version.  See the file COPYING.  */
 
 typedef enum {
     nobrains = 0,
-    mplayertype = 1
+    mplayertype = 1,
+    iplayertype =2
     /* plus other types as defined */
 #ifdef MPW_C
     /* Works around an MPW C bug where byte-sized enums lose - symptom
