@@ -1,5 +1,5 @@
 /* The combat-related actions of Xconq.
-   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@ static void fire_on_unit(Unit *atker, Unit *other);
 static void attack_unit(Unit *atker, Unit *other);
 static void maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff);
 static void hit_unit(Unit *unit, int hit, Unit *atker);
+static void model_1_attack(Unit *unit, int x, int y);
 
 static void reckon_damage(int fire);
 static void reckon_damage_here(int x, int y);
@@ -56,9 +57,7 @@ int numsoundplays;
 /* This is an attack on a given unit at a given level of commitment. */
 
 int
-prep_attack_action(unit, unit2, defender, n)
-Unit *unit, *unit2, *defender;
-int n;
+prep_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
@@ -70,9 +69,7 @@ int n;
 }
 
 int
-net_prep_attack_action(unit, unit2, defender, n)
-Unit *unit, *unit2, *defender;
-int n;
+net_prep_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
 {
     int rslt;
 
@@ -82,9 +79,7 @@ int n;
 }
 
 int
-do_attack_action(unit, unit2, defender, n)
-Unit *unit, *unit2, *defender;
-int n;
+do_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
 {
     int u2 = unit2->type, u3 = defender->type;
     int withdrawchance, surrenderchance;
@@ -125,12 +120,13 @@ int n;
 }
 
 int
-check_attack_action(unit, unit2, defender, n)
-Unit *unit, *unit2, *defender;
-int n;
+check_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
 {
     int u, u2, u3, acp, u2x, u2y, dfx, dfy, dist, m;
 
+    /* In combat model 1, we can't attack units directly. */
+    if (g_combat_model() == 1)
+      return A_ANY_ERROR;
     if (!in_play(unit))
       return A_ANY_ERROR;
     if (!in_play(unit2))
@@ -140,7 +136,7 @@ int n;
     /* We can't attack ourselves. */
     if (unit2 == defender)
       return A_ANY_ERROR;
-    if (unit2->side != NULL && unit2->side == defender->side)
+    if (!indep(unit2) && unit2->side == defender->side)
       return A_ANY_ERROR;
     u = unit->type;
     u2 = unit2->type;
@@ -214,78 +210,89 @@ do_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
     action_point(unit2->side, x, y);
     acpused = 0;
-    /* Attack every defender in turn. */
-    for_all_stack(x, y, defender) {
-	u3 = defender->type;
-	/* Don't attack any of our buddies. */
-	if (unit_trusts_unit(unit2, defender))
-	  continue;
-	action_point(defender->side, x, y);
-	gotonext = FALSE;
-	/* Defender might be a type that can sneak away to avoid attack. */
-	withdrawchance = uu_withdraw_per_attack(u2, u3);
-	if (withdrawchance > 0) {
-	    if (probability(withdrawchance)) {
-		if (retreat_unit(defender, unit2)) {
+    switch(g_combat_model()) {
+      case 0:
+	/* Attack every defender in turn. */
+	for_all_stack(x, y, defender) {
+	    u3 = defender->type;
+	    /* Don't attack any of our buddies. */
+	    if (unit_trusts_unit(unit2, defender))
+	      continue;
+	    action_point(defender->side, x, y);
+	    gotonext = FALSE;
+	    /* Defender might be a type that can sneak away to avoid
+               attack. */
+	    withdrawchance = uu_withdraw_per_attack(u2, u3);
+	    if (withdrawchance > 0) {
+		if (probability(withdrawchance)) {
+		    if (retreat_unit(defender, unit2)) {
+			gotonext = TRUE;
+		    }
+		}
+	    }
+	    /* Defender might instead choose to surrender right off. */
+	    surrenderchance = uu_surrender_per_attack(u2, u3);
+	    if (surrenderchance > 0
+		&& unit2->side
+		&& unit2->side->tech[u3] >= u_tech_to_own(u3)) {
+		if (probability(surrenderchance)) {
+		    capture_unit(unit2, defender, H_UNIT_CAPTURED);
 		    gotonext = TRUE;
 		}
 	    }
-	}
-	/* Defender might instead choose to surrender right off. */
-	surrenderchance = uu_surrender_per_attack(u2, u3);
-	if (surrenderchance > 0
-	    && unit2->side
-	    && unit2->side->tech[u3] >= u_tech_to_own(u3)) {
-	    if (probability(surrenderchance)) {
-		capture_unit(unit2, defender, H_UNIT_CAPTURED);
-		gotonext = TRUE;
-	    }
-	}
-	/* (should automatically prefer direct attack if better odds) */
-	if (can_fire(unit)) {
-	    do_fire_at_action(unit, unit, defender, -1); 
-	    acpused += u_acp_to_fire(u2);
-	    /* Unlike the case below, target acp consumption has
-	       already been dealt with in fire_on_unit, which is
-	       called by do_fire_at_action. */
-	} else if (can_attack(unit)) {
-	    if (!gotonext)
-	      one_attack(unit2, defender);
-	    if (alive(unit))
-	      use_up_acp(unit, uu_acp_to_attack(u2, u3));
-	    acpused += uu_acp_to_attack(u2, u3);
-	    /* The target of an attack has to take time to defend
+	    /* (should automatically prefer direct attack if better odds) */
+	    if (can_fire(unit)) {
+		do_fire_at_action(unit, unit, defender, -1); 
+		acpused += u_acp_to_fire(u2);
+		/* Unlike the case below, target acp consumption has
+		   already been dealt with in fire_on_unit, which is
+		   called by do_fire_at_action. */
+	    } else if (can_attack(unit)) {
+		if (!gotonext)
+		  one_attack(unit2, defender);
+		if (alive(unit))
+		  use_up_acp(unit, uu_acp_to_attack(u2, u3));
+		acpused += uu_acp_to_attack(u2, u3);
+		/* The target of an attack has to take time to defend
 	       itself. */
-	    if (!gotonext && alive(defender)) {
-		use_up_acp(defender, uu_acp_to_defend(u, u3));
+		if (!gotonext && alive(defender)) {
+		    use_up_acp(defender, uu_acp_to_defend(u, u3));
+		}
 	    }
 	}
-    }
-    if (!alive(unit2))
-      return A_OVERRUN_FAILED;
-    if (in_blocking_zoc(unit2, x, y, z))
-      return A_OVERRUN_FAILED;
-    /* Try to enter the cleared cell now - might still have
-       friendlies filling it up already, so check first. */
-    if (can_occupy_cell(unit2, x, y)) {
-	mpcost = move_unit(unit2, x, y);
-	/* Note that we'll say the action succeeded even if
-	   the cell did not have enough room for us to actually
-	   be in it, which is a little weird. */
-	/* Now add up any extra movement costs of entering the new cell. */
-	mpcost += zoc_move_cost(unit2, ox, oy, oz);
-	acpcost = 0;
-	speed = unit_speed(unit2, x, y);
-        if (speed > 0)
-	  acpcost = (mpcost * 100) / speed;
-	acpcost -= acpused;
-	if (acpcost > 0) {
-	    /* Take the movement cost out of the moving unit if possible. */
-	    use_up_acp(unit2, acpcost);
+	if (!alive(unit2))
+	  return A_OVERRUN_FAILED;
+	if (in_blocking_zoc(unit2, x, y, z))
+	  return A_OVERRUN_FAILED;
+	/* Try to enter the cleared cell now - might still have
+	   friendlies filling it up already, so check first. */
+	if (can_occupy_cell(unit2, x, y)) {
+	    mpcost = move_unit(unit2, x, y);
+	    /* Note that we'll say the action succeeded even if
+	       the cell did not have enough room for us to actually
+	       be in it, which is a little weird. */
+	    /* Now add up any extra movement costs of entering the new cell. */
+	    mpcost += zoc_move_cost(unit2, ox, oy, oz);
+	    acpcost = 0;
+	    speed = unit_speed(unit2, x, y);
+	    if (speed > 0)
+	      acpcost = (mpcost * 100) / speed;
+	    acpcost -= acpused;
+	    if (acpcost > 0) {
+		/* Take the movement cost out of the moving unit if
+                   possible. */
+		use_up_acp(unit2, acpcost);
+	    }
+	    /* Count the unit as having actually moved. */
+	    if (unit2->act)
+	      ++(unit2->act->actualmoves);
 	}
-	/* Count the unit as having actually moved. */
-	if (unit2->act)
-	  ++(unit2->act->actualmoves);
+      break;
+      case 1:
+	model_1_attack(unit2, x, y);
+	break;
+      default:
+	break;
     }
     return A_OVERRUN_SUCCEEDED;
 }
@@ -333,14 +340,23 @@ check_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
 	if (unit2->supply[m] < um_to_attack(u2, m))
 	  return A_ANY_NO_MATERIAL;
     }
-    for_all_stack(x, y, defender) {
-	/* If we can't attack the unit, then we can't possibly overrun. */
-	if (uu_acp_to_attack(u2, defender->type) == 0)
-	  return A_ANY_ERROR;
-	/* (should test if units here can be attacked en masse) */
-    	/* (should prorate ammo needs by intensity of overrun) */
-    	if (!enough_ammo(unit2, defender))
-    	  return A_ANY_NO_MATERIAL;
+    switch(g_combat_model()) {
+      case 0:
+	for_all_stack(x, y, defender) {
+	    /* If we can't attack the unit, then we can't possibly overrun. */
+	    if (uu_acp_to_attack(u2, defender->type) == 0)
+	      return A_ANY_ERROR;
+	    /* (should test if units here can be attacked en masse) */
+	    /* (should prorate ammo needs by intensity of overrun) */
+	    if (!enough_ammo(unit2, defender))
+	      return A_ANY_NO_MATERIAL;
+	}
+	break;
+      case 1:
+	/* Assume we can always attempt to overrun */
+	break;
+      default:
+	return A_ANY_ERROR;
     }
     return A_ANY_OK;
 }
@@ -399,97 +415,188 @@ one_attack(Unit *atker, Unit *defender)
     return (alive(atker) && unit_at(ox, oy) == NULL);
 }
 
+static void
+model_1_attack(Unit *unit, int x, int y)
+{
+    int u = unit->type, u2, d, maxdef, def, att, winround, limit, dmg;
+    Unit *unit2, *defender, *occ, *defocc, *victim;
+    char *hitmovietype;
+    Side *side3;
+    SideMask sidemask;    
+
+    /* Non-combat units can't do anything. */
+    if (u_attack(u) == 0)
+      return;
+    /* Choose the defender of a stack. */
+    maxdef = -1;
+    defender = NULL;
+    for_all_stack(x, y, unit2) {
+	/* Don't attack any of our buddies. */
+	if (unit_trusts_unit(unit, unit2))
+	  continue;
+	/* Identify the best defender. */
+	u2 = unit2->type;
+	def = u_defend(u2);
+	if (def > maxdef) {
+	    maxdef = def;
+	    defender = unit2;
+	}
+    }
+    /* If the location is empty for some reason, escape quietly. */
+    if (defender == NULL)
+      return;
+    if (u_advanced(defender->type)) {
+	maxdef = -1;
+	defocc = NULL;
+	for_all_occupants(defender, occ) {
+	    u2 = occ->type;
+	    def = u_defend(u2);
+	    if (def > maxdef) {
+		maxdef = def;
+		defocc = occ;
+	    }
+	}
+	/* Cities with no defenders can be captured directly. */
+	if (defocc == NULL) {
+	    attempt_to_capture_unit(unit, defender);
+	    return;
+	}
+	defender = defocc;
+    }
+    d = defender->type;
+    limit = 100;
+    while (alive(unit) && alive(defender) && limit-- > 0) {
+	att = u_attack(u);
+	def = u_defend(d);
+	winround = probability((att * 100) / (att + def));
+	if (winround) {
+	    dmg = roll_dice(uu_damage(u, d));
+	    victim = defender;
+	    defender->hp2 -= dmg;
+	} else {
+	    dmg = roll_dice(uu_damage(d, u));
+	    victim = unit;
+	    unit->hp2 -= dmg;
+	}
+	if (defender->hp2 <= 0) {
+	    report_combat(unit, defender, "destroy");
+	} else if (unit->hp2 <= 0) {
+	    report_combat(defender, unit, "destroy");
+	}
+	/* Pick the hit movie to show. */
+	hitmovietype = ((victim->hp2 <= 0) ? "death" : "hit-short");
+	/* Make up the list of sides that will see it. */
+	sidemask = NOSIDES;
+	for_all_sides(side3) {
+	  /* Let all sides that can see the attacker's cell see it. */
+	  if (units_visible(side3, victim->x, victim->y))
+	    sidemask = add_side_to_set(side3, sidemask);
+	}
+	/* Show the movie. */
+	for_all_sides(side3) {
+	  if (side_in_set(side3, sidemask))
+	    schedule_movie(side3, hitmovietype, victim->id);
+	}
+	play_movies(ALLSIDES);
+	damage_unit(unit, combat_dmg, defender);
+	damage_unit(defender, combat_dmg, unit);
+    }
+    /* The whole stack dies if its defender dies. */
+    if (!alive(defender) && defender->transport == NULL) {
+	for_all_stack(x, y, unit2) {
+	    /* (should except spies etc?) */
+	    unit2->hp2 = 0;
+	    report_combat(unit, unit2, "destroy");
+	}
+	reckon_damage_here(x, y);
+    }
+}
+
 /* Fire-at action. */
 
 /* Shooting at a given unit. */
 
 int
-prep_fire_at_action(unit, unit2, defender, m)
-Unit *unit, *unit2, *defender;
-int m;
+prep_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
 {
-    if (unit == NULL || unit->act == NULL || unit2 == NULL || defender == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL || unit3 == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_FIRE_AT;
-    unit->act->nextaction.args[0] = defender->id;
+    unit->act->nextaction.args[0] = unit3->id;
     unit->act->nextaction.args[1] = m;
     unit->act->nextaction.actee = unit2->id;
     return TRUE;
 }
 
 int
-net_prep_fire_at_action(unit, unit2, defender, m)
-Unit *unit, *unit2, *defender;
-int m;
+net_prep_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
 {
     int rslt;
 
-    rslt = prep_fire_at_action(unit, unit2, defender, m);
+    rslt = prep_fire_at_action(unit, unit2, unit3, m);
     broadcast_next_action(unit);
     return rslt;
 }
 
 int
-do_fire_at_action(unit, unit2, defender, m)
-Unit *unit, *unit2, *defender;
-int m;
+do_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
 {
     int ux = unit->x, uy = unit->y, ox, oy, oz;
     SideMask sidemask;
     Side *side;
 
-    action_point(unit2->side, defender->x, defender->y);
-    action_point(defender->side, defender->x, defender->y);
+    action_point(unit2->side, unit3->x, unit3->y);
+    action_point(unit3->side, unit3->x, unit3->y);
     /* Make up the list of sides that will see the fire. */
     sidemask = NOSIDES;
     for_all_sides(side) {
     /* Let all sides that can see the attacker's cell see it. */
 	if (units_visible(side, unit2->x, unit2->y))
 		sidemask = add_side_to_set(side, sidemask);
-    /* Let all sides that can see the defender's cell see it. */
-	if (units_visible(side, defender->x, defender->y))
+    /* Let all sides that can see the unit3's cell see it. */
+	if (units_visible(side, unit3->x, unit3->y))
 		sidemask = add_side_to_set(side, sidemask);
     }
     /* Show the fire. */
     for_all_sides(side) {
 	if (side_in_set(side, sidemask))
-		update_fire_at_display(side, unit, defender, m, TRUE);
+		update_fire_at_display(side, unit, unit3, m, TRUE);
     }
-    ox = defender->x;  oy = defender->y;  oz = defender->z;
-    amain = unit;  omain = defender;
-    fire_on_unit(unit, defender);
+    ox = unit3->x;  oy = unit3->y;  oz = unit3->z;
+    amain = unit;  omain = unit3;
+    fire_on_unit(unit, unit3);
     reckon_damage(TRUE);
     if (alive(unit))
       use_up_acp(unit, u_acp_to_fire(unit2->type));
-    /*	if (alive(defender)) use_up_acp(defender, 1); */
+    /*	if (alive(unit3)) use_up_acp(unit3, 1); */
     /* Each side sees what happened to its own unit. */
     update_unit_display(unit2->side, unit2, TRUE);
     if (unit != unit2)
       update_unit_display(unit->side, unit, TRUE);
-    update_unit_display(defender->side, defender, TRUE);
+    update_unit_display(unit3->side, unit3, TRUE);
     /* The attacking side also sees the remote cell. */
     update_cell_display(unit2->side, ox, oy, UPDATE_ALWAYS);
-    update_cell_display(defender->side, ox, oy, UPDATE_ALWAYS);
+    update_cell_display(unit3->side, ox, oy, UPDATE_ALWAYS);
     /* Victim might see something in attacker's cell. */
-    update_cell_display(defender->side, ux, uy, UPDATE_ALWAYS);
+    update_cell_display(unit3->side, ux, uy, UPDATE_ALWAYS);
     /* Actually, everybody might be seeing the combat. */
     all_see_cell(ux, uy);
     all_see_cell(ox, oy);
     /* Permit attempts to capture after fire, but only from adjacent
        or same cell. */
-    if (distance(unit2->x, unit2->y, defender->x, defender->y) < 2) {
-	Side *os = defender->side;
+    if (distance(unit2->x, unit2->y, unit3->x, unit3->y) < 2) {
+	Side *os = unit3->side;
 
-	attempt_to_capture_unit(unit2, defender);
-	/* If the defender was not captured, it might turn the tables! */
-	/* (Note that we cannot cache the attacker's and defender's
+	attempt_to_capture_unit(unit2, unit3);
+	/* If the unit3 was not captured, it might turn the tables! */
+	/* (Note that we cannot cache the attacker's and unit3's
 	   types, because the type might have changed due to damage,
 	   and we want to know if the new type might countercapture.)  */
-	if (alive(defender)
+	if (alive(unit3)
 	    && alive(unit2)
-	    && defender->side == os
-	    && uu_countercapture(unit2->type, defender->type) > 0) {
-	    attempt_to_capture_unit(defender, unit2);
+	    && unit3->side == os
+	    && uu_countercapture(unit2->type, unit3->type) > 0) {
+	    attempt_to_capture_unit(unit3, unit2);
 	}
     }
     /* Always expend the ammo (but only if m is a valid mtype). */
@@ -499,12 +606,13 @@ int m;
 /* Test a fire action for plausibility. */
 
 int
-check_fire_at_action(unit, unit2, unit3, m)
-Unit *unit, *unit2, *unit3;
-int m;
+check_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
 {
     int u, u2, u3, ux, uy, uz, acp, dist, m2;
 
+    /* In combat model 1, we can't attack units directly. */
+    if (g_combat_model() == 1)
+      return A_ANY_ERROR;
     if (!in_play(unit))
       return A_ANY_ERROR;
     if (!in_play(unit2))
@@ -530,7 +638,7 @@ int m;
       return A_ANY_TOO_FAR;
     if (dist < u_range_min(u2))
       return A_ANY_TOO_NEAR;
-    if (unit2->side != NULL && unit2->side == unit3->side)
+    if (!indep(unit2) && unit2->side == unit3->side)
       return A_ANY_ERROR;
     /* Gunners won't shoot at anything they know they can't hit. */
     if ((uu_fire_hit(u2, u3) != -1 ? uu_fire_hit(u2, u3) : uu_hit(u2, u3)) == 0)
@@ -1640,9 +1748,7 @@ Side *newside;
 /* Do additional consequences of capture. */
 
 static void
-capture_unit_2(unit, pris, prevside)
-Unit *unit, *pris;
-Side *prevside;
+capture_unit_2(Unit *unit, Unit *pris, Side *prevside)
 {
     int chance, x, y, notesee;
     Unit *occ, *unit2;
@@ -1652,7 +1758,8 @@ Side *prevside;
        capture really means (change of crew perhaps). */
     pris->cxp = (pris->cxp * u_cxp_on_capture(pris->type)) / 100;
     pris->cxp = min(unit->cxp, u_cxp_max(pris->type));
-    /* Getting captured is always bad for morale, but getting liberated is good. */
+    /* Getting captured is always bad for morale, but getting
+       liberated is good. */
     if (pris->side == pris->origside)
       change_morale(pris, 1, ((pris->morale + 1) * 50) + 50);
     else
@@ -1665,10 +1772,10 @@ Side *prevside;
     /* (should probably adjust side's acp sums by new unit's amounts) */
     init_unit_plan(pris);
     /* Maybe get a pile of interesting information. */
+    /* Independent units won't know anything about other independents
+       though (they're independent, eh?). */
     newside = unit->side;
-    if (newside
-	&& !newside->see_all
-	&& prevside != NULL) {
+    if (newside && !newside->see_all && prevside != indepside) {
 	notesee = FALSE;
 	chance = u_see_terrain_captured(pris->type);
 	if (!g_terrain_seen() && probability(chance)) {
@@ -1743,8 +1850,8 @@ do_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
     /* Note that if the maxrange is further than the actual range of this
        detonation, only just-damaged units will be looked at. */
     reckon_damage_around(x, y, max_u_detonate_effect_range);
-    /* Unit might have detonated outside its range of effect, so need this to make
-       its own damage is accounted for. */
+    /* Unit might have detonated outside its range of effect, so need
+       this to make its own damage is accounted for. */
     if (alive(unit2))
       reckon_damage_around(unit2->x, unit2->y, 0);
     use_up_acp(unit, u_acp_to_detonate(u2));
@@ -1980,8 +2087,7 @@ damage_terrain(int u, int x, int y)
 }
 
 int
-damaged_terrain_type(t)
-int t;
+damaged_terrain_type(int t)
 {
     int t2, tot, othertot, test, sum, rslt;
 
@@ -2017,18 +2123,23 @@ int t;
     return rslt;
 }
 
-/* Tests for whether given units or types can do particular types of actions. */
+/* Tests for whether given units or types can do particular types of
+   actions. */
 
 int
-can_attack(unit)
-Unit *unit;
+could_hit(int u1, int u2)
+{
+    return (uu_hit(u1, u2) > 0 || uu_fire_hit(u1, u2) > 0);
+}
+
+int
+can_attack(Unit *unit)
 {
     return type_can_attack(unit->type);
 }
 
 int
-type_can_attack(u)
-int u;
+type_can_attack(int u)
 {
     int u2;
 	
@@ -2042,15 +2153,13 @@ int u;
 }
 
 int
-can_fire(unit)
-Unit *unit;
+can_fire(Unit *unit)
 {
     return type_can_fire(unit->type);
 }
 
 int
-type_can_fire(u)
-int u;
+type_can_fire(int u)
 {
     int u2, fhit, fdam;
 	
@@ -2067,8 +2176,7 @@ int u;
 }
 
 int
-type_can_capture(u)
-int u;
+type_can_capture(int u)
 {
     int u2;
 	
@@ -2083,8 +2191,7 @@ int u;
 }
 
 int
-can_detonate(unit)
-Unit *unit;
+can_detonate(Unit *unit)
 {
     return (u_acp_to_detonate(unit->type) > 0);
 }
