@@ -126,9 +126,7 @@ extern void action_point(Side *side, int x, int y);
 
 extern void notify_all(char *str, ...);
 extern void notify(Side *side, char *str, ...);
-#ifdef ANSI_PROTOTYPES
 extern void vnotify(Side *side, char *fmt, va_list ap);
-#endif
 extern void low_notify(Side *side, char *str);
 
 extern void init_warning(char *str, ...);
@@ -147,56 +145,8 @@ extern void close_displays(void);
 
 /* Declarations of functions not elsewhere declared. */
 
-/* init.c */
-
-extern void init_library_path(char *path);
-extern void init_data_structures(void);
-extern void check_game_validity(void);
-extern void calculate_globals(void);
-extern void patch_object_references(void);
-extern void make_trial_assignments(void);
-extern Side *make_up_a_side(void);
-extern void init_side_advantage(Side *side);
-extern int add_side_and_player(void);
-extern int exchange_players(int n, int n2);
-extern int remove_side_and_player(void);
-extern void run_synth_methods(void);
-extern int make_random_date(int calls, int runs);
-extern int make_weather(int calls, int runs);
-extern void final_init(void);
-extern void configure_sides(void);
-extern void init_all_views(void);
-extern int make_initial_materials(int calls, int runs);
-extern void init_supply(Unit *unit);
-extern int saved_game(void);
-extern void init_side_balance(void);
-extern void check_consistency(void);
-extern void assign_players_to_sides(void);
-extern char *version_string(void);
-extern char *copyright_string(void);
-extern char *license_string(void);
-extern int already_seen_chance(Side *side, Unit *unit);
-extern void add_remote_players(void);
-extern void set_player_advantage(int n, int newadv);
-extern void rename_side_for_player(int n);
-extern void set_ai_for_player(int n, char *aitype);
-
-/* read.c */
-
-extern void syntax_error(Obj *x, char *msg);
-extern void type_error(Obj *x, char *msg);
-extern void interp_form(Module *module, Obj *form);
-extern void interp_game_module(Obj *form, Module *module);
-extern void do_module_variants(Module *module, Obj *lis);
-extern void fill_in_side(Side *side, Obj *props, int userdata);
-extern void read_warning(char *str, ...);
-extern int utype_from_name(char *str);
-extern int utype_from_symbol(Obj *sym);
-extern int mtype_from_name(char *str);
-extern int ttype_from_name(char *str);
-extern int atype_from_name(char *str);
-
 /* actions.c */
+
 extern int can_extract_at(Unit *unit, int x, int y, int *mp);
 extern int can_load_at(Unit *unit, int x, int y, int *mp);
 extern void init_actions(void);
@@ -248,6 +198,8 @@ extern int type_can_remove_terrain(int u);
 extern int any_construction_possible(void);
 extern char *action_desig(Action *act);
 
+/* combat.c */
+
 extern int can_attack(Unit *unit);
 extern int type_can_attack(int u);
 extern int can_fire(Unit *unit);
@@ -260,11 +212,61 @@ extern int type_can_carry(int u);
 extern int defended_by_occupants(Unit *unit);
 extern int occ_can_defend_transport(int o, int t);
 extern int enough_to_garrison(Unit *unit, int defenders);
-extern Unit * mobile_enemy_threat(Unit *unit, int range);
+extern Unit *mobile_enemy_threat(Unit *unit, int range);
 extern int found_blocking_elevation(int u, int ux, int uy, int uz, int u2, int u2x, int u2y, int u2z);
 extern int enough_ammo(Unit *unit, Unit *other);
 extern int capture_chance(int u, int u2, Side *side2);
 extern int multiply_dice(int dice, int mult);
+extern int could_hit(int u1, int u2);
+
+/* init.c */
+
+extern void init_library_path(char *path);
+extern void init_data_structures(void);
+extern void check_game_validity(void);
+extern void calculate_globals(void);
+extern void patch_object_references(void);
+extern void make_trial_assignments(void);
+extern Side *make_up_a_side(void);
+extern void init_side_advantage(Side *side);
+extern int add_side_and_player(void);
+extern int exchange_players(int n, int n2);
+extern int remove_side_and_player(void);
+extern void run_synth_methods(void);
+extern int make_random_date(int calls, int runs);
+extern int make_weather(int calls, int runs);
+extern void final_init(void);
+extern void configure_sides(void);
+extern void init_all_views(void);
+extern int make_initial_materials(int calls, int runs);
+extern void init_supply(Unit *unit);
+extern int saved_game(void);
+extern void init_side_balance(void);
+extern void check_consistency(void);
+extern void assign_players_to_sides(void);
+extern char *version_string(void);
+extern char *copyright_string(void);
+extern char *license_string(void);
+extern int already_seen_chance(Side *side, Unit *unit);
+extern void add_remote_players(void);
+extern void set_player_advantage(int n, int newadv);
+extern void rename_side_for_player(int n);
+extern void set_ai_for_player(int n, char *aitype);
+
+/* read.c */
+
+extern void syntax_error(Obj *x, char *msg);
+extern void type_error(Obj *x, char *msg);
+extern void interp_form(Module *module, Obj *form);
+extern void interp_game_module(Obj *form, Module *module);
+extern void do_module_variants(Module *module, Obj *lis);
+extern void fill_in_side(Side *side, Obj *props, int userdata);
+extern void read_warning(char *str, ...);
+extern int utype_from_name(char *str);
+extern int utype_from_symbol(Obj *sym);
+extern int mtype_from_name(char *str);
+extern int ttype_from_name(char *str);
+extern int atype_from_name(char *str);
 
 /* run.c */
 
@@ -351,6 +353,8 @@ extern char *unit_handle(Side *side, Unit *unit);
 extern char *short_unit_handle(Unit *unit);
 extern void name_or_number(Unit *unit, char *buf);
 extern void construction_desc(char *buf, Unit *unit, int u);
+extern void research_desc(char *buf, Unit *unit, int a);
+extern void researchible_desc(char *buf, Unit *unit, int a);
 extern void constructible_desc(char *buf, Side *side, int u, Unit *unit);
 extern int est_completion_time(Unit *unit, int u2);
 extern void historical_event_date_desc(HistEvent *hevt, char *buf);
@@ -439,7 +443,7 @@ extern int visible_to(Unit *unit, Unit *unit2);
 
 extern int construction_possible(int u2);
 extern int can_build_or_help(Unit *unit);
-extern int can_build_or_help(Unit *unit);
+extern int can_research(Unit *unit);
 extern int can_produce(Unit *unit);
 extern void add_to_unit_hp(Unit *unit, int hp);
 extern void try_sharing(Unit *from, Unit *to, int m);
