@@ -700,8 +700,7 @@ int u;
 }
 
 int
-any_mp_to_leave_unit(u)
-int u;
+any_mp_to_leave_unit(int u)
 {
     int u2;
 	
@@ -713,8 +712,7 @@ int u;
 }
 
 int
-any_enter_indep(u)
-int u;
+any_enter_indep(int u)
 {
     int u2;
 	
@@ -730,10 +728,7 @@ int u;
 /* (The defaults should come from the *.def defaults!!) */
 
 static void
-describe_utype(u, key, buf)
-int u;
-char *key;
-TextBuffer *buf;
+describe_utype(int u, char *key, TextBuffer *buf)
 {
     char sidetmpbuf[BUFSIZE];
     int m, first, speedvaries, usesm, a;
@@ -1624,8 +1619,7 @@ TextBuffer *buf;
 }
 
 int
-may_detonate(u)
-int u;
+may_detonate(int u)
 {
     return ((u_acp(u) > 0 && u_acp_to_detonate(u) > 0)
 	    || u_detonate_on_death(u) > 0
@@ -1637,10 +1631,7 @@ int u;
 }
 
 static void
-describe_mtype(m, key, buf)
-int m;
-char *key;
-TextBuffer *buf;
+describe_mtype(int m, char *key, TextBuffer *buf)
 {
     append_help_phrase(buf, m_help(m));
     if (m_people(m) > 0) {
@@ -1657,13 +1648,12 @@ TextBuffer *buf;
 	tbcat(buf, "\nFOR DESIGNERS:\n");
     }
 #endif /* DESIGNERS */
+    /* Add an extra space, to make sure the description isn't empty. */
+    tbcat(buf, " ");
 }
 
 static void
-describe_ttype(t, key, buf)
-int t;
-char *key;
-TextBuffer *buf;
+describe_ttype(int t, char *key, TextBuffer *buf)
 {
     int m, ct;
 
@@ -1683,13 +1673,10 @@ TextBuffer *buf;
     }
     if (t_liquid(t))
       tbcat(buf, "Represents water or other liquid.\n");
-    switch (t_subtype_x(t)) {
-      case 10: /* K_RIVER_X */
+    if (t_subtype_x(t) == c_number(symbol_value(intern_symbol(keyword_name(K_RIVER_X))))) {
 	tbcat(buf, "Considered to be a type of river.\n");
-	break;
-      case 12: /* K_ROAD_X */
+    } else if (t_subtype_x(t) == c_number(symbol_value(intern_symbol(keyword_name(K_ROAD_X))))) {
 	tbcat(buf, "Considered to be a type of road.\n");
-	break;
     }
     tbprintf(buf, "Generic unit capacity is %d.\n", t_capacity(t));
     if (t_people_max(t) >= 0) {
@@ -1711,10 +1698,10 @@ TextBuffer *buf;
     if (any_temp_variation) {
 	if (t_temp_min(t) == t_temp_max(t)) {
 	    tbprintf(buf, "Temperature is always %d.\n",
-		    t_temp_min(t));
+		     t_temp_min(t));
 	} else {
 	    tbprintf(buf, "Temperatures fall between %d and %d, averaging %d.\n",
-		    t_temp_min(t), t_temp_max(t), t_temp_avg(t));
+		     t_temp_min(t), t_temp_max(t), t_temp_avg(t));
 	}
 	if (t_temp_variability(t) > 0) {
 	    tbprintf(buf, "Temperature varies randomly, up to %d each turn.\n",
@@ -1724,43 +1711,48 @@ TextBuffer *buf;
     if (any_wind_variation) {
 	if (t_wind_force_min(t) == t_wind_force_max(t)) {
 	    tbprintf(buf, "Wind force is always %d.\n",
-		    t_wind_force_min(t));
+		     t_wind_force_min(t));
 	} else {
 	    tbprintf(buf, "Wind forces fall between %d and %d, averaging %d.\n",
-		    t_wind_force_min(t), t_wind_force_max(t), t_wind_force_avg(t));
+		     t_wind_force_min(t), t_wind_force_max(t), t_wind_force_avg(t));
 	}
 	if (t_wind_force_variability(t) > 0) {
 	    tbprintf(buf, "%d%% chance each turn that wind force will change.\n",
-		    t_wind_force_variability(t));
+		     t_wind_force_variability(t));
 	}
 	if (t_wind_variability(t) > 0) {
 	    tbprintf(buf, "%d%% chance each turn that wind direction will change.\n",
-		    t_wind_variability(t));
+		     t_wind_variability(t));
 	}
     }
     if (any_clouds) {
 	if (t_clouds_min(t) == t_clouds_max(t)) {
 	    tbprintf(buf, "Cloud cover is always %d.\n",
-		    t_clouds_min(t));
+		     t_clouds_min(t));
 	} else {
 	    tbprintf(buf, "Cloud cover falls between %d and %d\n",
-		    t_clouds_min(t), t_clouds_max(t));
+		     t_clouds_min(t), t_clouds_max(t));
 	}
     }
     /* Display relationships with materials. */
     if (nummtypes > 0) {
 	for_all_material_types(m) {
+	    if (tm_storage_x(t, m) > 0
+		|| tm_production(t, m) > 0
+		|| tm_consumption(t, m) > 0) {
+		tbprintf(buf, "%s:", m_type_name(m));
+	    }
 	    if (tm_storage_x(t, m) > 0) {
-	    	tbprintf(buf, "Can store up to %d %s", tm_storage_x(t, m), m_type_name(m));
+	    	tbprintf(buf, "  Can store up to %d", tm_storage_x(t, m));
 	    	tbprintf(buf, " (normally starts game with %d)",
-	    		min(tm_initial(t, m), tm_storage_x(t, m)));
+			 min(tm_initial(t, m), tm_storage_x(t, m)));
 	    	tbcat(buf, ".\n");
-	    	tbprintf(buf, "Sides will%s always know current amount accurately.\n",
+	    	tbprintf(buf, "  Sides will%s always know current amount accurately.\n",
 			 (tm_see_always(t, m) ? "" : " not"));
 	    }
 	    if (tm_production(t, m) > 0 || tm_consumption(t, m) > 0) {
-		tbprintf(buf, " Produces %d and consumes %d each turn.\n",
-			tm_production(t, m), tm_consumption(t, m));
+		tbprintf(buf, "  Produces %d and consumes %d each turn.\n",
+			 tm_production(t, m), tm_consumption(t, m));
 	    }
 	}
     }
@@ -1770,7 +1762,8 @@ TextBuffer *buf;
     	for_all_terrain_types(ct) {
 	    if (t_is_coating(ct)) {
 		tbprintf(buf, "%s coats, depths %d up to %d",
-			t_type_name(ct), tt_coat_min(ct, t), tt_coat_max(ct, t));
+			 t_type_name(ct),
+			 tt_coat_min(ct, t), tt_coat_max(ct, t));
 	    }
     	}
     }
@@ -1810,10 +1803,7 @@ TextBuffer *buf;
 }
 
 static void
-describe_atype(a, key, buf)
-int a;
-char *key;
-TextBuffer *buf;
+describe_atype(int a, char *key, TextBuffer *buf)
 {
     append_help_phrase(buf, a_help(a));
     tbprintf(buf, "Research points (RP) to discover: %d.\n", a_rp(a));
@@ -1845,10 +1835,7 @@ TextBuffer *buf;
 }
 
 static void
-describe_scorekeepers(arg, key, buf)
-int arg;
-char *key;
-TextBuffer *buf;
+describe_scorekeepers(int arg, char *key, TextBuffer *buf)
 {
     int i = 1;
     Scorekeeper *sk;
@@ -1879,10 +1866,7 @@ TextBuffer *buf;
 /* List each synthesis method and its parameters. */
 
 static void
-describe_setup(arg, key, buf)
-int arg;
-char *key;
-TextBuffer *buf;
+describe_setup(int arg, char *key, TextBuffer *buf)
 {
     int u, t, t2, methkey;
     Obj *synthlist, *methods, *method;
@@ -2190,9 +2174,7 @@ TextBuffer *buf;
 }
 
 static void
-describe_synth_run(buf, methkey)
-TextBuffer *buf;
-int methkey;
+describe_synth_run(TextBuffer *buf, int methkey)
 {
     int calls, runs;
   
@@ -2215,10 +2197,7 @@ int methkey;
 }
 
 static void
-describe_world(arg, key, buf)
-int arg;
-char *key;
-TextBuffer *buf;
+describe_world(int arg, char *key, TextBuffer *buf)
 {
     tbprintf(buf, "World circumference: %d.\n", world.circumference);
     tbcat(buf, "\n");
@@ -2279,10 +2258,7 @@ TextBuffer *buf;
    interfaces can use. */
 
 void
-describe_command (ch, name, help, onechar, buf)
-int ch, onechar;
-char *name, *help;
-TextBuffer *buf;
+describe_command (int ch, char *name, char *help, int onechar, TextBuffer *buf)
 {
     if (onechar && ch != '\0') {
 	if (ch < ' ' || ch > '~') { 
@@ -2304,9 +2280,7 @@ TextBuffer *buf;
 }
 
 static int
-u_property_not_default(fn, dflt)
-int (*fn)(int i);
-int dflt;
+u_property_not_default(int (*fn)(int i), int dflt)
 {
     int u, val;
 
@@ -2319,9 +2293,7 @@ int dflt;
 }
 
 static int
-t_property_not_default(fn, dflt)
-int (*fn)(int i);
-int dflt;
+t_property_not_default(int (*fn)(int i), int dflt)
 {
     int t, val;
 
@@ -2463,8 +2435,7 @@ struct histo {
    (most common values first). */
 
 static int
-histogram_compare(h1, h2)
-CONST void *h1, *h2;
+histogram_compare(CONST void *h1, CONST void *h2)
 {
     if (((struct histo *) h2)->num != ((struct histo *) h1)->num) {
     	return ((struct histo *) h2)->num - ((struct histo *) h1)->num;
