@@ -52,7 +52,7 @@ static Image *add_magnified_image(ImageFamily *imf);
 static int bitmaps_match(int w, int h, Obj *lispdata, char *rawdata);
 static int color_matches_mono(Image *img);
 static void write_pixmap(FILE *fp, int w, int h, int aw, int ah,
-				 int pixelsize,
+				 int pixelsize, int orig_pixelsize,
 				 Obj *palette, int *rawpalette, int numcolors,
 				 Obj *lispdata, char *rawdata);
 static void write_bitmap(FILE *fp, char *subtyp, int w, int h,
@@ -1359,9 +1359,7 @@ write_imf_dir(char *filename, ImageFamily **images, int num)
 /* Write out the entire image family. */
 
 void
-write_imf(fp, imf)
-FILE *fp;
-ImageFamily *imf;
+write_imf(FILE *fp, ImageFamily *imf)
 {
     Obj *palent;
     Image *img;
@@ -1455,7 +1453,7 @@ ImageFamily *imf;
 			&& !color_matches_mono(img)) {
 		fprintf(fp, "\n  ");
 		write_pixmap(fp, img->w, img->h, img->actualw, img->actualh,
-			     img->pixelsize,
+			     img->pixelsize, img->orig_pixelsize,
 			     img->palette, img->rawpalette, img->numcolors,
 			     img->colrdata, img->rawcolrdata);
 	    }
@@ -1585,12 +1583,10 @@ color_matches_mono(Image *img)
 }
 
 static void
-write_pixmap(fp, w, h, actualw, actualh, pixelsize,
-	     palette, rawpalette, numcolors, lispdata, rawdata)
-FILE *fp;
-int w, h, actualw, actualh, pixelsize, *rawpalette, numcolors;
-Obj *palette, *lispdata;
-char *rawdata;
+write_pixmap(FILE *fp, int w, int h, int actualw, int actualh,
+	     int pixelsize, int orig_pixelsize,
+	     Obj *palette, int *rawpalette, int numcolors,
+	     Obj *lispdata, char *rawdata)
 {
     int dolisp, i, j = 0, rowbytes, numbytes, byte;
     char *datastr = NULL;
@@ -1603,7 +1599,9 @@ char *rawdata;
     fprintf(fp, "(%s", keyword_name(K_COLOR));
     if (actualw != w || actualh != h)
       fprintf(fp, " (%s %d %d)", keyword_name(K_ACTUAL), actualw, actualh);
-    fprintf(fp, " (%s %d)", keyword_name(K_PIXEL_SIZE), pixelsize);
+    /* (should not use orig_pixelsize if !dolisp?) */
+    fprintf(fp, " (%s %d)", keyword_name(K_PIXEL_SIZE),
+	    (orig_pixelsize ? orig_pixelsize : pixelsize));
     if (palette != lispnil || (rawpalette && numcolors))
       write_palette_contents(fp, palette, rawpalette, numcolors);
     fprintf(fp, "\n   \"");
@@ -1633,12 +1631,8 @@ char *rawdata;
 }
 
 static void
-write_bitmap(fp, subtyp, w, h, lispdata, rawdata)
-FILE *fp;
-char *subtyp;
-int w, h;
-Obj *lispdata;
-char *rawdata;
+write_bitmap(FILE *fp, char *subtyp, int w, int h, Obj *lispdata,
+	     char *rawdata)
 {
     int dolisp, i, j = 0, rowbytes, numbytes, byte;
     char *datastr = NULL;
@@ -1678,10 +1672,7 @@ char *rawdata;
 }
 
 static void
-write_palette_contents(fp, palette, rawpalette, numcolors)
-FILE *fp;
-Obj *palette;
-int *rawpalette, numcolors;
+write_palette_contents(FILE *fp, Obj *palette, int *rawpalette, int numcolors)
 {
     int len, i, col, red, grn, blu;
     Obj *restpal;
@@ -1713,9 +1704,7 @@ int *rawpalette, numcolors;
 }
 
 static void
-write_color(fp, n, r, g, b)
-FILE *fp;
-int n, r, g, b;
+write_color(FILE *fp, int n, int r, int g, int b)
 {
     char *colorname;
 
@@ -1735,9 +1724,7 @@ int n, r, g, b;
 }
 
 void
-write_imp(fp, imp)
-FILE *fp;
-ImagePalette *imp;
+write_imp(FILE *fp, ImagePalette *imp)
 {
     if (imp == NULL)
       return;
@@ -1819,6 +1806,15 @@ int *col, *red, *grn, *blu;
 	*red = c_number(car(colorcomp));
 	*grn = c_number(cadr(colorcomp));
 	*blu = c_number(caddr(colorcomp));
+	/* Assume small values are 8-bit rather than 16-bit colors.
+	   (Works because as 16-bit colors, they would all be nearly
+	   identical shades of black.) */
+	if (*red < 256)
+	  *red *= 256;
+	if (*grn < 256)
+	  *grn *= 256;
+	if (*blu < 256)
+	  *blu *= 256;
     } else {
 	init_warning("palette color info is not a name or set of numbers, ignoring");
     }
