@@ -17,14 +17,9 @@ any later version.  See the file COPYING.  */
    preventing fatal conflicts. */
 
 #include "conq.h"
-extern void add_remote_players(void);
-extern void set_player_advantage(int n, int newadv);
-extern void rename_side_for_player(int n);
-extern void set_ai_for_player(int n, char *aitype);
 #include "kernel.h"
 #include "ai.h"
 #include "version.h"
-extern int gameinited;
 
 static void cache_possible_types(void);
 static void kill_excess_units(void);
@@ -606,14 +601,15 @@ patch_object_references(void)
 /* (when does locking get done?) */
 
 void
-make_trial_assignments()
+make_trial_assignments(void)
 {
     int i = 0;
     Side *side;
     Player *player;
 
     Dprintf("Making trial side/player assignments\n");
-    /* Fill in the side's predefined default and range of initial advantage. */
+    /* Fill in the sides' predefined default and range of initial
+       advantage. */
     for_all_sides(side) {
     	init_side_advantage(side);
     }
@@ -621,27 +617,35 @@ make_trial_assignments()
     while (numsides < g_sides_min()) {
 	make_up_a_side();
     }
-    while (numsides < min(numplayers, g_sides_max())) {
+    /* Note: numplayers - 1 since the indepside now has a player. */
+    while (numsides < min(numplayers - 1, g_sides_max())) {
 	make_up_a_side();
     }
     /* Put in all the sides. */
     for_all_sides(side) {
 	assignments[i++].side = side;
     }
-    if (numremotes > 0)
-      add_remote_players();
-    /* If no players have been created, make one that is human-run,
-       presumably by the person who started up this program. */
+    /* First create a player to handle indepside if necessary. */
     if (numplayers == 0) {
+	add_player();
+    }
+    /* Add remote players. */
+    if (numremotes > 0) {
+    	add_remote_players();
+    }
+    /* If only one player (the indepside player) has been created so
+       far, make one that is human-run, presumably by the person who
+       started up this program. */
+    if (numplayers == 1) {
 	add_default_player();
     }
-    if (numsides < numplayers) {
+    if (numtotsides < numplayers) {
 	/* We have too many players. */
 	init_warning("too many players (%d) for %d sides, ignoring extra",
-		     numplayers, numsides);
+		     numplayers, numtotsides);
     }
     /* Make any prespecified assignments. */
-    for (i = 0; i < numsides; ++i) {
+    for (i = 0; i < numtotsides; ++i) {
 	if (assignments[i].side != NULL
 	    && assignments[i].side->player != NULL
 	    && assignments[i].player == NULL) {
@@ -651,7 +655,7 @@ make_trial_assignments()
     }
     /* Assign any remaining players. */
     player = playerlist;
-    for (i = 0; i < numsides; ++i) {
+    for (i = 0; i < numtotsides; ++i) {
 	if (assignments[i].side != NULL
 	    && assignments[i].side->ingame
 	    && assignments[i].player == NULL) {
@@ -666,7 +670,7 @@ make_trial_assignments()
     /* Add default players for sides with none (in separate loop so player
        creation doesn't confuse player list traversal above).  Don't
        add for any sides not actually participating in the game. */
-    for (i = 0; i < numsides; ++i) {
+    for (i = 0; i < numtotsides; ++i) {
 	if (assignments[i].side != NULL
 	    && assignments[i].side->ingame
 	    && assignments[i].player == NULL) {
@@ -677,6 +681,12 @@ make_trial_assignments()
 	    trial_assign_player(i, player);
 	}
     }
+    /* Finally set the indepside AI type correctly. */
+    if (g_indepside_has_ai()) {
+	indepside->player->aitypename = "iplayer";
+    } else {
+	indepside->player->aitypename = NULL;
+    }
     /* This warning can happen if there are more players than active
        sides. */
     for_all_players(player) {
@@ -793,7 +803,7 @@ add_side_and_player(void)
     side = make_up_a_side();
     if (side == NULL)
       return -1;
-    n = numsides - 1;
+    n = numtotsides - 1;
     assignments[n].side = side;
     player = add_player();
     assignments[n].player = player;
@@ -850,15 +860,18 @@ exchange_players(int n, int n2)
     if (n < 0)
       n = 0;
     if (n2 < 0) {
-	for (i = n + 1; i <= numsides + n; ++i) {
-	    n2 = i % numsides;
+	for (i = n + 1; i <= numtotsides + n; ++i) {
+	    n2 = i % numtotsides;
 	    if (assignments[n2].side && (assignments[n2].side)->ingame)
 	      break;
 	}
 	/* No sides to exchange with, return. */
-    	if (i == numsides + n)
+    	if (i == numtotsides + n)
 	  return -1;
     }
+    /* Don't switch with the indepside player. */
+    if (n2 == 0)
+    	n2 = 1;
     assignments[n].player = assignments[n2].player;
     assignments[n2].player = tmpplayer;
     /* Doesn't seem like these should be needed, but they are. */
@@ -1118,28 +1131,28 @@ configure_sides()
 void
 init_all_views()
 {
-    int x, y, i = 0, todo = max(1, numsides * area.numcells);
+    int x, y, i = 0, todo = max(1, numtotsides * area.numcells);
     int terrainset;
     Side *side;
 
-    /* Snapshot the value here, so is permanently on. */
-    all_see_all = g_see_all();
     /* Set up the basic view structures for all sides first. */
     for_all_sides(side) {
-	if (all_see_all) {
+	if (g_see_all()) {
 	    side->see_all = TRUE;
 	} else {
 	    side->see_all = FALSE;
 	    terrainset = init_view(side);
 	    init_view_2(side, terrainset);
 	}
-	/* We're not normally allowed to choose whether to see all or not. */
-	side->may_set_see_all = FALSE;
+	/* We're not normally allowed to choose whether to display all
+           or not. */
+	side->may_set_show_all = FALSE;
+	side->show_all = side->see_all;
 	/* Can't think of any other place to put this... */
 	calc_start_xy(side);
     }
-    /* Nothing more to do here. */
-    if (all_see_all)
+    /* Possibly nothing more to do here. */
+    if (g_see_all())
       return;
     announce_lengthy_process("Computing current view at each location");
     /* (coverage is also done at this point) */
@@ -1270,7 +1283,7 @@ Side *side;
 {
     int dir, x1, y1;
 
-    if (!inside_area(x, y) || side == NULL)
+    if (!inside_area(x, y))
       return FALSE;
     for_all_directions(dir) {
 	if (point_in_dir(x, y, dir, &x1, &y1)) {
@@ -1347,7 +1360,7 @@ init_ranged_views()
 			  side->finalradius - 2, side->finalradius + 2,
 			  maybe_init_view_cell);
 	}
-	announce_progress((100 * i++) / numsides);
+	announce_progress((100 * i++) / numtotsides);
     }
     finish_lengthy_process();
 }
@@ -1454,11 +1467,11 @@ check_consistency()
 	}
 	add_edge_terrain();
     }
-    if (numsides <= 0) {
+    if (numtotsides <= 0) {
 	init_error("There are no player sides at all in this game");
-    } else if (numsides < numplayers) {
+    } else if (numtotsides < numplayers) {
 	init_warning("Only made %d of the %d sides requested",
-		     numsides, numplayers);
+		     numtotsides, numplayers);
     }
     /* Need any kind of unit checks? */
 }
@@ -1501,8 +1514,8 @@ assign_players_to_sides()
     Player *player;
 
     announce_lengthy_process("Assigning players to sides");
-    for (i = 0; i < numsides; ++i) {
-	announce_progress((100 * i) / numsides);
+    for (i = 0; i < numtotsides; ++i) {
+	announce_progress((100 * i) / numtotsides);
 	side = assignments[i].side;
 	player = assignments[i].player;
 	if (player == NULL)
@@ -1565,7 +1578,7 @@ assign_players_to_sides()
 #endif /* DESIGNERS */
     if (numais > 0) {
 	announce_lengthy_process("Setting up AIs");
-	for (i = 0; i < numsides; ++i) {
+	for (i = 0; i < numtotsides; ++i) {
 	    if (numais > 1)
 	      announce_progress((100 * n++) / numais);
 	    side = assignments[i].side;
