@@ -45,15 +45,8 @@ typedef struct a_strategy {
 static void iplayer_init(Side *side);
 static void iplayer_init_turn(Side *side);
 static void create_strategy(Side *side);
-static void analyze_the_game(Side *side);
-static void determine_subgoals(Side *side);
-static void review_goals(Side *side);
-static void review_units(Side *side);
 static void update_unit_plans(Side *side);
-static void estimate_strengths(Side *side);
-static void add_goal(Side *side, Goal *goal);
-static Goal *has_goal(Side *side, GoalType goaltype);
-static Goal *has_unsatisfied_goal(Side *side, GoalType goaltype);
+static void review_units(Side *side);
 static void iplayer_decide_plan(Side *side, Unit *unit);
 static int iplayer_adjust_plan(Side *side, Unit *unit);
 
@@ -69,22 +62,13 @@ static int iplayer_go_after_victim(Unit *unit, int range);
 static int iplayer_go_after_captive(Unit *unit, int range);
 static int iplayer_fire_at_opportunity(Unit *unit);
 
-static int need_this_type_to_explore(Side *side, int u);
 /* static int compare_weights(struct weightelt *w1, struct weightelt *w2); */
-static void assign_to_exploration(Side *side, Unit *unit);
-static int need_this_type_to_build_explorers(Side *side, int u);
-static void assign_to_explorer_construction(Side *side, Unit *unit);
 static void assign_to_offense(Side *side, Unit *unit);
 static void assign_to_offense_support(Side *side, Unit *unit);
 static int type_can_build_attackers(Side *side, int u);
 static int preferred_build_type(Side *side, Unit *unit, int plantype);
-static int need_more_transportation(Side *side);
 static void assign_to_defense(Side *side, Unit *unit);
 static void assign_to_defense_support(Side *side, Unit *unit);
-static int assign_to_develop_on(Side *side, Unit *unit, int u2);
-static int can_develop_on(int u, int u2);
-static int needs_develop(Side *side, int u);
-static int probably_explorable(Side *side, int x, int y, int u);
 static int build_depot_for_self(Side *side, Unit *unit);
 static void assign_to_colonize(Side *side, Unit *unit);
 static void assign_to_improve(Side *side, Unit *unit);
@@ -94,7 +78,6 @@ static int blocked_by_enemy(Unit *unit, int x, int y, int shortest);
 static int attack_blockage(Side *side, Unit *unit, int x, int y, int shortest);
 static void iplayer_react_to_new_side(Side *side, Side *side2);
 static void iplayer_finish_movement(Side *side);
-static Unit *search_for_available_transport(Unit *unit, int purpose);
 static void assign_to_defend_unit(Unit *unit, Unit *unit2);
 static void assign_to_defend_cell(Unit *unit, int x, int y);
 static void assign_to_defend_vicinity(Unit *unit, int x, int y, int w, int h);
@@ -105,7 +88,6 @@ static char *iplayer_at_desig(Side *side, int x, int y);
 static int iplayer_theater_at(Side *side, int x, int y);
 static int iplayer_read_strengths(Side *side);
 static Obj *iplayer_save_state(Side *side);
-static int compare_weights(const void *w1, const void *w2);
 
 /* This is the set of operations that generic code will use. */
 
@@ -396,7 +378,7 @@ Unit *unit;
 		if (task == NULL || task->type != TASK_BUILD) {
 		    DMprintf("%s directed to build %s\n",
 			     unit_desig(unit), u_type_name(u3));
-		    net_set_build_task(unit, u3, 2);
+		    net_set_build_task(unit, u3, 2, 0, 0);
 		} else {
 		    DMprintf("%s already building, leaving alone\n",
 			     unit_desig(unit));
@@ -1238,7 +1220,7 @@ build_depot_for_self(Side *side, Unit *unit)
     if (cando) {
 	DMprintf("%s building %s as a depot for itself\n",
 		 unit_desig(unit), u_type_name(u2));
-	net_set_build_task(unit, u2, 1);
+	net_set_build_task(unit, u2, 1, 0, 0);
 	return TRUE;
     }
     return FALSE;
@@ -1575,38 +1557,13 @@ iplayer_read_strengths(Side *side)
     return found;
 }
 
-/* Write out any state that the iplayer must preserve.  We don't actually write;
-   instead we build a Lisp object and pass that back to the writing routines. */
+/* Write out any state that the iplayer must preserve.  We don't
+   actually write; instead we build a Lisp object and pass that back
+   to the writing routines. */
+/* At present, the iplayer has no state to save. */
 
 static Obj *
-iplayer_save_state(side)
-Side *side;
+iplayer_save_state(Side *side)
 {
-    int sn1, u;
-    Obj *rslt, *vallist, *uvallist;
-    Side *side1;
-
-    rslt = lispnil;
-    /* We're pushing bindings onto a list, so do in reverse of desired order. */
-    vallist = lispnil;
-    for_all_sides(side1) {
-	sn1 = side1->id;
-	uvallist = lispnil;
-	push_binding(&vallist, new_number(sn1), uvallist);
-    }
-    vallist = reverse(vallist);
-    push_cdr_binding(&rslt, intern_symbol("alstrengths0"), vallist);
-    vallist = lispnil;
-    for_all_sides(side1) {
-	sn1 = side1->id;
-	uvallist = lispnil;
-	push_binding(&vallist, new_number(sn1), uvallist);
-    }
-    vallist = reverse(vallist);
-    push_cdr_binding(&rslt, intern_symbol("strengths0"), vallist);
-    vallist = lispnil;
-    push_cdr_binding(&rslt, intern_symbol("alpoints0"), vallist);
-    vallist = lispnil;
-    push_cdr_binding(&rslt, intern_symbol("points0"), vallist);
     return lispnil;
 }
