@@ -65,6 +65,7 @@ extern void send_message(Side *side, SideMask sidemask, char *str);
 extern void set_side_self_unit(Side *side, struct a_unit *unit);
 extern void set_doctrine(Side *side, char *spec);
 extern void set_controlled_by(Side *side, Side *side2, int val);
+extern void set_side_research(Side *side, int a);
 
 #ifdef DESIGNERS
 extern void paint_view(Side *side, int x, int y, int r, int tview, int uview);
@@ -79,6 +80,10 @@ extern void set_unit_asleep(Side *side, Unit *unit, int flag, int recurse);
 extern void set_unit_reserve(Side *side, Unit *unit, int flag, int recurse);
 extern void set_unit_ai_control(Side *side, Unit *unit, int flag, int recurse);
 extern void set_unit_main_goal(Side *side, Unit *unit, Goal *goal);
+extern void set_unit_curadvance(Side *side, Unit *unit, int a);
+extern void set_unit_autoplan(Side *side, Unit *unit, int flag);
+extern void set_unit_autoresearch(Side *side, Unit *unit, int flag);
+extern void set_unit_autobuild(Side *side, Unit *unit, int flag);
 extern void set_unit_waiting_for_transport(Side *side, Unit *unit, int flag);
 extern void set_unit_name(Side *side, Unit *unit, char *newname);
 extern int disband_unit(Side *side, Unit *unit);
@@ -94,7 +99,6 @@ extern int designer_change_side(Unit *unit, Side *side);
 extern int designer_disband(Unit *unit);
 #endif /* DESIGNERS */
 
-extern void reserve_unit(Side *side, Unit *unit);
 extern void wake_unit(Side *side, Unit *unit, int wakeoccs);
 extern void wake_area(Side *side, int x, int y, int n, int occs);
 extern void set_formation(Unit *unit, Unit *leader, int x, int y,
