@@ -1,5 +1,5 @@
 /* The movement-related actions of Xconq.
-   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,9 +38,7 @@ extern int retreating_from;
 /* Record a move action as the next to do. */
 
 int
-prep_move_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
@@ -53,9 +51,7 @@ int x, y, z;
 }
 
 int
-net_prep_move_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+net_prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int rslt;
 
@@ -70,41 +66,39 @@ int x, y, z;
    task- and plan-level behavior. */
 
 int
-do_move_action(unit, unit2, nx, ny, nz)
-Unit *unit, *unit2;
-int nx, ny, nz;
+do_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int u, u2, t, rslt, speed, mpcost, acpcost, ox, oy, oz, ot, dist;
     int nummoves, dirs[NUMDIRS], numdirs, i, ix, iy, ndist;
 
     u = unit->type;  u2 = unit2->type;
-    t = terrain_at(nx, ny);
+    t = terrain_at(x, y);
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
     ot = terrain_at(ox, oy);
     speed = 100;
     mpcost = 1;
-    dist = distance(ox, oy, nx, ny);
+    dist = distance(ox, oy, x, y);
     if (dist == 0) {
 	/* Change of altitude within same cell and/or transport departure. */
-	if (unit->transport != NULL && nz == unit->z) {
-	    mpcost = move_unit(unit2, nx, ny);
+	if (unit->transport != NULL && z == unit->z) {
+	    mpcost = move_unit(unit2, x, y);
 	} else {
-	    unit2->z = nz;
+	    unit2->z = z;
 	    /* (should do more stuff - LOS view might have changed) */
 	}
 	acpcost = 1;
 	nummoves = 1;
 	rslt = A_ANY_DONE;
     } else if (dist == 1
-	       || (dist == 2 && border_slide_possible(u2, ox, oy, nx, ny))) {
+	       || (dist == 2 && border_slide_possible(u2, ox, oy, x, y))) {
 	/* Movement to an adjacent cell. */
-	if (!inside_area(nx, ny)) {
+	if (!inside_area(x, y)) {
 	    kill_unit(unit2, H_UNIT_LEFT_WORLD);
 	    rslt = A_ANY_DONE;
-	} else if (ut_vanishes_on(u2, t) && !can_move_via_conn(unit2, nx, ny)) {
+	} else if (ut_vanishes_on(u2, t) && !can_move_via_conn(unit2, x, y)) {
 	    kill_unit(unit2, H_UNIT_VANISHED);
 	    rslt = A_MOVE_UNIT_GONE;
-	} else if (ut_wrecks_on(u2, t) && !can_move_via_conn(unit2, nx, ny)) {
+	} else if (ut_wrecks_on(u2, t) && !can_move_via_conn(unit2, x, y)) {
 	    if (u_wrecked_type(u2) == NONUTYPE) {
 		/* Occupants always die if the wrecked unit disappears. */
 		kill_unit(unit, H_UNIT_WRECKED);
@@ -113,8 +107,8 @@ int nx, ny, nz;
 		   if the unit will vanish, so that occupants can escape if
 		   allowed for. */
 		if (!ut_vanishes_on(u_wrecked_type(u2), t)) {
-		    speed = unit_speed(unit2, nx, ny);
-		    mpcost = move_unit(unit2, nx, ny);
+		    speed = unit_speed(unit2, x, y);
+		    mpcost = move_unit(unit2, x, y);
 		}
 		wreck_unit(unit2);
 		/* Now make it go away, taking unlucky occupants with. */
@@ -123,10 +117,11 @@ int nx, ny, nz;
 		}
 	    }
 	    rslt = A_MOVE_UNIT_GONE;
+#if 0 /* seems better just to prevent via move costs... */
 	} else if (ut_vanishes_on(u2, ot)
 		   && (unit->transport == NULL
 		       || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
-		   && !can_move_via_conn(unit2, nx, ny)) {
+		   && !can_move_via_conn(unit2, x, y)) {
 	    /* This case is if the unit is already on a connection in
 	       hostile terrain and tries to move to hospitable terrain
 	       without using a bridge, or if it is in a transport that
@@ -136,7 +131,7 @@ int nx, ny, nz;
 	} else if (ut_wrecks_on(u2, ot)
 		   && (unit->transport == NULL
 		       || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
-		   && !can_move_via_conn(unit2, nx, ny)) {
+		   && !can_move_via_conn(unit2, x, y)) {
 	    if (u_wrecked_type(u2) == NONUTYPE) {
 		/* Occupants always die if the wrecked unit disappears. */
 		kill_unit(unit, H_UNIT_WRECKED);
@@ -151,9 +146,10 @@ int nx, ny, nz;
 		}
 	    }
 	    rslt = A_MOVE_UNIT_GONE;
+#endif
 	} else {
-	    speed = unit_speed(unit2, nx, ny);
-	    mpcost = move_unit(unit2, nx, ny);
+	    speed = unit_speed(unit2, x, y);
+	    mpcost = move_unit(unit2, x, y);
 	    /* ZOC move cost is added after action is done. */
 	    mpcost += zoc_move_cost(unit2, ox, oy, oz);
 	    rslt = A_ANY_DONE;
@@ -174,7 +170,7 @@ int nx, ny, nz;
 	acpcost = 0;
 	nummoves = 0;
 	while (dist > 0) {
-	    numdirs = choose_move_dirs(unit2, nx, ny, TRUE,
+	    numdirs = choose_move_dirs(unit2, x, y, TRUE,
 				       plausible_move_dir, sort_directions, dirs);
 	    if (numdirs == 0)
 	      break;
@@ -195,7 +191,7 @@ int nx, ny, nz;
 		    break;
 		}
 	    }
-	    ndist = distance(unit2->x, unit2->y, nx, ny);
+	    ndist = distance(unit2->x, unit2->y, x, y);
 	    if (ndist >= dist)
 	      break;
 	    dist = ndist;
@@ -217,9 +213,7 @@ int nx, ny, nz;
 }
 
 int
-check_move_action(unit, unit2, nx, ny, nz)
-Unit *unit, *unit2;
-int nx, ny, nz;
+check_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int u, u2, u3, ox, oy, oz, acp, acpavail, mpavail, totcost, m, speed;
     int dist, maxmpavail;
@@ -229,7 +223,7 @@ int nx, ny, nz;
     if (!in_play(unit2))
       return A_ANY_ERROR;
     /* Note that edge cell dests (used to leave the world) are allowed. */
-    if (!in_area(nx, ny))
+    if (!in_area(x, y))
       return A_ANY_ERROR;
     u = unit->type;  u2 = unit2->type;
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
@@ -253,25 +247,26 @@ int nx, ny, nz;
     if (!has_supply_to_act(unit2))
       return A_ANY_NO_MATERIAL;
     /* Destination is outside the world and we're not allowed to leave. */
-    if (!inside_area(nx, ny) && u_mp_to_leave_world(u2) < 0)
+    if (!inside_area(x, y) && u_mp_to_leave_world(u2) < 0)
 	return A_MOVE_CANNOT_LEAVE_WORLD;
     /* Check if the destination is within our move range. */
-    dist = distance(ox, oy, nx, ny);
-    if ((dist == 2 && !border_slide_possible(u, ox, oy, nx, ny))
+    dist = distance(ox, oy, x, y);
+    if ((dist == 2 && !border_slide_possible(u, ox, oy, x, y))
 	&& dist > u_move_range(u2))
       return A_ANY_TOO_FAR;
-    if (nz > 0)
+    /* Disallow nonzero altitudes for now. */
+    if (z > 0)
       return A_ANY_TOO_FAR;
     /* Check if the destination is in a blocking ZOC. */
-    if (in_blocking_zoc(unit, nx, ny, nz))
+    if (in_blocking_zoc(unit, x, y, z))
       return A_ANY_ERROR;
     /* Now start looking at the move costs. */
     u3 = (unit2->transport ? unit2->transport->type : NONUTYPE);
-    totcost = total_move_cost(u2, u3, ox, oy, oz, nx, ny, nz);
-    speed = unit_speed(unit2, nx, ny);
+    totcost = total_move_cost(u2, u3, ox, oy, oz, x, y, z);
+    speed = unit_speed(unit2, x, y);
     /* A unit with an adjusted speed of zero cannot move except within
        a cell. */
-    if (speed == 0 && !(ox == nx && oy == ny && oz == nz))
+    if (speed == 0 && !(ox == x && oy == y && oz == z))
       return A_ANY_CANNOT_DO;
     /* Check if unit can possibly have enough mp for the move. */
     maxmpavail =
@@ -286,17 +281,17 @@ int nx, ny, nz;
     if (u_acp_min(u) < 0)
       mpavail = ((unit->act->acp - u_acp_min(u)) * speed) / 100;
     /* Zero mp always disallows movement, unless intra-cell. */
-    if (mpavail <= 0 && !(ox == nx && oy == ny && oz == nz))
+    if (mpavail <= 0 && !(ox == x && oy == y && oz == z))
       return A_MOVE_NO_MP;
     /* The free mp might get us enough moves, so add it before comparing. */
     if (mpavail + u_free_mp(u2) < totcost)
       return A_MOVE_NO_MP;
     /* If destination is too small or already full, we can't move into it. */
-    if ((nz & 1) == 0) {
-	if (!can_occupy_cell(unit2, nx, ny))
+    if ((z & 1) == 0) {
+	if (!can_occupy_cell(unit2, x, y))
 	  return A_MOVE_DEST_FULL;
     } else {
-	if (!can_occupy_conn(unit2, nx, ny, nz))
+	if (!can_occupy_conn(unit2, x, y, z))
 	  return A_MOVE_DEST_FULL;
     }
     /* We have to have a minimum level of supply to be able to move. */
@@ -308,9 +303,7 @@ int nx, ny, nz;
 }
 
 int
-can_move_via_conn(unit, nx, ny)
-Unit *unit;
-int nx, ny;
+can_move_via_conn(Unit *unit, int nx, int ny)
 {
     int c, dir;
 
@@ -330,8 +323,7 @@ int nx, ny;
 }
 
 int
-border_slide_possible(u, ox, oy, nx, ny)
-int u, ox, oy, nx, ny;
+border_slide_possible(int u, int ox, int oy, int nx, int ny)
 {
     int dx, dy, dir, bx, by, bdir, b;
 
@@ -636,8 +628,7 @@ in_blocking_zoc(Unit *unit, int x, int y, int z)
 }
 
 static void
-test_blocking_zoc(x, y)
-int x, y;
+test_blocking_zoc(int x, int y)
 {
     int u = tmpunit->type, u2, t = terrain_at(x, y), range;
     Unit *unit2;
@@ -657,8 +648,7 @@ int x, y;
 /* This is true if unit2 wants to block unit from moving. */
 
 int
-unit_blockable_by(unit, unit2)
-Unit *unit, *unit2;
+unit_blockable_by(Unit *unit, Unit *unit2)
 {
     return (!trusted_side(unit->side, unit2->side) /* should make a better test */
 	    && uu_mp_to_enter_zoc(unit->type, unit2->type) < 0);
@@ -668,8 +658,7 @@ Unit *unit, *unit2;
    move. */
 
 int
-total_move_cost(u, u2, x1, y1, z1, x2, y2, z2)
-int u, u2, x1, y1, z1, x2, y2, z2;
+total_move_cost(int u, int u2, int x1, int y1, int z1, int x2, int y2, int z2)
 {
     int cost, ferry, b, c, conncost, travcost, dist, dir;
 
@@ -703,6 +692,16 @@ int u, u2, x1, y1, z1, x2, y2, z2;
     }
     if (ferry < over_own) {
     	cost += ut_mp_to_leave(u, terrain_at(x1, y1));
+	/* Add in effect of any coatings at the origin. */
+	if (numcoattypes > 0) {
+	    for_all_terrain_types(c) {
+		if (t_is_coating(c)
+		    && aux_terrain_defined(c)
+		    && aux_terrain_at(x1, y1, c) > 0) {
+		    cost += ut_mp_to_leave(u, c);
+		}
+	    }
+	}
     }
     if (numbordtypes > 0 && ferry < over_border) {
 	/* Add any necessary border crossing costs. */
@@ -719,6 +718,16 @@ int u, u2, x1, y1, z1, x2, y2, z2;
     }
     if (ferry < over_all) {
 	cost += ut_mp_to_enter(u, terrain_at(x2, y2));
+	/* Add in effect of any coatings at the destination. */
+	if (numcoattypes > 0) {
+	    for_all_terrain_types(c) {
+		if (t_is_coating(c)
+		    && aux_terrain_defined(c)
+		    && aux_terrain_at(x2, y2, c) > 0) {
+		    cost += ut_mp_to_enter(u, c);
+		}
+	    }
+	}
     }
     /* Use a connection traversal if it would be cheaper.  This is
        only automatic if the connection on/off costs are small enough,
