@@ -15,6 +15,7 @@ any later version.  See the file COPYING.  */
    different language.) */
 
 #include "conq.h"
+extern void size_desc(char *buf, Unit *unit, int label);
 #include "kernel.h"
 
 static void notify_combat(Unit *unit, Unit *atker, char *str);
@@ -22,6 +23,7 @@ static int pattern_matches_combat(Obj *pattern, Unit *unit, Unit *unit2);
 static void combat_desc_from_list(Side *side, Obj *lis, Unit *unit,
 				  Unit *unit2, char *str, char *buf);
 static void init_calendar(void);
+static void parse_date_step_range(Obj *form);
 static void maybe_mention_date(Side *side);
 static int gain_count(Side *side, int u, int r);
 static int loss_count(Side *side, int u, int r);
@@ -78,14 +80,21 @@ typedef struct a_usualdate {
     int year;
 } UsualDate;
 
+typedef struct a_usualdatesteprange {
+    int turn_start;
+    int turn_end;
+    UsualDateStepType step_type;
+    int step_size;
+} UsualDateStepRange;
+
 static char *usual_date_string(int date);
-static void parse_usual_date(char *datestr, UsualDate *udate);
+static void parse_usual_date(char *datestr, int range, UsualDate *udate);
 
 static int turn_initial;
 
-static UsualDateStepType datesteptype;
+static UsualDateStepRange date_step_ranges[20];
 
-static int datestep;
+static int num_date_step_ranges;
 
 static char *months[] = {
   "Jan", "Feb", "Mar", "Apr", "May", "Jun",
@@ -723,6 +732,52 @@ int u;
     }
 }
 
+/* Given a unit and optional advance a, summarize research status
+   and timing. */
+
+void
+research_desc(buf, unit, a)
+char *buf;
+Unit *unit;
+int a;
+{
+    char abuf[10], tmpbuf[100];
+
+    if (a != NONATYPE) {
+	if (u_advanced(unit->type)) {
+	    sprintf(abuf, "[%2d] ", a_rp(a));
+	} else {
+	    strcpy(abuf, " --  ");
+	}
+    } else {
+	abuf[0] = '\0';
+    }
+    name_or_number(unit, tmpbuf);
+    sprintf(buf, "%s%s %s", abuf, u_type_name(unit->type), tmpbuf);
+    pad_out(buf, 25);
+}
+
+void
+researchible_desc(buf, unit, a)
+char *buf;
+Unit *unit;
+int a;
+{
+    char abuf[10];
+
+    if (a != NONATYPE && unit != NULL) {
+	if (u_advanced(unit->type)) {
+	    sprintf(abuf, "[%2d] ", a_rp(a));
+	} else {
+	    strcpy(abuf, " --  ");
+	}
+    } else {
+	abuf[0] = '\0';
+    }
+    sprintf(buf, "%s%s", abuf, a_type_name(a));
+    pad_out(buf, 25);
+}
+
 /* This generates a textual description of a type's construction info,
    including estimated time for the given unit to build one, tooling &
    tech, plus number of that type in existence already. */
@@ -1525,10 +1580,16 @@ int x, y;
 #endif
 
 void
-hp_desc(buf, unit, label)
-char *buf;
-Unit *unit;
-int label;
+size_desc(char *buf, Unit *unit, int label)
+{
+    buf[0] = '\0';
+    if (!u_advanced(unit->type))
+      return;
+    tprintf(buf, "(%d)", unit->size);
+}
+
+void
+hp_desc(char *buf, Unit *unit, int label)
 {
     if (label) {
 	sprintf(buf, "HP ");
@@ -1544,10 +1605,7 @@ int label;
 }
 
 void
-acp_desc(buf, unit, label)
-char *buf;
-Unit *unit;
-int label;
+acp_desc(char *buf, Unit *unit, int label)
 {
     int u = unit->type;
 
@@ -2529,8 +2587,7 @@ static void
 init_calendar(void)
 {
     int caltypeunknown = FALSE;
-    char *stepnamestr;
-    Obj *cal, *caltype, *stepname, *step;
+    Obj *cal, *caltype, *stepname, *steprest;
 
     cal = g_calendar();
     /* Turn-based calendar is the default. */
@@ -2540,38 +2597,21 @@ init_calendar(void)
 	/* Default is fine. */
     } else if (consp(cal)) {
 	caltype = car(cal);
-	if ((symbolp(caltype) || stringp(caltype))
-	    && strcmp("usual", c_string(caltype)) == 0) {
+	if (symbolp(caltype) && strcmp("usual", c_string(caltype)) == 0) {
 	    calendar_type = cal_usual;
 	    stepname = cadr(cal);
-	    if (symbolp(stepname) || stringp(stepname)) {
-		stepnamestr = c_string(stepname);
-		if (strcmp(stepnamestr, "second") == 0) {
-		    datesteptype = ds_second;
-		} else if (strcmp(stepnamestr, "minute") == 0) {
-		    datesteptype = ds_minute;
-		} else if (strcmp(stepnamestr, "hour") == 0) {
-		    datesteptype = ds_hour;
-		} else if (strcmp(stepnamestr, "day") == 0) {
-		    datesteptype = ds_day;
-		} else if (strcmp(stepnamestr, "week") == 0) {
-		    datesteptype = ds_week;
-		} else if (strcmp(stepnamestr, "month") == 0) {
-		    datesteptype = ds_month;
-		} else if (strcmp(stepnamestr, "season") == 0) {
-		    datesteptype = ds_season;
-		} else if (strcmp(stepnamestr, "year") == 0) {
-		    datesteptype = ds_year;
-		} else {
-		    init_warning("\"%s\" not a known date step name", stepnamestr);
+	    if (symbolp(stepname)) {
+		parse_date_step_range(cdr(cal));
+	    } else if (consp(stepname)) {
+		for_all_list(cdr(cal), steprest) {
+		    parse_date_step_range(car(steprest));
 		}
 	    } else {
 		init_warning("No name for date step type, substituting `day'");
-		datesteptype = ds_day;
+		date_step_ranges[0].step_type = ds_day;
+		date_step_ranges[0].step_size = 1;
+		num_date_step_ranges = 1;
 	    }
-	    /* Collect an optional multiple. */
-	    step = caddr(cal);
-	    datestep = (numberp(step) ? c_number(step) : 1);
 	    if (!empty_string(g_initial_date())) {
 		set_initial_date(g_initial_date());
 	    }
@@ -2589,11 +2629,61 @@ init_calendar(void)
        could allow bogus game designs to proceed. */
 }
 
+static void
+parse_date_step_range(Obj *form)
+{
+    int n;
+    char *stepnamestr;
+    UsualDateStepType steptype;
+    Obj *step;
+
+    n = num_date_step_ranges;
+    /* Peel off a turn range if supplied. */
+    if (numberp(car(form))) {
+	date_step_ranges[n].turn_start = c_number(car(form));
+	form = cdr(form);
+    }
+    if (numberp(car(form))) {
+	date_step_ranges[n].turn_end = c_number(car(form));
+	form = cdr(form);
+    }
+    if (symbolp(car(form))) {
+	stepnamestr = c_string(car(form));
+	if (strcmp(stepnamestr, "second") == 0) {
+	    steptype = ds_second;
+	} else if (strcmp(stepnamestr, "minute") == 0) {
+	    steptype = ds_minute;
+	} else if (strcmp(stepnamestr, "hour") == 0) {
+	    steptype = ds_hour;
+	} else if (strcmp(stepnamestr, "day") == 0) {
+	    steptype = ds_day;
+	} else if (strcmp(stepnamestr, "week") == 0) {
+	    steptype = ds_week;
+	} else if (strcmp(stepnamestr, "month") == 0) {
+	    steptype = ds_month;
+	} else if (strcmp(stepnamestr, "season") == 0) {
+	    steptype = ds_season;
+	} else if (strcmp(stepnamestr, "year") == 0) {
+	    steptype = ds_year;
+	} else {
+	    init_warning("\"%s\" not a known date step name", stepnamestr);
+	    steptype = ds_day;
+	}
+    } else {
+	/* (should warn) */
+	steptype = ds_day;
+    }
+    date_step_ranges[n].step_type = steptype;
+    /* Collect an optional multiple. */
+    step = cadr(form);
+    date_step_ranges[n].step_size = (numberp(step) ? c_number(step) : 1);
+    ++num_date_step_ranges;
+}
+
 /* Compose a date, omitting components supplied by the base date. */
 
 char *
-relative_date_string(date, base)
-int date, base;
+relative_date_string(int date, int base)
 {
     if (calendar_type == cal_unknown)
       init_calendar();
@@ -2615,8 +2705,7 @@ int date, base;
 /* Given two dates, figure out how many turns encompassed by them. */
 
 int
-turns_between(datestr1, datestr2)
-char *datestr1, *datestr2;
+turns_between(char *datestr1, char *datestr2)
 {
     int rslt, turn1, turn2;
     UsualDate date1, date2;
@@ -2629,29 +2718,34 @@ char *datestr1, *datestr2;
 	sscanf("%d", datestr2, &turn2);
 	return (turn2 - turn1);
       case cal_usual:
-	parse_usual_date(datestr1, &date1);
-	parse_usual_date(datestr2, &date2);
+	parse_usual_date(datestr1, 0, &date1);
+	parse_usual_date(datestr2, 0, &date2);
 	rslt = date2.year - date1.year;
-	if (datesteptype == ds_year)
-	  return (rslt / datestep);
-	if (datesteptype < ds_year) {
-	    rslt = (12 * rslt) - (date2.month - date1.month);
+	if (num_date_step_ranges == 1) {
+	    if (date_step_ranges[0].step_type == ds_year)
+	      return (rslt / date_step_ranges[0].step_size);
+	    if (date_step_ranges[0].step_type < ds_year) {
+		rslt = (12 * rslt) - (date2.month - date1.month);
+	    }
+	    if (date_step_ranges[0].step_type == ds_month)
+	      return (rslt / date_step_ranges[0].step_size);
+	    if (date_step_ranges[0].step_type < ds_month) {
+		rslt = (30 * rslt) - (date2.day - date1.day);
+	    }
+	    if (date_step_ranges[0].step_type == ds_week)
+	      return (((rslt + 6) / 7) / date_step_ranges[0].step_size);
+	    if (date_step_ranges[0].step_type == ds_day)
+	      return (rslt / date_step_ranges[0].step_size);
+	    if (date_step_ranges[0].step_type < ds_day) {
+		rslt = (24 * rslt) - (date2.hour - date1.hour);
+	    }
+	    if (date_step_ranges[0].step_type == ds_hour)
+	      return (rslt / date_step_ranges[0].step_size);
+	    return (rslt / date_step_ranges[0].step_size); /* semi-bogus */
+	} else {
+	    /* Too complicated for now, give up. */
+	    return 1;
 	}
-	if (datesteptype == ds_month)
-	  return (rslt / datestep);
-	if (datesteptype < ds_month) {
-	    rslt = (30 * rslt) - (date2.day - date1.day);
-	}
-	if (datesteptype == ds_week)
-	  return (((rslt + 6) / 7) / datestep);
-	if (datesteptype == ds_day)
-	  return (rslt / datestep);
-	if (datesteptype < ds_day) {
-	    rslt = (24 * rslt) - (date2.hour - date1.hour);
-	}
-	if (datesteptype == ds_hour)
-	  return (rslt / datestep);
-	return (rslt / datestep); /* semi-bogus */
       default:
 	case_panic("calendar type", calendar_type);
 	break;
@@ -2660,8 +2754,7 @@ char *datestr1, *datestr2;
 }
 
 void
-set_initial_date(str)
-char *str;
+set_initial_date(char *str)
 {
     if (calendar_type == cal_unknown)
       init_calendar();
@@ -2672,7 +2765,7 @@ char *str;
       case cal_usual:
 	if (usual_initial == NULL)
 	  usual_initial = (UsualDate *) xmalloc(sizeof(UsualDate));
-	parse_usual_date(str, usual_initial);
+	parse_usual_date(str, 0, usual_initial);
 	break;
       default:
 	case_panic("calendar type", calendar_type);
@@ -2686,9 +2779,7 @@ char *str;
 /* (should use strtol etc instead of sscanf) */
 
 static void
-parse_usual_date(datestr, udate)
-char *datestr;
-UsualDate *udate;
+parse_usual_date(char *datestr, int range, UsualDate *udate)
 {
     char aname[BUFSIZE];
     int i, cnt;
@@ -2698,7 +2789,7 @@ UsualDate *udate;
     aname[0] = '\0';
     if (!empty_string(datestr)) {
 	/* Assume it's in a standard date format. */
-	switch (datesteptype) {
+	switch (date_step_ranges[0].step_type) {
 	  case ds_second:
 	    cnt = sscanf(datestr, "%d:%d:%d %d %s %d",
 			 &(udate->second), &(udate->minute), &(udate->hour),
@@ -2768,7 +2859,8 @@ UsualDate *udate;
 	      goto bad_format;
 	    return;
 	  default:
-	    init_warning("%d not an allowed date step type", datesteptype);
+	    init_warning("%d not an allowed date step type",
+			 date_step_ranges[0].step_type);
 	    break;
 	}
 	for (i = 0; i < 12; ++i) {
@@ -2787,13 +2879,14 @@ UsualDate *udate;
 
 /* Given a numeric date, convert it into something understandable. */
 
+static int ever_mentioned_bc;
+
 static char *
-usual_date_string(date)
-int date;
+usual_date_string(int date)
 {
     int year = 0, season = 0, month = 0, day = 0;
     int hour = 0, second = 0, minute = 0;
-    int i;
+    int i, r, range;
 
     /* The date, which is a turn number, should be 1 or more, but this
        routine may be called before the game really starts, so return
@@ -2803,14 +2896,24 @@ int date;
     /* First displayed date is normally turn 1; be zero-based for
        the benefit of calculation. */
     --date;
+    /* Find the date step type and size. */
+    for (range = 0; range < num_date_step_ranges; ++range) {
+	if (between(date_step_ranges[range].turn_start,
+		    date,
+		    date_step_ranges[range].turn_end))
+	  break;
+    }
+    /* If no matches, just use the last range in the array. */
+    if (range >= num_date_step_ranges)
+      range = num_date_step_ranges - 1;
     /* If multiples of basic step, convert to basic step by multiplying. */
-    date *= datestep;
+    date *= date_step_ranges[range].step_size;
     if (usual_initial == NULL) {
 	usual_initial = (UsualDate *) xmalloc(sizeof(UsualDate));
 	if (!empty_string(g_initial_date()))
-	  parse_usual_date(g_initial_date(), usual_initial);
+	  parse_usual_date(g_initial_date(), 0, usual_initial);
     }
-    switch (datesteptype) {
+    switch (date_step_ranges[range].step_type) {
       case ds_second:
 	second = date % 60;
 	minute = date / 60;
@@ -2840,7 +2943,8 @@ int date;
 	}
 	++day;
 	year = date / 365 + usual_initial->year;
-	sprintf(datebuf, "%d:00 %2d %s %d", hour, day, months[month], ABS(year));
+	sprintf(datebuf, "%d:00 %2d %s %d",
+		hour, day, months[month], ABS(year));
 	break;
       case ds_week:
 	/* Convert to days, then proceed as for days. */
@@ -2874,15 +2978,28 @@ int date;
 	sprintf(datebuf, "%s %d", seasons[season], ABS(year));
 	break;
       case ds_year:
-	year = date + usual_initial->year;
+	year = usual_initial->year;
+	/* (should do something similar for other step types) */
+	for (r = 0; r < range; ++r)
+	  year += ((date_step_ranges[r].turn_end
+		    - date_step_ranges[r].turn_start)
+		   * date_step_ranges[r].step_size);
+	year += (date - (date_step_ranges[range].turn_start
+			 * date_step_ranges[range].step_size));
 	sprintf(datebuf, "%d", ABS(year));
 	break;
       default:
-	sprintf(datebuf, "%d is unknown date step type", datesteptype);
+	sprintf(datebuf, "%d is unknown date step type",
+		date_step_ranges[0].step_type);
 	break;
     }
     if (year < 0) {
 	strcat(datebuf, " BC");
+	ever_mentioned_bc = TRUE;
+    } else if (ever_mentioned_bc) {
+	/* If any date was displayed as "BC", use "AD" with all positive
+	   year numbers. */
+	strcat(datebuf, " AD");
     }
     return datebuf;
 }
