@@ -1,5 +1,5 @@
 /* Unit plan handling for Xconq.
-   Copyright (C) 1991-1999 Stanley T. Shebs.
+   Copyright (C) 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,6 +8,7 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 extern int target_visible(Unit *unit, Task *task);
+extern int production_at(int x, int y, int m);
 #include "kernel.h"
 extern int repair_if_damaged(Unit *unit);
 
@@ -21,9 +22,7 @@ static void plan_exploration(Unit *unit);
 static void plan_explorer_support(Unit *unit);
 static void plan_improve(Unit *unit);
 static void plan_colonize(Unit *unit);
-#if 0 /* not used currently */
-static void plan_colonizer_support(Unit *unit);
-#endif
+static void plan_colonize_support(Unit *unit);
 static int good_cell_to_colonize(int u, Unit *unit);
 static void plan_random(Unit *unit);
 static void wake_at(int x, int y);
@@ -52,8 +51,7 @@ char *plantypenames[] = {
    incomplete units to give them a plan. */
 
 void
-init_unit_plan(unit)
-Unit *unit;
+init_unit_plan(Unit *unit)
 {
     if (u_acp(unit->type) > 0) {
 	/* Might already have a plan, so don't always realloc. */
@@ -83,10 +81,7 @@ Unit *unit;
 }
 
 void
-set_unit_plan_type(side, unit, type)
-Side *side;
-Unit *unit;
-int type;
+set_unit_plan_type(Side *side, Unit *unit, int type)
 {
     int oldtype;
 
@@ -177,10 +172,31 @@ int flag, recurse;
 }
 
 void
-set_unit_main_goal(side, unit, goal)
-Side *side;
-Unit *unit;
-Goal *goal;
+set_unit_curadvance(Side *side, Unit *unit, int a)
+{    
+	unit->curadvance = a;
+}
+
+void
+set_unit_autoplan(Side *side, Unit *unit, int flag)
+{    
+	unit->autoplan = flag;
+}
+
+void
+set_unit_autoresearch(Side *side, Unit *unit, int flag)
+{    
+	unit->autoresearch = flag;
+}
+
+void
+set_unit_autobuild(Side *side, Unit *unit, int flag)
+{    
+	unit->autobuild = flag;
+}
+
+void
+set_unit_main_goal(Side *side, Unit *unit, Goal *goal)
 {
     if (unit->plan) {
 	unit->plan->maingoal = goal;
@@ -188,10 +204,7 @@ Goal *goal;
 }
 
 void
-set_unit_waiting_for_transport(side, unit, flag)
-Side *side;
-Unit *unit;
-int flag;
+set_unit_waiting_for_transport(Side *side, Unit *unit, int flag)
 {
     if (unit->plan) {
 	unit->plan->waitingfortransport = flag;
@@ -206,7 +219,10 @@ execute_plan(Unit *unit)
     Plan *plan = unit->plan;
 
     /* Plan execution for advanced units is (for now) handled in run.c. */
-    if (u_advanced(unit->type)) {
+    if (u_advanced(unit->type)
+	/* not the ideal test, but good enough */
+	&& u_advanced_auto_construct(unit->type)
+	&& u_advanced_auto_research(unit->type)) {
 	unit->act->acp = unit->act->initacp = 0;
 	return 0;
     }
@@ -270,8 +286,7 @@ execute_plan(Unit *unit)
    to move back if so. */
 
 int
-move_into_formation(unit)
-Unit *unit;
+move_into_formation(Unit *unit)
 {
     int nx, ny, dist; 
     Plan *plan = unit->plan;
@@ -310,9 +325,7 @@ int task_is_in_agenda(Plan *plan, Task *task);
    and schedule a task if so.  Return TRUE if a task was added. */
 
 int
-execute_standing_order(unit, addtask)
-Unit *unit;
-int addtask;
+execute_standing_order(Unit *unit, int addtask)
 {
     Unit *transport;
     Side *side = unit->side;
@@ -419,7 +432,7 @@ plan_passive(Unit *unit)
 		&& strcmp(mainmodule->origmodulename, INTRO_GAME) == 0))
 	&& ((strcmp(u_type_name(unit->type), "city") == 0)
 	    || (strcmp(u_type_name(unit->type), "town") == 0))) {
-	push_build_task(unit, 0, 99);
+	push_build_task(unit, 0, 99, 0, 0);
     }
     if (plan->supply_is_low && plan->supply_alarm) {
 	plan->supply_alarm = FALSE;
@@ -604,12 +617,11 @@ Unit *unit;
     }
 }
 
-/* Look through list of occupants to see if an occupant needs the transport to do
-   something. */
+/* Look through list of occupants to see if an occupant needs the
+   transport to do something. */
 
 int
-do_for_occupants(unit)
-Unit *unit;
+do_for_occupants(Unit *unit)
 {
     Unit *occ;
     Goal *goal;
@@ -652,8 +664,7 @@ Unit *unit;
 }
 
 int
-self_build_base_for_self(unit)
-Unit *unit;
+self_build_base_for_self(Unit *unit)
 {
     int u = unit->type, u2, cando = FALSE;
 
@@ -672,15 +683,14 @@ Unit *unit;
     if (cando) {
 	DMprintf("%s building %s as a base for itself\n",
 		     unit_desig(unit), u_type_name(u2));
-	set_build_task(unit, u2, 1);
+	set_build_task(unit, u2, 1, 0, 0);
 	return TRUE;
     }
     return FALSE;
 }
 
 static void
-plan_offense_support(unit)
-Unit *unit;
+plan_offense_support(Unit *unit)
 {
     int u = unit->type, u2, u3 = NONUTYPE, backup = NONUTYPE;
     Task *task;
@@ -705,7 +715,7 @@ Unit *unit;
     	if (task == NULL || task->type != TASK_BUILD) {
 	    DMprintf("%s supporting offense by building %s\n",
 		     unit_desig(unit), u_type_name(u3));
-	    set_build_task(unit, u3, 2);
+	    set_build_task(unit, u3, 2, 0, 0);
 	} else {
 	    DMprintf("%s already building, leaving alone\n",
 		     unit_desig(unit));
@@ -893,135 +903,143 @@ Unit *unit;
 static void
 plan_colonize(Unit *unit)
 {
-	Plan 	*plan = unit->plan;
-	Unit	*unit2;
-	int 	x, y, u2;
+    Plan *plan = unit->plan;
+    Unit *unit2;
+    int x, y, u2;
 
-	if (resupply_if_low(unit)) {
-		if (plan->tasks)
-			execute_task(unit);
-		return;
-	}
-	if (repair_if_damaged(unit)) {
-		if (plan->tasks)
-	  		execute_task(unit);
-		return;
-	}
-	/* Set build task if we are colonizing and have found a good spot. */
-	if (plan->maingoal && plan->maingoal->type == GOAL_COLONIZE) {
-	     	u2 = plan->maingoal->args[0];
-	     	if (good_cell_to_colonize(u2, unit)) {
-			set_build_task(unit, u2, 1);	
-			DMprintf("%s colonizing by building %s\n", 
-		     			unit_desig(unit), u_type_name(u2));
-		}
-	}
-	/* Then proceed with the tasks. */
-	if (plan->tasks) {
-		execute_task(unit);
-		return;
-	}
-	/* The task queue was empty. Get a new task. */
-	if (plan->maingoal) {
-		switch (plan->maingoal->type) {
+    if (!mobile(unit->type)) {
+	plan_colonize_support(unit);
+    }
 
-		  case GOAL_UNIT_OCCUPIED:
-			/* Move to occupy our goal if necessary. */
-			unit2 = find_unit(plan->maingoal->args[0]);
-			if (unit->transport != unit2)
-				set_occupy_task(unit, unit2);
-		  break;
-
-		  case GOAL_CELL_OCCUPIED:
-			/* Move to our goal if necessary. */
-		    	x = plan->maingoal->args[0];  
-		    	y = plan->maingoal->args[1];
-			if (unit->x != x || unit->y != y)
-				set_move_to_task(unit, x, y, 0);
-		  break;
-
-		  case GOAL_COLONIZE:
-			/* Move in a random dir along a straight line. */
-			set_move_dir_task(unit, random_dir(), xrandom(20));
-		  break;
-
-		  default:
-			/* Should never happen! */
-			DMprintf("Colonizer %s has weird goal %s\n",
-					unit_desig(unit), goal_desig(plan->maingoal));
-		    break;
-		}
-	} else {
-		/* Should never happen! */
-		DMprintf("Colonizer %s has no goal\n", unit_desig(unit));
+    if (resupply_if_low(unit)) {
+	if (plan->tasks)
+	  execute_task(unit);
+	return;
+    }
+    if (repair_if_damaged(unit)) {
+	if (plan->tasks)
+	  execute_task(unit);
+	return;
+    }
+    /* Set build task if we are colonizing and have found a good spot. */
+    if (plan->maingoal && plan->maingoal->type == GOAL_COLONIZE) {
+	u2 = plan->maingoal->args[0];
+	if (good_cell_to_colonize(u2, unit)) {
+	    set_build_task(unit, u2, 1, 0, 0);	
+	    DMprintf("%s colonizing by building %s\n", 
+		     unit_desig(unit), u_type_name(u2));
 	}
-		/* Execute the new task. */
-	if (plan->tasks) {
-		execute_task(unit);
-		return;
-	} else {
-		/* Holding occupied unit and waiting for something to happen. */
-		DMprintf("Colonizer %s with goal %s unable to find new task\n",
-				unit_desig(unit), goal_desig(plan->maingoal));
-	}	
+    }
+    /* Then proceed with the tasks. */
+    if (plan->tasks) {
+	execute_task(unit);
+	return;
+    }
+    /* The task queue was empty. Get a new task. */
+    if (plan->maingoal) {
+	switch (plan->maingoal->type) {
+
+	  case GOAL_UNIT_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    unit2 = find_unit(plan->maingoal->args[0]);
+	    if (unit->transport != unit2)
+	      set_occupy_task(unit, unit2);
+	    break;
+
+	  case GOAL_CELL_OCCUPIED:
+	    /* Move to our goal if necessary. */
+	    x = plan->maingoal->args[0];  
+	    y = plan->maingoal->args[1];
+	    if (unit->x != x || unit->y != y)
+	      set_move_to_task(unit, x, y, 0);
+	    break;
+
+	  case GOAL_COLONIZE:
+	    /* Move in a random dir along a straight line. */
+	    set_move_dir_task(unit, random_dir(), xrandom(20));
+	    break;
+
+	  default:
+	    /* Should never happen! */
+	    DMprintf("Colonizer %s has weird goal %s\n",
+		     unit_desig(unit), goal_desig(plan->maingoal));
+	    break;
+	}
+    } else {
+	/* Should never happen! */
+	DMprintf("Colonizer %s has no goal\n", unit_desig(unit));
+    }
+    /* Execute the new task. */
+    if (plan->tasks) {
+	execute_task(unit);
+	return;
+    } else {
+	/* Holding occupied unit and waiting for something to happen. */
+	DMprintf("Colonizer %s with goal %s unable to find new task\n",
+		 unit_desig(unit), goal_desig(plan->maingoal));
+    }	
 }
 
-#if 0 /* not used currently */
 static void
-plan_colonizer_support(Unit *unit)
+plan_colonize_support(Unit *unit)
 {
-    int 	type[MAXUTYPES];
-    int 	numtypes = 0;
-    int 	u;
+    int u, type[MAXUTYPES], numtypes = 0;
 
-	/* Load all colonizing types into type vector. */
+    /* Load all colonizing types into type vector. */
     for_all_unit_types(u) {
 	if (type_allowed_on_side(u, unit->side) 
 	    /* No longer included in type_allowed_on_side. */
 	    && has_advance_to_build(unit->side, u)
 	    && could_create(unit->type, u)
 	    && u_colonizer_worth(u) > 0) {
-	    	type[numtypes] = u;
-	    	numtypes++;
-	} 
+	    type[numtypes] = u;
+	    numtypes++;
+	}
     }
     /* Then pick one of them at random. */
     if (numtypes) {
 	u = type[xrandom(numtypes)];
 	DMprintf("%s supporting colonization by building %s\n",
 		 unit_desig(unit), u_type_name(u));
-	push_build_task(unit, u, 1);
+	push_build_task(unit, u, 1, 0, 0);
 	return;
+    } else {
 	/* Load all possible types instead if we failed. */
-    } else for_all_unit_types(u) {
-	if (type_allowed_on_side(u, unit->side) 
-	    /* No longer included in type_allowed_on_side. */
-	    && has_advance_to_build(unit->side, u)
-	    && could_create(unit->type, u)) {
+	for_all_unit_types(u) {
+	    if (type_allowed_on_side(u, unit->side) 
+		/* No longer included in type_allowed_on_side. */
+		&& has_advance_to_build(unit->side, u)
+		&& could_create(unit->type, u)) {
 	    	type[numtypes] = u;
 	    	numtypes++;
-	} 
+	    }
+	}
     }
     /* Then pick one of them at random. */
     if (numtypes) {
 	u = type[xrandom(numtypes)];
 	DMprintf("%s can't build colonizers, building %s instead\n",
 		 unit_desig(unit), u_type_name(u));
-	push_build_task(unit, u, 1);
+	push_build_task(unit, u, 1, 0, 0);
 	return;
 	/* Nothing can be built. */
-    } else	DMprintf("Sorry, %s can't build anything!\n", unit_desig(unit));
+    } else {
+	DMprintf("Sorry, %s can't build anything!\n", unit_desig(unit));
+    }
 }
-#endif
 
 static void
 plan_improve(Unit *unit)
 {
-    int 	type[MAXUTYPES];
-    int 	numtypes = 0;
-    int 	u;
+    int u, type[MAXUTYPES], numtypes = 0;
 
-	/* Load all immobile types (facilities) into type vector. */
+    /* Execute the task if we have one. */
+    if (unit->plan->tasks) {
+    	execute_task(unit);
+    	return;
+    }
+#if 0
+    /* Load all immobile types (facilities) into type vector. */
     for_all_unit_types(u) {
 	if (type_allowed_on_side(u, unit->side) 
 	    /* No longer included in type_allowed_on_side. */
@@ -1037,7 +1055,7 @@ plan_improve(Unit *unit)
 	u = type[xrandom(numtypes)];
 	DMprintf("%s improving itself by building %s\n",
 		 unit_desig(unit), u_type_name(u));
-	push_build_task(unit, u, 1);
+	push_build_task(unit, u, 1, 0, 0);
 	return;
 	/* Load all possible types instead if we failed. */
     } else for_all_unit_types(u) {
@@ -1054,65 +1072,69 @@ plan_improve(Unit *unit)
 	u = type[xrandom(numtypes)];
 	DMprintf("%s can't build facility, building %s instead\n",
 		 unit_desig(unit), u_type_name(u));
-	push_build_task(unit, u, 1);
+	push_build_task(unit, u, 1, 0, 0);
 	return;
 	/* Nothing can be built. */
-    } else	DMprintf("Sorry, %s can't build anything!\n", unit_desig(unit));
+    } else
+      DMprintf("Sorry, %s can't build anything!\n", unit_desig(unit));
+#endif
 }
 
 static int 
 good_cell_to_colonize(int u, Unit *unit)
 {
-	int 	x1, y1, m, supply[MAXMTYPES], maxsize;
-	int	x = unit->x;
-	int	y = unit->y;
-	Unit	*unit2;
+    int x1, y1, m, supply[MAXMTYPES], maxsize;
+    int x = unit->x;
+    int y = unit->y;
+    Unit *unit2;
 
-	/* Check for nexthere prevents two colonizers from trying to colonize the same 
-	cell and blocking each other from doing so again and again due to terrain capacity 
-	restrictions. */
-	if (unit->nexthere != NULL)
-		return FALSE;
+    /* Check for nexthere prevents two colonizers from trying to
+       colonize the same cell and blocking each other from doing so
+       again and again due to terrain capacity restrictions. */
+    if (unit->nexthere != NULL)
+      return FALSE;
 
-	/* Then check if another advanced unit is too close to this cell. */
-	for_all_units(unit2) {
-		if (u_advanced(unit2->type)
-		    && distance(x, y, unit2->x, unit2->y) 
-		          < g_ai_advanced_unit_separation())
-		    	return FALSE;
+    /* Then check if another advanced unit is too close to this cell. */
+    for_all_units(unit2) {
+	if (u_advanced(unit2->type)
+	    && (distance(x, y, unit2->x, unit2->y)
+		< g_ai_advanced_unit_separation()))
+	  return FALSE;
+    }
+    /* Then check that the minimal size goal can be attained for all
+       possible advanced units (even if we start with a village it may
+       eventually grow into a metropolis). */
+    for_all_unit_types(u) {
+	if (!u_advanced(u))
+	  continue;
+	maxsize = PROPHI;
+
+	/* First zero the supply vector. */
+	for_all_material_types (m)
+	  supply[m] = 0;
+	/* Go through all cells within reach and compute supply of
+           materials. */
+	for_all_cells_within_range(x, y, u_reach(u), x1, y1) {
+	    if (!inside_area(x1, y1))
+	      continue;
+	    /* Don't count supply in cells already used by others. */
+	    if (user_at(x1, y1) != NOUSER)
+	      continue;
+	    /* Add the supply of materials from this cell. */
+	    for_all_material_types (m)
+	      supply[m] += production_at(x1, y1, m); 
 	}
-	/* Then check that the minimal size goal can be attained for all possible advanced 
-	units (even if we start with a village it may eventually grow into a metropolis). */
-	for_all_unit_types(u) {
-		if (!u_advanced(u))
-			continue;
-		maxsize = PROPHI;
-
-		/* First zero the supply vector. */
-		for_all_material_types (m)
-			supply[m] = 0;
-		/* Go through all cells within reach and compute supply of materials. */
-		for_all_cells_within_range(x, y, u_reach(u), x1, y1) {
-			if (!inside_area(x1, y1))
-				continue;
-			/* Don't count supply in cells already used by others. */
-			if (user_at(x1, y1) != NOUSER)
-				continue;
-			/* Add the supply of materials from this cell. */
-			for_all_material_types (m)
-				supply[m] += tm_production(terrain_at(x1, y1), m); 
-		}
-		/* Calculate max sustainable city size at this location. */
-		for_all_material_types (m) {
-			if (um_consumption_per_size(u, m) > 0)
-				maxsize = min(maxsize, supply[m] / um_consumption_per_size(u, m));
-		}
-		/* Return FALSE if there is not enough food around 
-		to sustain the minimal size goal for this unit type. */
-		if (maxsize < u_minimal_size_goal(u))
-			return FALSE;
+	/* Calculate max sustainable city size at this location. */
+	for_all_material_types (m) {
+	    if (um_consumption_per_size(u, m) > 0)
+	      maxsize = min(maxsize, supply[m] / um_consumption_per_size(u, m));
 	}
-	return TRUE;
+	/* Return FALSE if there is not enough food around to sustain
+	   the minimal size goal for this unit type. */
+	if (maxsize < u_minimal_size_goal(u))
+	  return FALSE;
+    }
+    return TRUE;
 }
 
 static void
@@ -1262,7 +1284,7 @@ Unit *unit;
     	if (task == NULL || task->type != TASK_BUILD) {
 	    DMprintf("%s supporting exploration by building %s\n",
 		     unit_desig(unit), u_type_name(u3));
-	    push_build_task(unit, u3, 2);
+	    push_build_task(unit, u3, 2, 0, 0);
 	} else {
 	    DMprintf("%s already building, leaving alone\n",
 		     unit_desig(unit));
@@ -1957,7 +1979,12 @@ plan_random(Unit *unit)
 	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_BUILD:
-	    task = create_build_task(unit, xrandom(numutypes), xrandom(99));
+	    if (flip_coin())
+	      random_point_near(unit->x, unit->y, 4, &x1, &y1);
+	    else
+	      random_point(&x1, &y1);
+	    task = create_build_task(unit, xrandom(numutypes), xrandom(99),
+				     x1, y1);
 	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_CAPTURE:
@@ -1977,10 +2004,6 @@ plan_random(Unit *unit)
 	  case TASK_DISBAND:
 	    set_disband_task(unit);
 	    break;
-	  case TASK_DO_ACTION:
-	    task = create_task(tasktype);
-	    add_task(unit, CLEAR_AGENDA, task);
-	    break;
 	  case TASK_HIT_POSITION:
 	    task = create_task(tasktype);
 	    if (flip_coin())
@@ -2159,20 +2182,6 @@ int flag;
       unit->side->numwaiting += (unit->plan->waitingfortasks ? 1 : -1);
 }
 
-/* Put a unit in reserve. */
-
-void
-reserve_unit(side, unit)
-Side *side;
-Unit *unit;
-{
-    if (unit->plan) {
-	unit->plan->reserve = TRUE;
-	set_waiting_for_tasks(unit, FALSE);
-	update_unit_display(side, unit, TRUE);
-    }
-}
-
 /* General routine to wake a unit up (and maybe all its cargo). */
 
 void
@@ -2539,10 +2548,7 @@ Unit *unit;
 /* Clear a unit's plan out. */
 
 void
-force_replan(side, unit, passive_only)
-Side *side;
-Unit *unit;
-int passive_only;
+force_replan(Side *side, Unit *unit, int passive_only)
 {
     extern int need_ai_planning;
 
@@ -2737,6 +2743,18 @@ Unit *unit;
     return FALSE;
 }
 
+int
+can_research(unit)
+Unit *unit;
+{
+	int a;
+
+	/* Kind of crude, but works for now. */
+	if (u_advanced(unit->type))
+	    return TRUE;
+	return FALSE;
+}
+
 int
 can_produce(unit)
 Unit *unit;
@@ -2943,7 +2961,6 @@ int u, u2;
       (uu_cp_per_build(u, u2) * u_acp(u));
 }
 
-
 /* True if anybody at all is on any adjacent cell. */
 
 int
