@@ -1113,7 +1113,7 @@ char *filename;
 			    for_all_directions(d) {
 				if (border_at(x, y, d, t) &&
 				    (!side
-				     || all_see_all
+				     || g_see_all()
 				     || seen_border(side, x, y, d))) {
 				    fprintf(fp, "%d %g %d %g li\n",
 					    cx + hexc_x[d],
@@ -1148,7 +1148,7 @@ char *filename;
 			if (t_is_connection(t) && aux_terrain_defined(t)) {
 			    for_all_directions(d) {
 				if (connection_at(x, y, d, t) &&
-				    (!side || all_see_all ||
+				    (!side || g_see_all() ||
 				     terrain_view(side, x, y) != UNSEEN)) {
 				    fprintf(fp, "%d %g %d %g li\n",
 					    cx,
