@@ -10,10 +10,6 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 extern int lookup_action_type(char *name);
-extern void interp_utype_list(short *arr, Obj *lis);
-extern void interp_short_array(short *arr, Obj *lis, int n);
-extern void interp_long_array(long *arr, Obj *lis, int n);
-extern int eval_number(Obj *val, int *isnumber);
 #include "kernel.h"
 #include "imf.h"
 
