@@ -1,5 +1,5 @@
 /* The main simulation-running code in Xconq.
-   Copyright (C) 1986-1989, 1991-1999 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,6 +9,7 @@ any later version.  See the file COPYING.  */
 /* This is the main simulation-running code. */
 
 #include "conq.h"
+extern int production_at(int x, int y, int m);
 #include "kernel.h"
 #include "kpublic.h"
 #include "ai.h"
@@ -225,6 +226,9 @@ run_game(int maxactions)
     } else if (paused) {
 	/* Don't do anything if we're paused. */
     	Dprintf("run_game: paused.\n");
+    } else if (numdesigners > 0) {
+	/* Don't do anything if any designers are around. */
+    	Dprintf("run_game: designing.\n");
     } else if (!beforestart) {
 	if (at_turn_start) {
 	    if (midturnrestore)
@@ -531,6 +535,13 @@ units_still_acting(Side *side)
 	if (unit_still_acting(unit) && side_controls_unit(side, unit))
 	  return TRUE;
     }
+    /* Try not to let a side not be researching something. */
+    if (numatypes > 0
+	&& g_side_can_research()
+	&& side->research_topic == NOADVANCE
+	/* (should also test that any advances are available) */
+	)
+      return TRUE;
     return FALSE;
 }
 
@@ -1278,12 +1289,11 @@ extern void run_people(void);
 
 extern void run_advanced_units(void);
 extern void free_used_cells(Unit *unit);
-extern void unit_advance_dialog(Unit *unit);
-extern void global_advance_dialog(Side *side, Unit *unit, int advance);
+extern void unit_research_dialog(Unit *unit);
+extern void side_research_dialog(Side *side, Unit *unit, int advance);
 extern void auto_pick_new_research(Unit *unit);
 
 extern void unit_plan_dialog(Unit *unit);
-extern void net_resume_build_task(Unit *unit, Unit *unit2, int run);
 extern int has_advance_to_research(Side *side, int a);
 extern int select_by_weight(int *arr, int numvals);
 extern int unit_build_dialog(Unit *unit);
@@ -1367,8 +1377,8 @@ run_advanced_units()
 	    if (!inside_area(x, y))
 	  	continue;
 	    if (user_at(x, y) == unit->id)
-		for_all_material_types (m)
-		    unit->production[m] += tm_production(terrain_at(x, y), m);
+	      for_all_material_types(m)
+		unit->production[m] += production_at(x, y, m);
 	}
 	/* Support for city improvements. */
 
@@ -1405,6 +1415,35 @@ run_advanced_units()
 		  if (unit->side->advance[a] == DONE)
 		    unit->production[m] *= (float) am_adv_mult_production(a, m) / 100;
 	    }
+	    {
+		int m2, anyconversion = FALSE, incrs[MAXMTYPES], tot = 0;
+
+		for_all_material_types(m2) {
+		    incrs[m2] = 0;
+		    if (mm_conversion(m, m2) > 0
+			&& unit->side->c_rates != NULL
+			&& unit->side->c_rates[m2] > 0) {
+			anyconversion = TRUE;
+			tot += unit->side->c_rates[m2];
+		    }
+		}
+		while (anyconversion && unit->production[m] > 0) {
+		    for_all_material_types(m2) {
+			if (mm_conversion(m, m2) > 0
+			    && unit->side->c_rates != NULL
+			    && unit->side->c_rates[m2] > 0) {
+			    incrs[m2] += unit->side->c_rates[m2];
+			}
+		    }
+		    for_all_material_types(m2) {
+			if (incrs[m2] >= tot && unit->production[m] > 0) {
+			    incrs[m2] -= tot;
+			    unit->supply[m2] += 1;
+			    unit->production[m] -= 1;
+			}
+		    }
+		}
+	    }
 	    /* Finally add production to unit supply. */
 	    unit->supply[m] += unit->production[m];
 	    /* Borrow material from treasury if it exists. */
@@ -1424,16 +1463,19 @@ run_advanced_units()
 	    allocate_used_cells(unit);
 	    /* Also reflect any changes of the city size text. */
 	    for_all_sides(side) {
-		if (side->see_all || side_sees_unit(side, unit) 
+		if (side->see_all
+		    || side_sees_unit(side, unit) 
 		    || side_tracking_unit(side, unit)) {
 		    update_cell_display(side, unit->x, unit->y, UPDATE_ALWAYS);
 		}
 	    }
 	}
 	/* Run building tasks. */
-	run_construction(unit);
-	/* Do the research. */		
-	run_research(unit);
+	if (u_advanced_auto_construct(unit->type))
+	  run_construction(unit);
+	/* Do individual unit research. */
+	if (u_advanced_auto_research(unit->type))
+	  run_research(unit);
 	/* Return any borrowed material to treasury. */
 	for_all_material_types(m) {
 	    if (side_has_treasury(unit->side, m)
@@ -1445,6 +1487,25 @@ run_advanced_units()
     }
 }
 
+/* Compute the total production of the given material at the given
+   location within a city radius. */
+
+int
+production_at(int x, int y, int m)
+{
+    int prod, t;
+
+    prod = tm_production(terrain_at(x, y), m);
+    for_all_terrain_types(t) {
+	if (t_is_connection(t)
+	    && aux_terrain_defined(t)
+	    && any_connections_at(x, y, t)
+	    && tm_production(t, m) > 0)
+	  prod += tm_production(t, m);
+    }
+    return prod;
+}
+
 /* Starve or grow city population depending on food supply. */
 
 void
@@ -1453,7 +1514,8 @@ run_population(Unit *unit)
     int starved = FALSE;
     int u = unit->type;
     int a, m, u2;
-	
+    Side *side = unit->side;
+
     /* First consume necessary materials (food) in proportion to unit
        size. */
     for_all_material_types(m) {
@@ -1468,23 +1530,23 @@ run_population(Unit *unit)
     /* Starvation! */
     if (starved) {
 	/* Shrink size by one. */
-	unit->size -=1;
+	unit->size -= 1;
 	/* Run warning. */
 	if (unit->size > 0) {
-	    notify(unit->side, "The people in your %s %s are starving!", 
-	    				u_type_name(u), unit->name);			
+	    notify(side, "The people in %s are starving!",
+		   unit_handle(side, unit));
 	} else {
 	    /* Kill the unit, but first return borrowed material to
                treasury. */
 	    for_all_material_types(m) {
-	    	if (side_has_treasury(unit->side, m)
+	    	if (side_has_treasury(side, m)
 	    	    && um_gives_to_treasury(unit->type, m)) {
-		    unit->side->treasury[m] += unit->supply[m];
+		    side->treasury[m] += unit->supply[m];
 		    unit->supply[m] = 0; 			
 		}
 	    }
-	    notify(unit->side, "Your %s %s is no more. The people starved to death.", 
-	    				u_type_name(u), unit->name);			
+	    notify(side, "%s is no more. The people starved to death.", 
+		   unit_handle(side, unit));
 	    kill_unit(unit, H_UNIT_STARVED);
 	}
 	/* Done with handling starvation. */
@@ -1497,10 +1559,10 @@ run_population(Unit *unit)
 	return;
     /* Then check if we have the required advances for the next size. */
     for_all_advance_types(a) {
-	if (unit->side->advance[a] != DONE 
+	if (side->advance[a] != DONE 
 	    && ua_size_limit_without_adv(u, a) <= unit->size) {
-	    notify(unit->side, "Your %s %s needs %s advance to grow in size.", 
-		   			u_type_name(u), unit->name, a_type_name(a));			
+	    notify(side, "%s needs %s advance to grow in size.", 
+		   unit_handle(side, unit), a_type_name(a));
 	    return;
 	}
     }
@@ -1508,7 +1570,7 @@ run_population(Unit *unit)
     for_all_unit_types(u2) {
 	if (uu_size_limit_without_occ(u, u2) <= unit->size) {
 	    int	hasocc = FALSE;
-	    Unit	*unit2;
+	    Unit *unit2;
 
 	    for_all_occupants(unit, unit2) {			
 		if (unit2->type == u2) {
@@ -1517,21 +1579,30 @@ run_population(Unit *unit)
 		}
 	    }
 	    if (!hasocc) {
-		notify(unit->side, "Your %s %s needs a %s to grow in size.", 
-		       			   u_type_name(u), unit->name, u_type_name(u2));			
+		notify(side, "%s needs a %s to grow in size.", 
+		       unit_handle(side, unit), u_type_name(u2));
 		return;
 	    }
 	}
     }				    	
     /* If we got this far we are ready for a size increase! */
-    for_all_material_types(m)
+    for_all_material_types(m) {
+      int consump;
+      Unit *occ;
+
+      consump = unit->size * um_consumption_to_grow(u, m);
+      for_all_occupants(unit, occ) {
+	if (strcmp(u_type_name(occ->type), "Granary") == 0)
+	  consump /= 2;
+      }
       /* Use up more supplies as required to prepare growth. */
-      unit->supply[m] -= unit->size * um_consumption_to_grow(u, m);
+      unit->supply[m] -= consump;
+    }
     /* Then increase the size. */
     unit->size += 1;
     /* And bragg about it. */
-    notify(unit->side, "Your %s %s prospers and reaches size %d.", 
-		   		u_type_name(u), unit->name, unit->size);			
+    notify(side, "%s prospers and reaches size %d.", 
+	   unit_handle(side, unit), unit->size);			
 }
 
 void
@@ -1556,6 +1627,9 @@ run_construction(Unit *unit)
 	else
 	  auto_pick_new_plan(unit);
     }
+    /* Plan creation still failing, get out. */
+    if (unit->plan == NULL)
+      return;
 
     while (builds) {
 	Unit *unit2 = NULL;
@@ -1584,7 +1658,7 @@ run_construction(Unit *unit)
 	    }
 	    /* Resume building of incomplete occupant. */
 	    if (is_unit(unit2)){
-		net_resume_build_task(unit, unit2, 1);
+		net_resume_build_task(unit, unit2, 1, 0, 0);
 		u2 = unit2->type;
 	    } else {
 		/* Else select a new unit type to build. */
@@ -1690,7 +1764,7 @@ run_research(Unit *unit)
 	if (side_has_display(unit->side)
 	    && !side_has_ai(unit->side)
 	    && !unit->autoresearch)
-	  unit_advance_dialog(unit);
+	  unit_research_dialog(unit);
 	/* Pick new research automatically for AIs. */
 	else
 	  auto_pick_new_research(unit);
@@ -1736,7 +1810,7 @@ run_research(Unit *unit)
 		}
 		/* Run global advance dialog for idle non-auto units. */
 		if (idle)
-		  global_advance_dialog(unit->side, unit, a);
+		  side_research_dialog(unit->side, unit, a);
 	    } else {
 		for_all_side_units(unit->side, unit2) {
 		    if (!u_advanced(unit2->type))
@@ -1764,6 +1838,8 @@ allocate_used_cells(Unit *unit)
 
     /* Set max used cells to unit size. */
     unit->maxcells = unit->size;
+    if (u_use_own_cell(u))
+      ++(unit->maxcells);
     /* First compute additive effect of each occupant/facility. */
     for_all_occupants(unit, unit2)
       unit->maxcells += uu_occ_add_maxcells(u, unit2->type);
@@ -1794,7 +1870,7 @@ allocate_used_cells(Unit *unit)
 	}
     }
 
-    /* Free all cells within reach */
+    /* Free all of this unit's doubtfulcells within reach */
     for_all_cells_within_reach(unit, x, y) {
 	/* Skip if not inside area */
 	if (!inside_area(x, y))
@@ -1802,7 +1878,7 @@ allocate_used_cells(Unit *unit)
 	/* Free cells used by the unit itself if it is independent or
            played by an AI */
 	if (user_at(x, y) == unit->id
-	    && (!unit->side || side_has_ai(unit->side))) {
+	    && (indep(unit) || side_has_ai(unit->side))) {
 	    set_user_at(x, y, NOUSER);
 	    unit->usedcells -= 1;
 	} else if (user_at(x, y) != NOUSER && unit_at(x, y) != NULL) {
@@ -1816,12 +1892,14 @@ allocate_used_cells(Unit *unit)
 	    }
 	}
     }
-
+    /* May want to use own cell automatically. */
+    if (u_use_own_cell(u) && user_at(unit->x, unit->y) == NOUSER) {
+	set_user_at(unit->x, unit->y, unit->id);
+	unit->usedcells += 1;
+    }
     /* Then select new cells up to unit size */
     for (i = unit->usedcells + 1; i <= unit->maxcells; i++) {
-	int tmpprod = 0;
-	int tmpx = unit->x;
-	int tmpy = unit->y;
+	int prod, tmpprod = 0, tmpx = unit->x, tmpy = unit->y;
 
 	for_all_cells_within_reach(unit, x, y) {
 	    /* Skip if not inside area */
@@ -1848,14 +1926,13 @@ allocate_used_cells(Unit *unit)
 	    /* Make this tmp used cell if it produces more limiting
 	       material than previous tmp used cell or if the previous
 	       tmp cell is already in use. */
-	    if (tm_production(terrain_at(x, y), mlim) > tmpprod
-		|| user_at(tmpx, tmpy) != NOUSER) {
-		tmpprod = tm_production(terrain_at(x, y), mlim);
-		tmpx = x;
-		tmpy = y;
+	    prod = production_at(x, y, mlim);
+	    if ((prod > tmpprod) || (user_at(tmpx, tmpy) != NOUSER)) {
+		tmpprod = prod;
+		tmpx = x;  tmpy = y;
 	    }
 	}
-	/* Make tmp used cell permanently used if it is unused */
+	/* Make tmp used cell permanently used if it is unused. */
 	if (user_at(tmpx, tmpy) == NOUSER) {
 	    set_user_at(tmpx, tmpy, unit->id);
 	    unit->usedcells += 1;
@@ -2048,7 +2125,7 @@ auto_pick_new_build_task(Unit *unit)
 	/* Use prefs weight to pick unit type to build. */
 	u = select_by_weight(prefs, numutypes);
 	if (is_unit_type(u))
-	  net_set_build_task(unit, u, 1); 
+	  net_set_build_task(unit, u, 1, 0, 0); 
 	else
 	  u = NONUTYPE;
     } else
