@@ -1,5 +1,5 @@
 /* The main simulation-running code in Xconq, turn start and end calcs.
-   Copyright (C) 1986-1989, 1991-1999 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -70,6 +70,7 @@ static void damage_unit_with_temperature(Unit *unit, int n);
 static void run_people_side_changes(void);
 static void run_appearances(void);
 static void run_disappearances(void);
+static void run_side_research(void);
 static void run_hp_recovery(void);
 static void run_auto_repair(void);
 static void auto_repair_from_here(int x, int y);
@@ -235,13 +236,14 @@ run_turn_start(void)
     run_environment();
     /* Optionally run population, economy and supply code. */
     if (g_people())
-	run_people();
+      run_people();
     if (g_economy())
-	run_economy();
+      run_economy();
     if (g_supply())
-	run_supply();
+      run_supply();
     /* Advanced unit support. */
     run_advanced_units();
+    run_side_research();
     run_hp_recovery();
     run_auto_repair();
     run_self_builds();
@@ -899,7 +901,7 @@ compute_sun(void)
 /* Compute the position of the sun for this turn. */
 
 static void
-run_sun()
+run_sun(void)
 {
     int x, y;
     Side *side;
@@ -932,8 +934,7 @@ static void calc_key_point_temps(int yearpart);
 static int interpolate_temperature(int x, int y);
 
 static void
-calc_key_point_temps(yearpart)
-int yearpart;
+calc_key_point_temps(int yearpart)
 {
     int i, err, rslt;
     Obj *lis, *item, *loc;
@@ -962,49 +963,40 @@ int yearpart;
 }
 
 static int
-interpolate_temperature(x, y)
-int x, y;
+interpolate_temperature(int x, int y)
 {
-    int d0, d1, t0, t1, i, besti, nextbesti, d;
+    int dbest, dnextbest, tbest, tnextbest, i, besti, nextbesti, d;
 
-    if (num_key_points == 1) {
-	return key_points[0].temp;
-    } else if (num_key_points == 2) {
-	d0 = distance(x, y, key_points[0].x, key_points[0].y);
-	t0 = key_points[0].temp;
-	d1 = distance(x, y, key_points[1].x, key_points[1].y);
-	t1 = key_points[1].temp;
-	return ((t0 * d1 + t1 * d0) / (d0 + d1));
-    } else if (num_key_points > 2) {
-	besti = 0;
-	nextbesti = -1;
-	d0 = distance(x, y, key_points[besti].x, key_points[besti].y);
-	for (i = 2; i < num_key_points; ++i) {
-	    d = distance(x, y, key_points[i].x, key_points[i].y);
-	    if (d < d0) {
-		nextbesti = besti;
-		d1 = d0;
-		besti = i;
-		d0 = distance(x, y, key_points[besti].x, key_points[besti].y);
-	    } else if ((nextbesti < 0 || d < d1) && i != besti) {
-		nextbesti = i;
-		d1 = distance(x, y, key_points[nextbesti].x, key_points[nextbesti].y);
-	    }
+    if (num_key_points == 1)
+      return key_points[0].temp;
+    /* Find the closest and next-closest key points. */
+    besti = 0;
+    dbest = distance(x, y, key_points[besti].x, key_points[besti].y);
+    nextbesti = -1;
+    for (i = 1; i < num_key_points; ++i) {
+	d = distance(x, y, key_points[i].x, key_points[i].y);
+	if (d < dbest) {
+	    nextbesti = besti;
+	    dnextbest = dbest;
+	    besti = i;
+	    dbest = d;
+	} else if (nextbesti < 0 || d < dnextbest) {
+	    nextbesti = i;
+	    dnextbest = d;
 	}
-	d1 = distance(x, y, key_points[nextbesti].x, key_points[nextbesti].y);
-	t0 = key_points[besti].temp;
-	t1 = key_points[nextbesti].temp;
-	return ((t0 * d1 + t1 * d0) / (d0 + d1));
-    } else {
-	run_error("???");
-	return 0;
     }
+#if 0
+    return ((besti + 1) * 100 + (dbest == 0 ? 0 : (nextbesti + 1)));
+#endif
+    tbest = key_points[besti].temp;
+    tnextbest = key_points[nextbesti].temp;
+    return ((tbest * dnextbest + tnextbest * dbest) / (dnextbest + dbest));
 }
 
 /* Compute environment changes. */
 
 static void
-run_environment()
+run_environment(void)
 {
     int yrlen = world.yearlength, x, y, dir, t, celltemp;
     int anychanges = FALSE;
@@ -1043,9 +1035,10 @@ run_environment()
 	    set_tmp2_at(x, y, temperature_at(x, y));
 	    t = terrain_at(x, y);
 	    if (any_annual_temp_change)
-		celltemp = interpolate_temperature(x, y);
+	      celltemp = interpolate_temperature(x, y);
 	    else
-		celltemp = t_temp_avg(t);
+	      celltemp = t_temp_avg(t);
+#if 1
 	    /* Add in a random variation if specified. */
 	    if (t_temp_variability(t) > 0) {
 		celltemp += (xrandom(t_temp_variability(t))
@@ -1063,9 +1056,10 @@ run_environment()
 	    }
 	    /* Clip to terrain type's limits. */
 	    if (celltemp < t_temp_min(t))
-		celltemp = t_temp_min(t);
+	      celltemp = t_temp_min(t);
 	    if (celltemp > t_temp_max(t))
-		celltemp = t_temp_max(t);
+	      celltemp = t_temp_max(t);
+#endif
 	    /* Record the (unmoderated) temperature of the cell. */
 	    set_temperature_at(x, y, celltemp);
 	}
@@ -1098,6 +1092,54 @@ run_environment()
 		set_tmp1_at(x, y, TRUE);
 	    anychanges = TRUE;
 	}
+	if (numcoattypes > 0) {
+	    for_all_terrain_types(t) {
+		if (t_subtype(t) == coatingsubtype) {
+		    allocate_area_aux_terrain(t);
+		    /* (should make more generic) */
+		    if (strcmp(t_type_name(t), "snow") == 0) {
+			for_all_cells(x, y) {
+			    int olddepth = aux_terrain_at(x, y, t), newdepth;
+			    if (temperature_at(x, y) <= 3) {
+				newdepth = 1;
+			    } else {
+				newdepth = 0;
+			    }
+			    if (newdepth > tt_coat_max(t, terrain_at(x, y)))
+			      newdepth = tt_coat_max(t, terrain_at(x, y));
+			    if (newdepth < tt_coat_min(t, terrain_at(x, y)))
+			      newdepth = tt_coat_min(t, terrain_at(x, y));
+			    if (newdepth != olddepth) {
+				set_aux_terrain_at(x, y, t, newdepth);
+				update_cell_display_all_sides(x, y,
+							      UPDATE_ALWAYS
+							      | UPDATE_ADJ);
+			    }
+			}
+		    }
+		    if (strcmp(t_type_name(t), "mud") == 0) {
+			for_all_cells(x, y) {
+			    int olddepth = aux_terrain_at(x, y, t), newdepth;
+			    if (between(4, temperature_at(x, y), 9)) {
+				newdepth = 1;
+			    } else {
+				newdepth = 0;
+			    }
+			    if (newdepth > tt_coat_max(t, terrain_at(x, y)))
+			      newdepth = tt_coat_max(t, terrain_at(x, y));
+			    if (newdepth < tt_coat_min(t, terrain_at(x, y)))
+			      newdepth = tt_coat_min(t, terrain_at(x, y));
+			    if (newdepth != olddepth) {
+				set_aux_terrain_at(x, y, t, newdepth);
+				update_cell_display_all_sides(x, y,
+							      UPDATE_ALWAYS
+							      | UPDATE_ADJ);
+			    }
+			}
+		    }
+		}
+	    }
+	}
     }
     if (any_clouds /* and any cloud changes */) {
 	/* Save the previous state. */
@@ -1791,23 +1833,20 @@ int x, y;
 }
 
 static void
-try_sharing_with_cell(x, y)
-int x, y;
+try_sharing_with_cell(int x, int y)
 {
     Unit *unit2, *occ;
 
     for_all_stack(x, y, unit2) {
-	if (in_play(unit2) && unit2->side == tmpunit->side) {
+	if (in_play(unit2) && unit_trusts_unit(unit2, tmpunit)) {
 	    try_sharing_need(unit2, tmpunit, tmpmtype);
 	}
     }
     for_all_stack(x, y, unit2) {
-	if (in_play(unit2) && unit2->side == tmpunit->side) {
-
-    	/* Bug fix (same as above). */
-    	
+	if (in_play(unit2) && unit_trusts_unit(unit2, tmpunit)) {
+	    /* Bug fix (same as above). */
 	    for_all_occs_with_occs(unit2, occ) {
-		if (in_play(occ) && occ->side == tmpunit->side) {
+		if (in_play(occ) && unit_trusts_unit(occ, tmpunit)) {
 		    try_sharing_need(occ, tmpunit, tmpmtype);
 		}
 	    }
@@ -2026,7 +2065,8 @@ run_economy()
 		    if (dist >= 0) {
 			tmpunit = unit;
 			tmpmtype = m;
-			tmpneed = um_storage_x(u, m) / 2 - unit->supply[m];
+			/* Round up, to handle fractions usefully. */
+			tmpneed = (um_storage_x(u, m) + 1) / 2 - unit->supply[m];
 			search_and_apply(unit->x, unit->y, dist, need_left,
 					 &x1, &y1, 1, try_sharing_with_cell,
 					 0);
@@ -2568,10 +2608,59 @@ run_disappearances()
     }
 }
 
+/* Run any per-side research activities. */
+
+static void
+run_side_research(void)
+{
+    int a, m, consump, maxrp, rp, amt;
+    Side *side;
+
+    /* No research if there are no advances. */
+    if (numatypes == 0)
+      return;
+    if (!g_side_can_research())
+      return;
+    for_all_sides(side) {
+	a = side->research_topic;
+	/* Two different values that do nothing - one is player-specified,
+	   the other triggers a query to the player. */
+	if (a == NOADVANCE || a == NONATYPE)
+	  continue;
+	/* Compute the maximum number of rps that our material limits
+	   allow us to add. */
+	maxrp = 999999;
+	for_all_material_types(m) {
+	    consump = am_consumption_per_rp(a, m);
+	    if (consump > 0) {
+		rp = side->treasury[m] / consump;
+		maxrp = min(maxrp, rp);
+	    }
+	}
+	/* Add 1 rp per turn by default. */
+	if (maxrp == 999999)
+	  maxrp = 1;
+	side->advance[a] += maxrp;
+	if (side->advance[a] >= a_rp(a)) {
+	    side->advance[a] = DONE;
+	    notify(side, "Your wise men discover %s!", a_type_name(a));
+	    side->research_topic = NOADVANCE;
+	}
+	/* Consume the materials. */
+	for_all_material_types(m) {
+	    consump = am_consumption_per_rp(a, m);
+	    if (consump > 0) {
+		amt = maxrp * consump;
+		side->treasury[m] -= amt;
+	    }
+	}
+    }
+}
+
 /* Some types of units recover lost hp spontaneously. */
 
 static void
-run_hp_recovery()
+run_hp_recovery(void)
 {
     int u, hprecovery, hpmax, oldhp;
     Unit *unit;
