@@ -99,7 +99,7 @@ int next_doctrine_id;
 not be defined.  */
 
 void
-init_sides()
+init_sides(void)
 {
     /* Set up the list of sides. */
     sidelist = lastside = (Side *) xmalloc(sizeof(Side));
@@ -125,7 +125,7 @@ init_sides()
 /* Create an object representing a side. */
 
 Side *
-create_side()
+create_side(void)
 {
     int u, m;
     Side *newside;
@@ -191,6 +191,7 @@ create_side()
     if (numatypes > 0) {
    	 newside->advance = (short *) xmalloc(numatypes * sizeof(short));
     	 newside->numadvances = 0;
+	 newside->research_topic = NOADVANCE;
     }
     if (nummtypes > 0) {
 	/* Set up the side's supply of materials. */
@@ -198,6 +199,28 @@ create_side()
 	for_all_material_types(m)
 	     if (side_has_treasury(newside, m))
 	  newside->treasury[m] = m_initial_treasury(m);
+	/* Make space for default conversions and copy them over. */
+	newside->c_rates = (short *) xmalloc(nummtypes * sizeof(short));
+	{
+	    int m1, m2;
+	    
+	    /* First initilaize m. */
+	    m = NONMTYPE;
+	    /* For now, assume only one m. */
+	    for_all_material_types(m1) {
+		for_all_material_types(m2) {
+		    if (mm_conversion(m1, m2) > 0) {
+			m = m1;
+			break;
+		    }
+		}
+	    }
+	    if (m != NONMTYPE) {
+		for_all_material_types(m2) {
+		    newside->c_rates[m2] = mm_conversion_default(m, m2);
+		}
+	    }
+	}
     }
     /* Link in at the end of the list of sides. */
     newside->next = NULL;
@@ -216,8 +239,7 @@ create_side()
    as a head.  This unit should not be seen by any outside code. */
 
 void
-init_side_unithead(side)
-Side *side;
+init_side_unithead(Side *side)
 {
     if (side->unithead == NULL) {
 	side->unithead = create_bare_unit(0);
@@ -231,8 +253,7 @@ Side *side;
 /* This should be improved to not be fooled by dead units? */
 
 int
-side_has_units(side)
-Side *side;
+side_has_units(Side *side)
 {
     return (side->unithead != NULL
 	    && (side->unithead->next != side->unithead));
@@ -241,8 +262,7 @@ Side *side;
 /* Set up doctrine structures. */
 
 void
-init_doctrine(side)
-Side *side;
+init_doctrine(Side *side)
 {
     int u;
 
@@ -260,8 +280,7 @@ Side *side;
 }
 
 void
-init_self_unit(side)
-Side *side;
+init_self_unit(Side *side)
 {
     Unit *unit;
 
@@ -280,8 +299,7 @@ Side *side;
    side is created (during/after reading but before synthesis). */
 
 int
-init_view(side)
-Side *side;
+init_view(Side *side)
 {
     int terrainset = FALSE, t, u, m;
 
@@ -1457,6 +1475,12 @@ char *spec;
     }
 }
 
+void
+set_side_research(Side *side, int a)
+{
+    side->research_topic = a;
+}
+
 /* Vision. */
 
 /* What happens when a unit appears on a given cell. */
