@@ -97,7 +97,13 @@ typedef struct a_side {
     char *uorders;		/* bit vector of types that have orders */
     Obj *possible_units;	/* list of units to give to side */
     short ai_may_resign;	/* true if AI may draw or resign on its own */
-
+    short numadvances;		/* Number of completed advances */
+    short *advance;		/* State of research (accum pts) for each */ 
+    				/* advance. Set to -1 when completed. */
+    short research_topic;	/* Advance that the side is working on. */
+    short *treasury;		/* Globally available supply of each
+				   material type. */
+    short *c_rates;		/* Material conversion rates. */
     /* View layers. */
     char *terrview;		/* ptr to view of terrain */
     char **auxterrview;		/* ptr to view of aux terrain */
@@ -185,11 +191,6 @@ typedef struct a_side {
     int point_value_cache;
     int point_value_valid;
     int prefixarg;		/* cache of UI prefixarg, for generic cmds */
-    short numadvances;		/* Number of completed advances */
-    short *advance;		/* State of research (accum pts) for each */ 
-    				/* advance. Set to -1 when completed. */ 
-    short *treasury;		/* Globally available supply of each
-				   material type. */
     /* Links. */
     struct a_side *next;	/* pointer to next in list */
 } Side;
