@@ -1,5 +1,5 @@
 /* A minimal interface to Xconq.
-   Copyright (C) 1991-1997, 1999 Stanley T. Shebs.
+   Copyright (C) 1991-1997, 1999-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,11 @@ any later version.  See the file COPYING.  */
 #include "print.h"
 #include "cmdline.h"
 
+extern void auto_pick_new_research(Unit *unit);
+extern int auto_pick_new_build_task(Unit *unit);
+extern void auto_pick_new_plan(Unit *unit);
+void unit_research_dialog(Unit *unit);
+void side_research_dialog(Side *side, Unit *unit, int s);
 void unit_advance_dialog(Unit *unit);
 void global_advance_dialog(Side *side, Unit *unit, int s);
 int unit_build_dialog(Unit *unit);
@@ -1130,6 +1135,18 @@ global_advance_dialog(Side *side, Unit *unit, int s)
     auto_pick_new_research(unit);
 }
 
+void
+unit_research_dialog(Unit *unit)
+{
+    auto_pick_new_research(unit);
+}
+
+void
+side_research_dialog(Side *side, Unit *unit, int s)
+{
+    auto_pick_new_research(unit);
+}
+
 int
 unit_build_dialog(Unit *unit)
 {
