@@ -1,5 +1,5 @@
 /* Unit task execution and general task functions.
-   Copyright (C) 1992-1999 Stanley T. Shebs.
+   Copyright (C) 1992-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,6 +21,14 @@ extern int target_visible(Unit *unit, Task *task);
 
 #define CLEAR_AGENDA 99
 
+enum choicestate {
+    eitherway,
+    leftthenright,
+    rightthenleft,
+    leftonly,
+    rightonly
+};
+
 static int compare_directions(const void *a0, const void *a1);
 static int test_for_buildable(int x, int y);
 static int collect_test(int x, int y);
@@ -28,6 +36,8 @@ static int extractable_test(int x, int y);
 static int repair_test(int x, int y);
 static int resupply_test(int x, int y);
 static TaskOutcome execute_task_aux(Unit *unit, Task *task);
+static TaskOutcome do_approach_subtask(Unit *unit, Task *task, int tx, int ty,
+				       short *statep);
 
 /* Declare all the task functions. */
 
@@ -128,16 +138,13 @@ Task *oldtask;
 /* The empty task always succeeds immediately. */
 
 static TaskOutcome
-do_none_task(unit, task)
-Unit *unit;
-Task *task;
+do_none_task(Unit *unit, Task *task)
 {
     return TASK_IS_COMPLETE;
 }
 
 static int
-test_for_buildable(x, y)
-int x, y;
+test_for_buildable(int x, int y)
 {
     Unit *unit;
 
@@ -153,16 +160,14 @@ int x, y;
     return FALSE;
 }
 
-/* The build task handles the develop, tooling up, creation, and completion
-   for a given number of units of a given type. */
+/* The build task handles the development, tooling up, creation, and
+   completion for a given number of units of a given type. */
 
 static TaskOutcome
-do_build_task(unit, task)
-Unit *unit;
-Task *task;
+do_build_task(Unit *unit, Task *task)
 {
-    int u = unit->type, dir, nx, ny, tp;
-    int x = unit->x, y = unit->y;
+    int u = unit->type, dir, tmpdir, nx, ny, tp, rslt;
+    int x = unit->x, y = unit->y, x2, y2;
     int u2 = task->args[0], run = task->args[3];
     Unit *unit2 = NULL;
     Side *us = unit->side;
@@ -170,8 +175,8 @@ Task *task;
     if (task->args[2] >= run) {
         return TASK_IS_COMPLETE;
     }
-
-    /* See if our technology needs improvement in order to build this type. */
+    /* See if our technology needs improvement in order to build this
+       type. */
     if (is_unit_type(u2)
 	&& u_tech_to_build(u2) > 0
         && us->tech[u2] < u_tech_to_build(u2)) {
@@ -183,7 +188,6 @@ Task *task;
 	    return TASK_FAILED;
         }
     }
-
     /* See if we need to toolup to work on this type. */
     if (is_unit_type(u2)) {
 	tp = (unit->tooling ? unit->tooling[u2] : 0);
@@ -216,23 +220,42 @@ Task *task;
 	    }
 	}
     }
-
-    /* Find something to work on */
+    /* Find something to work on. */
     unit2 = find_unit_to_complete(unit, task);
-
+    /* Try to pick up an optional location. */
+    x2 = task->args[4];  y2 = task->args[5];
+    if (!inside_area(x2, y2)) {
+	x2 = x;  y2 = y;
+    }
     /* No incomplete unit found, so try to create one. */
     if (unit2 == NULL) {
-	if (valid(check_create_in_action(unit, unit, u2, unit))) {
+	/* First try to create as occupant, but not if an explicit
+	   location has been supplied. */
+	if ((x2 == x && y2 == y)
+	    && valid((rslt = check_create_in_action(unit, unit, u2, unit)))) {
 	    prep_create_in_action(unit, unit, u2, unit);
 	    return TASK_PREPPED_ACTION;
-	} else if (valid(check_create_at_action(unit, unit, u2, x, y, 0))) {
-	    prep_create_at_action(unit, unit, u2, x, y, 0);
+	} else if (rslt == A_ANY_NO_MATERIAL) {
+	    /* Assume that the builder is accumulating necessary materials,
+	       and just wait. */
+	    set_unit_reserve(unit->side, unit, TRUE, FALSE);
+	    return TASK_IS_INCOMPLETE;
+	} else if (valid((rslt = check_create_at_action(unit, unit, u2,
+							x2, y2, 0)))) {
+	    prep_create_at_action(unit, unit, u2, x2, y2, 0);
 	    return TASK_PREPPED_ACTION;
+	} else if (rslt == A_ANY_NO_MATERIAL) {
+	    /* Assume that the builder is accumulating necessary materials,
+	       and just wait. */
+	    set_unit_reserve(unit->side, unit, TRUE, FALSE);
+	    return TASK_IS_INCOMPLETE;
 	} else {
-	    /* Try creating in an adjacent cell. */
-	    for_all_directions(dir) {
+	    /* As a fallback, try creating in an adjacent cell. */
+	    /* (should try any cell within create range) */
+	    for_all_directions_randomly(dir,tmpdir) {
 		if (interior_point_in_dir(x, y, dir, &nx, &ny)
-		    && valid(check_create_at_action(unit, unit, u2, nx, ny, 0))) {
+		    && valid(check_create_at_action(unit, unit, u2,
+						    nx, ny, 0))) {
 		    prep_create_at_action(unit, unit, u2, nx, ny, 0);
 		    return TASK_PREPPED_ACTION;
 		}
@@ -240,7 +263,7 @@ Task *task;
 	    return TASK_FAILED;
 	}
     } else {
-	/* Record the unit's id for use the next time around. */
+	/* Record the incomplete unit's id for use the next time around. */
 	task->args[1] = unit2->id;
     }
     /* We have an incomplete unit to work on, try to do a build action. */
@@ -254,9 +277,7 @@ Task *task;
 }
 
 Unit *
-find_unit_to_complete(unit, task)
-Unit *unit;
-Task *task;
+find_unit_to_complete(Unit *unit, Task *task)
 {
     Unit *occ;
     int u = unit->type, nx, ny, range;
@@ -305,9 +326,7 @@ Task *task;
    increasing technology. */
 
 static TaskOutcome
-do_develop_task(unit, task)
-Unit *unit;
-Task *task;
+do_develop_task(Unit *unit, Task *task)
 {
     int u = unit->type;
     int u2 = task->args[0], lev = task->args[1];
@@ -334,9 +353,7 @@ Task *task;
 /* This is to capture a given type/side of unit at a given place. */
 
 static TaskOutcome
-do_capture_task(unit, task)
-Unit *unit;
-Task *task;
+do_capture_task(Unit *unit, Task *task)
 {
     int u = unit->type, tx, ty, tu2, ts2, dist;
     Unit *unit2;
@@ -391,9 +408,7 @@ Task *task;
    actions; when the unit goes away, so will the task. */
 
 static TaskOutcome
-do_disband_task(unit, task)
-Unit *unit;
-Task *task;
+do_disband_task(Unit *unit, Task *task)
 {
     if (valid(check_disband_action(unit, unit))) {
 	prep_disband_action(unit, unit);
@@ -404,35 +419,8 @@ Task *task;
     }
 }
 
-/* This task just attempts to do the explicitly specified action repeatedly. */
-
 static TaskOutcome
-do_action_task(unit, task)
-Unit *unit;
-Task *task;
-{
-    int i;
-
-    if (task->args[0] > 0) {
-	--(task->args[0]);
-    	if (unit->act == NULL)
-    	  return TASK_FAILED;
-    	unit->act->nextaction.type = task->args[1];
-    	for (i = 0; i < MAXACTIONARGS; ++i) {
-	    unit->act->nextaction.args[i] = task->args[i + 2];
-    	}
-    	/* act on self always? */
-    	unit->act->nextaction.actee = unit->id;
-	return TASK_PREPPED_ACTION;
-    } else {
-	return TASK_IS_COMPLETE;
-    }
-}
-
-static TaskOutcome
-do_hit_position_task(unit, task)
-Unit *unit;
-Task *task;
+do_hit_position_task(Unit *unit, Task *task)
 {
     int u = unit->type, tx, ty, dist;
 
@@ -452,9 +440,7 @@ Task *task;
 }
 
 static TaskOutcome
-do_hit_unit_task(unit, task)
-Unit *unit;
-Task *task;
+do_hit_unit_task(Unit *unit, Task *task)
 {
     int u = unit->type, tx, ty, dist, movedist, tu, ts;
     Unit *unit2;
@@ -467,35 +453,43 @@ Task *task;
     dist = distance(tx, ty, unit->x, unit->y);
     if (dist <= 1 /* direct attack range */) {
 	if (can_attack(unit)) {
-	  for_all_stack(tx, ty, unit2) {
-	    if (!trusted_side(us, unit2->side)
-	    	&& (tu == NONUTYPE || unit2->type == tu)
-	    	/* (should check designated side to hit, if defined) */
-	    	) {
-		if (valid(check_attack_action(unit, unit, unit2, 100))
-		    && attack_can_damage_or_capture(unit, unit2)) {
-		    prep_attack_action(unit, unit, unit2, 100);
-		    return TASK_PREPPED_ACTION;
+	    for_all_stack(tx, ty, unit2) {
+		if (!trusted_side(us, unit2->side)
+		    && (tu == NONUTYPE || unit2->type == tu)
+		    /* (should check designated side to hit, if defined) */
+		    ) {
+		    if (valid(check_attack_action(unit, unit, unit2, 100))
+			&& attack_can_damage_or_capture(unit, unit2)) {
+			prep_attack_action(unit, unit, unit2, 100);
+			return TASK_PREPPED_ACTION;
+		    }
 		}
 	    }
-	  }
+	    /* Maybe we can overrun, but not if our cell is known to
+	       be clear of enemies. */
+	    if (dist > 0
+		&& valid(check_overrun_action(unit, unit, tx, ty, unit->z,
+					      100))) {
+		prep_overrun_action(unit, unit, tx, ty, unit->z, 100);
+		return TASK_PREPPED_ACTION;
+	    }
 	}
 	/* Might be able to fire at pointblank range. */
 	if (can_fire(unit) && dist >= u_range_min(u)) {
-	  for_all_stack(tx, ty, unit2) {
-	    if (!trusted_side(us, unit2->side)
-	    	&& (tu == NONUTYPE || unit2->type == tu)
-	    	/* (should check designated side to hit, if defined) */
-	    	) {
-		if (valid(check_fire_at_action(unit, unit, unit2, -1))
-		    /* Also count with capture when firing at
-                       pointblank range. */
-		    && fire_can_damage_or_capture(unit, unit2)) {
-		    prep_fire_at_action(unit, unit, unit2, -1);
-		    return TASK_PREPPED_ACTION;
+	    for_all_stack(tx, ty, unit2) {
+		if (!trusted_side(us, unit2->side)
+		    && (tu == NONUTYPE || unit2->type == tu)
+		    /* (should check designated side to hit, if defined) */
+		    ) {
+		    if (valid(check_fire_at_action(unit, unit, unit2, -1))
+			/* Also count with capture when firing at
+			   pointblank range. */
+			&& fire_can_damage_or_capture(unit, unit2)) {
+			prep_fire_at_action(unit, unit, unit2, -1);
+			return TASK_PREPPED_ACTION;
+		    }
 		}
 	    }
-	  }
 	}
 	return TASK_FAILED;
     }
@@ -805,12 +799,11 @@ target_visible(Unit *unit, Task *task)
     return FALSE;
 }
 
-/* Move in a straight line, go through things or stop rather than going around. */
+/* Move in a straight line, go through things or stop rather than
+   going around. */
 
 static TaskOutcome
-do_move_dir_task(unit, task)
-Unit *unit;
-Task *task;
+do_move_dir_task(Unit *unit, Task *task)
 {
     int dir, tx, ty;
     Unit *unit2;
@@ -840,25 +833,13 @@ Task *task;
     }
 }
 
-enum choicestate {
-    eitherway,
-    leftthenright,
-    rightthenleft,
-    leftonly,
-    rightonly
-};
-
-static TaskOutcome do_approach_subtask(Unit *unit, Task *task, int tx, int ty, short *statep);
-
 /* The move-to task is the main way for units to get from point A to
    point B.  In addition to the destination, the task has a required
    distance, so it will succeed if the unit is within that distance to
    the nominal destination. */
 
 static TaskOutcome
-do_move_to_task(unit, task)
-Unit *unit;
-Task *task;
+do_move_to_task(Unit *unit, Task *task)
 {
     int dist, tx, ty, check;
     Unit *unit2, *occ;
@@ -920,6 +901,8 @@ Task *task;
 		set_unit_reserve(unit->side, unit, TRUE, FALSE);
 		return TASK_IS_INCOMPLETE;
 	    }
+	    Dprintf("%s move action fails check, result is %s\n",
+		    unit_desig(unit), hevtdefns[check].name);
 	    return TASK_FAILED;
 	}
 	break;
@@ -942,11 +925,7 @@ Task *task;
 }
 
 static TaskOutcome
-do_approach_subtask(unit, task, tx, ty, statep)
-Unit *unit;
-Task *task;
-int tx, ty;
-short *statep;
+do_approach_subtask(Unit *unit, Task *task, int tx, int ty, short *statep)
 {
     int nx, ny, dirs[NUMDIRS], numdirs, i, numdirs2, check;
     Unit *unit2;
@@ -1124,9 +1103,7 @@ short *statep;
 }
 
 static TaskOutcome
-do_occupy_task(unit, task)
-     Unit *unit;
-     Task *task;
+do_occupy_task(Unit *unit, Task *task)
 {
     int dist;
     Unit *transport = find_unit(task->args[0]);
@@ -1156,9 +1133,7 @@ do_occupy_task(unit, task)
    forthcoming. */
 
 static TaskOutcome
-do_pickup_task(unit, task)
-Unit *unit;
-Task *task;
+do_pickup_task(Unit *unit, Task *task)
 {
     Unit *occupant = find_unit(task->args[0]);
 
@@ -1186,9 +1161,7 @@ Task *task;
 }
 
 static TaskOutcome
-do_produce_task(unit, task)
-Unit *unit;
-Task *task;
+do_produce_task(Unit *unit, Task *task)
 {
     int m, tot, sofar, amt;
 
@@ -1203,7 +1176,7 @@ Task *task;
       return TASK_IS_COMPLETE;
     amt = um_material_per_production(unit->type, m);
     if (valid(check_produce_action(unit, unit, m, amt))) {
-	task->args[1] += amt;
+	task->args[2] += amt;
 	prep_produce_action(unit, unit, m, amt);
 	return TASK_PREPPED_ACTION;
     }
@@ -1215,8 +1188,8 @@ Task *task;
 
 static int tmpm;
 
-Unit *collect_here(int x, int y);
-Unit *aux_collect_here(Unit *unit);
+static Unit *collect_here(int x, int y);
+static Unit *aux_collect_here(Unit *unit);
 
 static TaskOutcome
 do_collect_task(Unit *unit, Task *task)
@@ -1298,7 +1271,7 @@ collect_test(int x, int y)
     return (collect_here(x, y) != NULL);
 }
 
-Unit *
+static Unit *
 collect_here(int x, int y)
 {
     Unit *unit, *collector;
@@ -1311,7 +1284,7 @@ collect_here(int x, int y)
     return NULL;
 }
 
-Unit *
+static Unit *
 aux_collect_here(Unit *unit)
 {
     Unit *occ;
@@ -1355,9 +1328,7 @@ extractable_test(int x, int y)
 }
 
 static TaskOutcome
-do_repair_task(unit, task)
-Unit *unit;
-Task *task;
+do_repair_task(Unit *unit, Task *task)
 {
     int x, y, u = unit->type, m, range = area.maxdim;
     int ux = unit->x, uy = unit->y;
@@ -1398,8 +1369,7 @@ Task *task;
 }
 
 Unit *
-repair_here(x, y)
-int x, y;
+repair_here(int x, int y)
 {
     Unit *unit;
 
@@ -1595,8 +1565,7 @@ Task *task;
    had decided on that action). */
 
 TaskOutcome
-execute_task(unit)
-Unit *unit;
+execute_task(Unit *unit)
 {
     Plan *plan = unit->plan;
     TaskOutcome rslt;
@@ -1670,9 +1639,7 @@ Unit *unit;
 /* Perform a single given task. */
 
 static TaskOutcome
-execute_task_aux(unit, task)
-Unit *unit;
-Task *task;
+execute_task_aux(Unit *unit, Task *task)
 {
     TaskOutcome (*fn)(Unit *unit, Task *task);
 
@@ -1699,11 +1666,10 @@ Task *task;
    getting to the same place going either direction (we pick shortest). */
 
 int
-choose_move_dirs(unit, tx, ty, shortest, dirtest, dirsort, dirs)
-Unit *unit;
-int tx, ty, shortest, *dirs;
-int (*dirtest)(Unit *, int);
-void (*dirsort)(Unit *, int *, int);
+choose_move_dirs(Unit *unit, int tx, int ty, int shortest,
+		 int (*dirtest)(Unit *, int),
+		 void (*dirsort)(Unit *, int *, int),
+		 int *dirs)
 {
     int dx, dxa, dy, dist, d1, d2, d3, d4, axis = -1, hextant = -1;
     int numdirs = 0, shortestnumdirs;
@@ -1777,22 +1743,33 @@ void (*dirsort)(Unit *, int *, int);
    to move in. */
 
 int
-plausible_move_dir(unit, dir)
-Unit *unit;
-int dir;
+plausible_move_dir(Unit *unit, int dir)
 {
-    int u = unit->type, ux = unit->x, uy = unit->y, nx, ny, t, c, cost;
+    int u = unit->type, ux = unit->x, uy = unit->y, u2, nx, ny, t, c, cost;
+    int totcost, speed, maxmpavail;
 
     point_in_dir(ux, uy, dir, &nx, &ny);
     if (unit_at(nx, ny))
       return TRUE;
     t = terrain_at(nx, ny);
-    if ((ut_vanishes_on(u, t)
-         || ut_wrecks_on(u, t))
+    /* Deadly terrain with no bridges is always implausible. */
+    if ((ut_vanishes_on(u, t) || ut_wrecks_on(u, t))
         && !can_move_via_conn(unit, nx, ny))
       return FALSE;
+    /* Try the easy test first. */
+    /* (should reconsider - what if unit is slow due to damage?) */
     if (ut_mp_to_enter(u, t) <= u_acp(u))
       return TRUE;
+    u2 = (unit->transport ? unit->transport->type : NONUTYPE);
+    totcost = total_move_cost(u, u2, ux, uy, 0, nx, ny, 0);
+    speed = unit_speed(unit, nx, ny);
+    /* Check if unit can possibly have enough mp for the move. */
+    maxmpavail =
+      (((u_acp_max(u) > 0 ? u_acp_max(u) : u_acp(u)) - u_acp_min(u)) * speed)
+      / 100
+      + u_free_mp(u);
+    if (maxmpavail >= totcost)
+      return TRUE;
     if (numconntypes > 0) {
 	/* Try each connection type to see if it works. */
 	for_all_terrain_types(c) {
@@ -1819,8 +1796,7 @@ int ys[NUMDIRS];
 int terrs[NUMDIRS];
 
 static int
-compare_directions(a0, a1)
-CONST void *a0, *a1;
+compare_directions(CONST void *a0, CONST void *a1)
 {
     int i0, i1;
     int u = tmputype, t0, t1;
@@ -1957,99 +1933,93 @@ Task *task;
 /* This routine sets up a task to build a unit of the given type. */
 
 Task *
-create_build_task(unit, u2, run)
-Unit *unit;
-int u2, run;
+create_build_task(Unit *unit, int u2, int run, int x, int y)
 {
     Task *task = create_task(TASK_BUILD);
 
     task->args[0] = u2;
     task->args[3] = run;
+    task->args[4] = x;  task->args[5] = y;
     return task;
 }
 
 void
-set_build_task(unit, u2, run)
-Unit *unit;
-int u2, run;
+set_build_task(Unit *unit, int u2, int run, int x, int y)
 {
-	/* First check if we are building something. */
-	if (unit->plan && unit->plan->tasks
-	 && unit->plan->tasks->type == TASK_BUILD) {
+    /* First check if we are building something. */
+    if (unit->plan && unit->plan->tasks
+	&& unit->plan->tasks->type == TASK_BUILD) {
 
-		Unit *unit2 = find_unit(unit->plan->tasks->args[1]);
+	Unit *unit2 = find_unit(unit->plan->tasks->args[1]);
 
-		/* Return if we already are building the desired unit. */
-		if (is_unit(unit2) && !completed(unit2) && unit2->type == u2) {
-			return;
-		/* Maybe disband incomplete unit of wrong type. */
-		} else if (is_unit(unit2) && !completed(unit2)
-			     && side_can_disband(unit->side, unit2)
-			     && g_disband_unfinished_units()) {
-			/* But first salvage either its cps or materials if permitted to do so. */
-			if (g_salvage_unfinished_cps())
-				unit->cp_stash += unit2->cp;	   			
-			else if (g_salvage_unfinished_materials()) {
-
-				int	m, u = unit2->type;
-
-				for_all_material_types(m) {
-					if (um_consumption_on_creation(u, m) > 0)
-						unit->supply[m] += um_consumption_on_creation(u, m);
-					if (um_consumption_per_cp(u2, m) > 0)
-						unit->supply[m] += (unit2->cp - uu_creation_cp(unit->type, u))
-									      * um_consumption_per_cp(u2, m);
-				}						
-			}
-		kill_unit(unit2, H_UNIT_DISBANDED);
-		}		
-	}	    		
-	add_task(unit, CLEAR_AGENDA, create_build_task(unit, u2, run));
+	/* Return if we already are building the desired unit. */
+	if (is_unit(unit2) && !completed(unit2) && unit2->type == u2) {
+	    return;
+	    /* Maybe disband incomplete unit of wrong type. */
+	} else if (is_unit(unit2) && !completed(unit2)
+		   && side_can_disband(unit->side, unit2)
+		   && g_disband_unfinished_units()) {
+	    /* But first salvage either its cps or materials if
+               permitted to do so. */
+	    if (g_salvage_unfinished_cps())
+	      unit->cp_stash += unit2->cp;	   			
+	    else if (g_salvage_unfinished_materials()) {
+		
+		int	m, u = unit2->type;
+		
+		for_all_material_types(m) {
+		    if (um_consumption_on_creation(u, m) > 0)
+		      unit->supply[m] += um_consumption_on_creation(u, m);
+		    if (um_consumption_per_cp(u2, m) > 0)
+		      unit->supply[m] += (unit2->cp - uu_creation_cp(unit->type, u))
+			* um_consumption_per_cp(u2, m);
+		}						
+	    }
+	    kill_unit(unit2, H_UNIT_DISBANDED);
+	}		
+    }	    		
+    add_task(unit, CLEAR_AGENDA, create_build_task(unit, u2, run, x, y));
 }
 
 void
-push_build_task(unit, u2, run)
-Unit *unit;
-int u2, run;
+push_build_task(Unit *unit, int u2, int run, int x, int y)
 {
-	/* First check if we are building something. */
-	if (unit->plan && unit->plan->tasks
-	 && unit->plan->tasks->type == TASK_BUILD) {
+    /* First check if we are building something. */
+    if (unit->plan && unit->plan->tasks
+	&& unit->plan->tasks->type == TASK_BUILD) {
 
-		Unit *unit2 = find_unit(unit->plan->tasks->args[1]);
+	Unit *unit2 = find_unit(unit->plan->tasks->args[1]);
 
-		/* Return if we already are building the desired unit. */
-		if (is_unit(unit2) && !completed(unit2) && unit2->type == u2) {
-			return;
-		/* Maybe disband incomplete unit of wrong type. */
-		} else if (is_unit(unit2) && !completed(unit2)
-			     && side_can_disband(unit->side, unit2)
-			     && g_disband_unfinished_units()) {
-			/* But first salvage either its cps or materials if permitted to do so. */
-			if (g_salvage_unfinished_cps())
-				unit->cp_stash += unit2->cp;	   			
-			else if (g_salvage_unfinished_materials()) {
-
-				int	m, u = unit2->type;
-
-				for_all_material_types(m) {
-					if (um_consumption_on_creation(u, m) > 0)
-						unit->supply[m] += um_consumption_on_creation(u, m);
-					if (um_consumption_per_cp(u2, m) > 0)
-						unit->supply[m] += (unit2->cp - uu_creation_cp(unit->type, u))
-									      * um_consumption_per_cp(u2, m);
-				}						
-			}
-		kill_unit(unit2, H_UNIT_DISBANDED);
-		}		
-	}	    		
-	 add_task(unit, 0, create_build_task(unit, u2, run));
+	/* Return if we already are building the desired unit. */
+	if (is_unit(unit2) && !completed(unit2) && unit2->type == u2) {
+	    return;
+	    /* Maybe disband incomplete unit of wrong type. */
+	} else if (is_unit(unit2) && !completed(unit2)
+		   && side_can_disband(unit->side, unit2)
+		   && g_disband_unfinished_units()) {
+	    /* But first salvage either its cps or materials if permitted to do so. */
+	    if (g_salvage_unfinished_cps())
+	      unit->cp_stash += unit2->cp;	   			
+	    else if (g_salvage_unfinished_materials()) {
+		
+		int	m, u = unit2->type;
+		
+		for_all_material_types(m) {
+		    if (um_consumption_on_creation(u, m) > 0)
+		      unit->supply[m] += um_consumption_on_creation(u, m);
+		    if (um_consumption_per_cp(u2, m) > 0)
+		      unit->supply[m] += (unit2->cp - uu_creation_cp(unit->type, u))
+			* um_consumption_per_cp(u2, m);
+		}						
+	    }
+	    kill_unit(unit2, H_UNIT_DISBANDED);
+	}		
+    }	    		
+    add_task(unit, 0, create_build_task(unit, u2, run, x, y));
 }
 
 Task *
-create_capture_task(unit, x, y)
-Unit *unit;
-int x, y;
+create_capture_task(Unit *unit, int x, int y)
 {
     Task *task = create_task(TASK_CAPTURE);
 
@@ -2060,17 +2030,13 @@ int x, y;
 }
 
 void
-set_capture_task(unit, x, y)
-Unit *unit;
-int x, y;
+set_capture_task(Unit *unit, int x, int y)
 {
     add_task(unit, CLEAR_AGENDA, create_capture_task(unit, x, y));
 }
 
 void
-push_capture_task(unit, x, y)
-Unit *unit;
-int x, y;
+push_capture_task(Unit *unit, int x, int y)
 {
     add_task(unit, 0, create_capture_task(unit, x, y));
 }
