@@ -1,5 +1,5 @@
 /* Implementation of the transfer protocol for Xconq.
-   Copyright (C) 1996-1999 Stanley T. Shebs.
+   Copyright (C) 1996-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,10 +12,6 @@ any later version.  See the file COPYING. */
 extern void send_variant_value(int i, int v1, int v2, int v3, int v4, int v5);
 extern void (*update_variant_callback)(int i, int v1, int v2, int v3, int v4, int v5);
 extern void (*update_assignment_callback)(int n);
-extern int net_add_side_and_player(void);
-extern void net_set_player_advantage(int n, int newadv);
-extern void net_rename_side_for_player(int n);
-extern void net_set_ai_for_player(int n, char *aitype);
 extern void rename_side_for_player(int n);
 extern void set_ai_for_player(int n, char *aitype);
 extern void add_to_packet(char *str);
@@ -696,9 +692,7 @@ net_set_autofinish(Side *side, int value)
 }
 
 void
-net_set_willing_to_save(side, flag)
-Side *side;
-int flag;
+net_set_willing_to_save(Side *side, int flag)
 {
     if (my_rid == master_rid) {
 	set_willing_to_save(side, flag);
@@ -709,9 +703,7 @@ int flag;
 }
 
 void
-net_set_willing_to_draw(side, flag)
-Side *side;
-int flag;
+net_set_willing_to_draw(Side *side, int flag)
 {
     if (my_rid == master_rid) {
 	set_willing_to_draw(side, flag);
@@ -722,9 +714,7 @@ int flag;
 }
 
 void
-net_set_side_self_unit(side, unit)
-Side *side;
-Unit *unit;
+net_set_side_self_unit(Side *side, Unit *unit)
 {
     if (my_rid == master_rid) {
 	set_side_self_unit(side, unit);
@@ -735,9 +725,7 @@ Unit *unit;
 }
 
 void
-net_set_side_ai(side, typename)
-Side *side;
-char *typename;
+net_set_side_ai(Side *side, char *typename)
 {
     if (my_rid == master_rid) {
 	set_side_ai(side, typename);
@@ -750,9 +738,7 @@ char *typename;
 }
 
 void
-net_set_doctrine(side, spec)
-Side *side;
-char *spec;
+net_set_doctrine(Side *side, char *spec)
 {
     if (my_rid == master_rid) {
 	set_doctrine(side, spec);
@@ -764,11 +750,21 @@ char *spec;
     }
 }
 
+void
+net_set_side_research(Side *side, int a)
+{
+    if (my_rid == master_rid) {
+	set_side_research(side, a);
+    }
+    if (numremotes > 0)  {
+	broadcast_side_property(side, "research", a);
+    }
+}
+
 #ifdef DESIGNERS
 
 void
-net_become_designer(side)
-Side *side;
+net_become_designer(Side *side)
 {
     if (my_rid == master_rid) {
 	become_designer(side);
@@ -779,8 +775,7 @@ Side *side;
 }
 
 void
-net_become_nondesigner(side)
-Side *side;
+net_become_nondesigner(Side *side)
 {
     if (my_rid == master_rid) {
 	become_nondesigner(side);
@@ -791,16 +786,14 @@ Side *side;
 }
 
 void
-net_paint_view(side, x, y, r, tview, uview)
-Side *side;
-int x, y, r, tview, uview;
+net_paint_view(Side *side, int x, int y, int r, int tview, int uview)
 {
     if (my_rid == master_rid) {
 	paint_view(side, x, y, r, tview, uview);
     }
     if (numremotes > 0)  {
 	sprintf(spbuf, "D%d view %d %d %d %d %d %d",
-		       side_number(side), side_number(side), x, y, r, tview, uview);
+		side_number(side), side_number(side), x, y, r, tview, uview);
 	broadcast_packet(spbuf);
     }
 }
@@ -808,30 +801,21 @@ int x, y, r, tview, uview;
 #endif /* DESIGNERS */
 
 static void
-broadcast_side_property(side, prop, val)
-Side *side;
-char *prop;
-int val;
+broadcast_side_property(Side *side, char *prop, int val)
 {
     sprintf(spbuf, "S%d %s %d", side_number(side), prop, val);
     broadcast_packet(spbuf);
 }
 
 static void
-broadcast_side_property_2(side, prop, val, val2)
-Side *side;
-char *prop;
-int val, val2;
+broadcast_side_property_2(Side *side, char *prop, int val, int val2)
 {
     sprintf(spbuf, "S%d %s %d %d", side_number(side), prop, val, val2);
     broadcast_packet(spbuf);
 }
 
 static void
-broadcast_side_str_property(side, prop, val)
-Side *side;
-char *prop;
-char *val;
+broadcast_side_str_property(Side *side, char *prop, char *val)
 {
     sprintf(spbuf, "S%d %s %s", side_number(side), prop, val);
     broadcast_packet(spbuf);
@@ -840,10 +824,7 @@ char *val;
 /* Unit property tweaking. */
 
 void
-net_set_unit_name(side, unit, newname)
-Side *side;
-Unit *unit;
-char *newname;
+net_set_unit_name(Side *side, Unit *unit, char *newname)
 {
     if (my_rid == master_rid) {
 	set_unit_name(side, unit, newname);
@@ -854,10 +835,7 @@ char *newname;
 }
 
 void
-net_set_unit_plan_type(side, unit, type)
-Side *side;
-Unit *unit;
-int type;
+net_set_unit_plan_type(Side *side, Unit *unit, int type)
 {
     if (my_rid == master_rid) {
 	set_unit_plan_type(side, unit, type);
@@ -868,10 +846,7 @@ int type;
 }
 
 void
-net_set_unit_asleep(side, unit, flag, recurse)
-Side *side;
-Unit *unit;
-int flag, recurse;
+net_set_unit_asleep(Side *side, Unit *unit, int flag, int recurse)
 {
     if (my_rid == master_rid) {
 	set_unit_asleep(side, unit, flag, recurse);
@@ -882,10 +857,7 @@ int flag, recurse;
 }
 
 void
-net_set_unit_reserve(side, unit, flag, recurse)
-Side *side;
-Unit *unit;
-int flag, recurse;
+net_set_unit_reserve(Side *side, Unit *unit, int flag, int recurse)
 {
     if (my_rid == master_rid) {
 	set_unit_reserve(side, unit, flag, recurse);
@@ -896,10 +868,7 @@ int flag, recurse;
 }
 
 void
-net_set_unit_main_goal(side, unit, goal)
-Side *side;
-Unit *unit;
-Goal *goal;
+net_set_unit_main_goal(Side *side, Unit *unit, Goal *goal)
 {
     if (my_rid == master_rid) {
 	set_unit_main_goal(side, unit, goal);
@@ -915,10 +884,51 @@ Goal *goal;
 }
 
 void
-net_set_unit_waiting_for_transport(side, unit, flag)
-Side *side;
-Unit *unit;
-int flag;
+net_set_unit_curadvance(Side *side, Unit *unit, int a)
+{
+    if (my_rid == master_rid) {
+	set_unit_curadvance(side, unit, a);
+    }
+    if (numremotes > 0) {
+	broadcast_unit_property(side, unit, "curadvance", a);
+    }
+}
+
+void
+net_set_unit_autoplan(Side *side, Unit *unit, int flag)
+{
+    if (my_rid == master_rid) {
+	set_unit_autoplan(side, unit, flag);
+    }
+    if (numremotes > 0) {
+	broadcast_unit_property(side, unit, "autoplan", flag);
+    }
+}
+
+void
+net_set_unit_autoresearch(Side *side, Unit *unit, int flag)
+{
+    if (my_rid == master_rid) {
+	set_unit_autoresearch(side, unit, flag);
+    }
+    if (numremotes > 0) {
+	broadcast_unit_property(side, unit, "autoresearch", flag);
+    }
+}
+
+void
+net_set_unit_autobuild(Side *side, Unit *unit, int flag)
+{
+    if (my_rid == master_rid) {
+	set_unit_autobuild(side, unit, flag);
+    }
+    if (numremotes > 0) {
+	broadcast_unit_property(side, unit, "autobuild", flag);
+    }
+}
+
+void
+net_set_unit_waiting_for_transport(Side *side, Unit *unit, int flag)
 {
     if (my_rid == master_rid) {
 	set_unit_waiting_for_transport(side, unit, flag);
@@ -929,23 +939,7 @@ int flag;
 }
 
 void
-net_reserve_unit(side, unit)
-Side *side;
-Unit *unit;
-{
-    if (my_rid == master_rid) {
-	reserve_unit(side, unit);
-    }
-    if (numremotes > 0) {
-	broadcast_unit_property(side, unit, "resunit", 0);
-    }
-}
-
-void
-net_wake_unit(side, unit, wakeocc)
-Side *side;
-Unit *unit;
-int wakeocc;
+net_wake_unit(Side *side, Unit *unit, int wakeocc)
 {
     if (my_rid == master_rid) {
 	wake_unit(side, unit, wakeocc);
@@ -956,9 +950,7 @@ int wakeocc;
 }
 
 void
-net_wake_area(side, x, y, n, occs)
-Side *side;
-int x, y, n, occs;
+net_wake_area(Side *side, int x, int y, int n, int occs)
 {
     if (my_rid == master_rid) {
 	wake_area(side, x, y, n, occs);
@@ -969,10 +961,7 @@ int x, y, n, occs;
 }
 
 void
-net_set_unit_ai_control(side, unit, flag, recurse)
-Side *side;
-Unit *unit;
-int flag, recurse;
+net_set_unit_ai_control(Side *side, Unit *unit, int flag, int recurse)
 {
     if (my_rid == master_rid) {
 	set_unit_ai_control(side, unit, flag, recurse);
@@ -983,9 +972,7 @@ int flag, recurse;
 }
 
 int
-net_clear_task_agenda(side, unit)
-Side *side;
-Unit *unit;
+net_clear_task_agenda(Side *side, Unit *unit)
 {
     if (my_rid == master_rid) {
 	clear_task_agenda(unit->plan);
@@ -997,10 +984,7 @@ Unit *unit;
 }
 
 void
-net_force_replan(side, unit, passive_only)
-Side *side;
-Unit *unit;
-int passive_only;
+net_force_replan(Side *side, Unit *unit, int passive_only)
 {
     if (my_rid == master_rid) {
 	force_replan(side, unit, passive_only);
@@ -1011,9 +995,7 @@ int passive_only;
 }
 
 int
-net_disband_unit(side, unit)
-Side *side;
-Unit *unit;
+net_disband_unit(Side *side, Unit *unit)
 {
     if (my_rid == master_rid) {
 	return disband_unit(side, unit);
@@ -1025,9 +1007,7 @@ Unit *unit;
 }
 
 void
-net_set_formation(unit, leader, x, y, dist, flex)
-Unit *unit, *leader;
-int x, y, dist, flex;
+net_set_formation(Unit *unit, Unit *leader, int x, int y, int dist, int flex)
 {
     if (my_rid == master_rid) {
 	set_formation(unit, leader, x, y, dist, flex);
@@ -1039,9 +1019,7 @@ int x, y, dist, flex;
 }
 
 void
-net_delay_unit(unit, flag)
-Unit *unit;
-int flag;
+net_delay_unit(Unit *unit, int flag)
 {
     if (my_rid == master_rid) {
 	delay_unit(unit, flag);
@@ -1056,9 +1034,7 @@ int flag;
 /* A designer can call this to create an arbitrary unit during the game. */
 
 Unit *
-net_designer_create_unit(side, u, s, x, y)
-Side *side;
-int u, s, x, y;
+net_designer_create_unit(Side *side, int u, int s, int x, int y)
 {
     Unit *rslt = NULL;
 
@@ -1075,9 +1051,7 @@ int u, s, x, y;
    This is for use by designers only! */
 
 int
-net_designer_teleport(unit, x, y, other)
-Unit *unit, *other;
-int x, y;
+net_designer_teleport(Unit *unit, int x, int y, Unit *other)
 {
     int rslt = 0;
 
@@ -1091,9 +1065,7 @@ int x, y;
 }
 
 int
-net_designer_change_side(unit, side)
-Unit *unit;
-Side *side;
+net_designer_change_side(Unit *unit, Side *side)
 {
     int rslt = 0;
 
@@ -1108,8 +1080,7 @@ Side *side;
 }
 
 int
-net_designer_disband(unit)
-Unit *unit;
+net_designer_disband(Unit *unit)
 {
     int rslt = 0;
 
@@ -1125,8 +1096,7 @@ Unit *unit;
 #endif /* DESIGNERS */
 
 Feature *
-net_create_feature(typename, name)
-char *typename, *name;
+net_create_feature(char *typename, char *name)
 {
     Feature *rslt = NULL;
 
@@ -1140,9 +1110,7 @@ char *typename, *name;
 }
 
 void
-net_set_feature_type_name(feature, typename)
-Feature *feature;
-char *typename;
+net_set_feature_type_name(Feature *feature, char *typename)
 {
     if (my_rid == master_rid) {
 	set_feature_type_name(feature, typename);
@@ -1153,9 +1121,7 @@ char *typename;
 }
 
 void
-net_set_feature_name(feature, name)
-Feature *feature;
-char *name;
+net_set_feature_name(Feature *feature, char *name)
 {
     if (my_rid == master_rid) {
 	set_feature_name(feature, name);
@@ -1166,8 +1132,7 @@ char *name;
 }
 
 void
-net_destroy_feature(feature)
-Feature *feature;
+net_destroy_feature(Feature *feature)
 {
     if (my_rid == master_rid) {
 	destroy_feature(feature);
@@ -1178,7 +1143,7 @@ Feature *feature;
 }
 
 void
-net_renumber_features()
+net_renumber_features(void)
 {
     if (my_rid == master_rid) {
 	renumber_features();
@@ -1190,11 +1155,7 @@ net_renumber_features()
 
 
 static void
-broadcast_unit_property(side, unit, prop, val)
-Side *side;
-Unit *unit;
-char *prop;
-int val;
+broadcast_unit_property(Side *side, Unit *unit, char *prop, int val)
 {
     char buf[BUFSIZE];
 
@@ -1203,11 +1164,7 @@ int val;
 }
 
 static void
-broadcast_unit_property_2(side, unit, prop, val, val2)
-Side *side;
-Unit *unit;
-char *prop;
-int val, val2;
+broadcast_unit_property_2(Side *side, Unit *unit, char *prop, int val, int val2)
 {
     char buf[BUFSIZE];
 
@@ -1216,23 +1173,18 @@ int val, val2;
 }
 
 static void
-broadcast_unit_property_5(side, unit, prop, val, val2, val3, val4, val5)
-Side *side;
-Unit *unit;
-char *prop;
-int val, val2, val3, val4, val5;
+broadcast_unit_property_5(Side *side, Unit *unit, char *prop,
+			  int val, int val2, int val3, int val4, int val5)
 {
     char buf[BUFSIZE];
 
-    sprintf(buf, "U%d %d %s %d %d %d %d %d", side_number(side), unit->id, prop, val, val2, val3, val4, val5);
+    sprintf(buf, "U%d %d %s %d %d %d %d %d", side_number(side), unit->id, prop,
+	    val, val2, val3, val4, val5);
     broadcast_packet(buf);
 }
 
 static void
-broadcast_unit_str_property(side, unit, prop, val)
-Side *side;
-Unit *unit;
-char *prop, *val;
+broadcast_unit_str_property(Side *side, Unit *unit, char *prop, char *val)
 {
     char buf[BUFSIZE];
 
@@ -1243,8 +1195,7 @@ char *prop, *val;
 /* Action networking. */
 
 void
-broadcast_next_action(unit)
-Unit *unit;
+broadcast_next_action(Unit *unit)
 {
     int j, atype, n;
     char buf[BUFSIZE];
@@ -1377,31 +1328,30 @@ int dir, n;
 }
 
 void
-net_set_build_task(unit, u2, run)
-Unit *unit;
-int u2, run;
+net_set_build_task(Unit *unit, int u2, int run, int x, int y)
 {
     if (my_rid == master_rid) {
-	set_build_task(unit, u2, run);
+	set_build_task(unit, u2, run, x, y);
     }
     if (numremotes > 0) {
-	tmptask = create_build_task(unit, u2, run);
+	tmptask = create_build_task(unit, u2, run, x, y);
 	broadcast_add_task(unit, CLEAR_AGENDA, tmptask);
 	free_task(tmptask);
     }
 }
 
-/* Same as above but handles cases where we resume building an incomplete occ. */
+/* Same as above but handles cases where we resume building an
+   incomplete occ. */
 
 void
-net_resume_build_task(Unit *unit, Unit *unit2, int run)
+net_resume_build_task(Unit *unit, Unit *unit2, int run, int x, int y)
 {
     if (my_rid == master_rid) {
-	set_build_task(unit, unit2->type, run);
+	set_build_task(unit, unit2->type, run, x, y);
 	unit->plan->tasks->args[1] = unit2->id;		
     }
     if (numremotes > 0) {
-	tmptask = create_build_task(unit, unit2->type, run);
+	tmptask = create_build_task(unit, unit2->type, run, x, y);
 	tmptask->args[1] = unit2->id;		
 	broadcast_add_task(unit, CLEAR_AGENDA, tmptask);
 	free_task(tmptask);
@@ -1409,24 +1359,20 @@ net_resume_build_task(Unit *unit, Unit *unit2, int run)
 }
 
 void
-net_push_build_task(unit, u2, run)
-Unit *unit;
-int u2, run;
+net_push_build_task(Unit *unit, int u2, int run, int x, int y)
 {
     if (my_rid == master_rid) {
-	push_build_task(unit, u2, run);
+	push_build_task(unit, u2, run, x, y);
     }
     if (numremotes > 0) {
-	tmptask = create_build_task(unit, u2, run);
+	tmptask = create_build_task(unit, u2, run, x, y);
 	broadcast_add_task(unit, 0, tmptask);
 	free_task(tmptask);
     }
 }
 
 void
-net_push_develop_task(unit, u2, n)
-Unit *unit;
-int u2, n;
+net_push_develop_task(Unit *unit, int u2, int n)
 {
     if (my_rid == master_rid) {
 	push_develop_task(unit, u2, n);
@@ -1541,9 +1487,7 @@ int m, n;
 }
 
 void
-net_set_sentry_task(unit, n)
-Unit *unit;
-int n;
+net_set_sentry_task(Unit *unit, int n)
 {
     if (my_rid == master_rid) {
 	set_sentry_task(unit, n);
@@ -1556,10 +1500,7 @@ int n;
 }
 
 static void
-broadcast_add_task(unit, pos, task)
-Unit *unit;
-int pos;
-Task *task;
+broadcast_add_task(Unit *unit, int pos, Task *task)
 {
     int numargs, i;
     char buf[BUFSIZE], *argtypes;
@@ -2433,13 +2374,17 @@ receive_side_prop(char *str)
       return;
     str = nstr;
     ++str;
-    if (strncmp(str, "af ", 3) == 0) {
+    if (strncmp(str, "adjective ", 10) == 0) {
+	set_side_adjective(side, side, str + 10);
+    } else if (strncmp(str, "af ", 3) == 0) {
 	str += 3;
 	val = strtol(str, &nstr, 10);
 	set_autofinish(side, val);
     } else if (strncmp(str, "ai ", 3) == 0) {
 	str += 3;
 	set_side_ai(side, str);
+    } else if (strncmp(str, "colorscheme ", 12) == 0) {
+	set_side_colorscheme(side, side, str + 12);
     } else if (strncmp(str, "controlledby ", 13) == 0) {
 	str += 13;
 	sn = strtol(str, &nstr, 10);
@@ -2460,14 +2405,24 @@ receive_side_prop(char *str)
 	str += 5;
 	val = strtol(str, &nstr, 10);
 	set_willing_to_draw(side, val);
+    } else if (strncmp(str, "emblemname ", 11) == 0) {
+	set_side_emblemname(side, side, str + 11);
     } else if (strncmp(str, "fin ", 4) == 0) {
 	finish_turn(side);
+    } else if (strncmp(str, "longname ", 9) == 0) {
+	set_side_longname(side, side, str + 9);
     } else if (strncmp(str, "mutualtrust ", 12) == 0) {
 	str += 12;
 	sn = strtol(str, &nstr, 10);
 	str = nstr;
 	val = strtol(str, &nstr, 10);
 	set_mutual_trust(side, side_n(sn), val);
+    } else if (strncmp(str, "name ", 5) == 0) {
+	set_side_name(side, side, str + 5);
+    } else if (strncmp(str, "noun ", 5) == 0) {
+	set_side_noun(side, side, str + 5);
+    } else if (strncmp(str, "pluralnoun ", 11) == 0) {
+	set_side_pluralnoun(side, side, str + 11);
     } else if (strncmp(str, "resign ", 7) == 0) {
 	str += 7;
 	val = strtol(str, &nstr, 10);
@@ -2481,28 +2436,18 @@ receive_side_prop(char *str)
 	val = strtol(str, &nstr, 10);
 	unit = find_unit(val);
 	set_side_self_unit(side, unit);
+    } else if (strncmp(str, "research ", 9) == 0) {
+	str += 9;
+	val = strtol(str, &nstr, 10);
+	set_side_research(side, val);
+    } else if (strncmp(str, "shortname ", 10) == 0) {
+	set_side_shortname(side, side, str + 10);
     } else if (strncmp(str, "trust ", 6) == 0) {
 	str += 6;
 	sn = strtol(str, &nstr, 10);
 	str = nstr;
 	val = strtol(str, &nstr, 10);
 	set_trust(side, side_n(sn), val);
-    } else if (strncmp(str, "adjective ", 10) == 0) {
-	set_side_adjective(side, side, str + 10);
-    } else if (strncmp(str, "colorscheme ", 12) == 0) {
-	set_side_colorscheme(side, side, str + 12);
-    } else if (strncmp(str, "emblemname ", 11) == 0) {
-	set_side_emblemname(side, side, str + 11);
-    } else if (strncmp(str, "longname ", 9) == 0) {
-	set_side_longname(side, side, str + 9);
-    } else if (strncmp(str, "name ", 5) == 0) {
-	set_side_name(side, side, str + 5);
-    } else if (strncmp(str, "noun ", 5) == 0) {
-	set_side_noun(side, side, str + 5);
-    } else if (strncmp(str, "pluralnoun ", 11) == 0) {
-	set_side_pluralnoun(side, side, str + 11);
-    } else if (strncmp(str, "shortname ", 10) == 0) {
-	set_side_shortname(side, side, str + 10);
     } else {
 	run_warning("Unknown S packet \"%s\", ignoring", str);
     }
@@ -2566,13 +2511,23 @@ receive_unit_prop(char *str)
 	    goal = NULL;
 	}
 	set_unit_main_goal(side, unit, goal);
+    } else if (strncmp(str, "curadvance ", 11) == 0) {
+	val = strtol(str + 11, &nstr, 10);
+	set_unit_curadvance(side, unit, val);
+    } else if (strncmp(str, "autoplan ", 9) == 0) {
+	val = strtol(str + 9, &nstr, 10);
+	set_unit_autoplan(side, unit, val);
+    } else if (strncmp(str, "autoresearch ", 13) == 0) {
+	val = strtol(str + 13, &nstr, 10);
+	set_unit_autoresearch(side, unit, val);
+    } else if (strncmp(str, "autobuild ", 10) == 0) {
+	val = strtol(str + 10, &nstr, 10);
+	set_unit_autobuild(side, unit, val);
     } else if (strncmp(str, "name ", 5) == 0) {
 	set_unit_name(side, unit, str + 5);
     } else if (strncmp(str, "plan ", 5) == 0) {
 	val = strtol(str + 5, &nstr, 10);
 	set_unit_plan_type(side, unit, val);
-    } else if (strncmp(str, "resunit ", 8) == 0) {
-	reserve_unit(side, unit);
     } else if (strncmp(str, "resv ", 5) == 0) {
 	str += 5;
 	val = strtol(str, &nstr, 10);
