@@ -1077,8 +1077,14 @@ calc_view_misc(VP *vp)
     /* (but it seems the adjustment is wrong for sizing - should figure
        out why this seems necessary) */
     vp->totsw = area.width * vp->hw + hexagon_adjust(vp);
+    
+    /* I don't understand why this adjustment is done. It has no apparent 
+    effect on either mac, xtconq or tcltk drawing. HR. */
+#if 0    
     if (area.xwrap && (vp->totsw > (vp->pxw - vp->hw)))
       vp->totsw += (vp->pxw - vp->hw);
+#endif
+
     /* Total scaled height is based on center-to-center height, plus
        an adjustment to include the bottom parts of the bottom row. */
     vp->totsh = area.height * vp->hch + (vp->hh - vp->hch);
@@ -1089,7 +1095,10 @@ calc_view_misc(VP *vp)
     else
       vp->sxmax = max(vp->sxmin, vp->totsw - vp->pxw);  
     vp->symax = max(vp->symin, vp->totsh - vp->pxh);
-    vp->sx = limitn(vp->sxmin, vp->sx, vp->sxmax);
+    
+    /* Limiting sx to totsw instead of sxmax fixes the misaligned updates 
+    bug at 8x8 resultion. */
+    vp->sx = limitn(vp->sxmin, vp->sx, vp->totsw);
     vp->sy = limitn(vp->symin, vp->sy, vp->symax);
 }
 
@@ -1408,51 +1417,57 @@ void (*text_callback)(int x1, int y1, int x1f, int y1f, char *str);
 		}
 		if (text_callback) {
 		    if (lon == lonmin || lon == lonmax) {
-			char minbuf[10];
-    			int latdeg, latminu;
+				char minbuf[10];
+	    		int latdeg, latminu;
 
-			lonmid = lon + incr / 2;
-			latlong_to_xy(lat, lonmid,
-				      &xmid, &ymid, &xmidf, &ymidf);
-			/* If adding the incr goes off the world, subtract
-			   instead. */
-			if (!in_area(xmid, ymid)) {
-			    lonmid = lon - incr / 2;
-			    latlong_to_xy(lat, lonmid,
-					  &xmid, &ymid, &xmidf, &ymidf);
-			}
-			latdeg = abs(lat) / 60;
-			latminu = abs(lat) % 60;
-			minbuf[0] = '\0';
-			if (latminu != 0)
-			  sprintf(minbuf, "%dm", latminu);
-			sprintf(tmpbuf, "%dd%s%c",
-				latdeg, minbuf, (lat >= 0 ? 'N' : 'S'));
+				lonmid = lon + incr / 2;
+				latlong_to_xy(lat, lonmid,
+					      &xmid, &ymid, &xmidf, &ymidf);
+				/* If adding the incr goes off the world, subtract
+				   instead. */
+				if (!in_area(xmid, ymid)) {
+				    lonmid = lon - incr / 2;
+				    latlong_to_xy(lat, lonmid,
+						  &xmid, &ymid, &xmidf, &ymidf);
+				}
+				/* Give up if this did not work. */
+				if (in_area(xmid, ymid)) {
+					latdeg = abs(lat) / 60;
+					latminu = abs(lat) % 60;
+					minbuf[0] = '\0';
+					if (latminu != 0)
+					  sprintf(minbuf, "%dm", latminu);
+					sprintf(tmpbuf, "%dd%s%c",
+						latdeg, minbuf, (lat >= 0 ? 'N' : 'S'));
 
-			(*text_callback)(xmid, ymid, xmidf, ymidf, tmpbuf);
+					(*text_callback)(xmid, ymid, xmidf, ymidf, tmpbuf);
+				}
 		    }
 		    if (lat == latmin || lat == latmax) {
-			char minbuf[10];
-    			int londeg, lonminu;
+				char minbuf[10];
+	    		int londeg, lonminu;
 
-			latmid = lat + incr / 2;
-			latlong_to_xy(latmid, lon,
-				      &xmid, &ymid, &xmidf, &ymidf);
-			/* If adding the incr goes off the world, subtract
-			   instead. */
-			if (!in_area(xmid, ymid)) {
-			    latmid = lat - incr / 2;
-			    latlong_to_xy(latmid, lon,
-					  &xmid, &ymid, &xmidf, &ymidf);
-			}
-			londeg = abs(lon) / 60;
-			lonminu = abs(lon) % 60;
-			minbuf[0] = '\0';
-			if (lonminu != 0)
-			  sprintf(minbuf, "%dm", lonminu);
-			sprintf(tmpbuf, "%dd%s%c",
-				londeg, minbuf, (lon >= 0 ? 'E' : 'W'));
-			(*text_callback)(xmid, ymid, xmidf, ymidf, tmpbuf);
+				latmid = lat + incr / 2;
+				latlong_to_xy(latmid, lon,
+					      &xmid, &ymid, &xmidf, &ymidf);
+				/* If adding the incr goes off the world, subtract
+				   instead. */
+				if (!in_area(xmid, ymid)) {
+				    latmid = lat - incr / 2;
+				    latlong_to_xy(latmid, lon,
+						  &xmid, &ymid, &xmidf, &ymidf);
+				}
+				/* Give up if this did not work. */
+				if (in_area(xmid, ymid)) {
+					londeg = abs(lon) / 60;
+					lonminu = abs(lon) % 60;
+					minbuf[0] = '\0';
+					if (lonminu != 0)
+					  sprintf(minbuf, "%dm", lonminu);
+					sprintf(tmpbuf, "%dd%s%c",
+						londeg, minbuf, (lon >= 0 ? 'E' : 'W'));
+					(*text_callback)(xmid, ymid, xmidf, ymidf, tmpbuf);
+			    }
 		    }
 		}
 #if 0 /* use to debug meridian plotting */
@@ -1740,8 +1755,8 @@ void
 oneliner(Side *side, VP *vp, int sx, int sy)
 {
     int x, y, xf, yf;
-    Unit *unit;
-    int t2, uview, u, s, ps = NOBODY, cs = NOBODY, dep, sayin = FALSE;
+    Unit *unit, *user;
+    int t2, uview, u, s, ps = NOBODY, cs = NOBODY, dep, sayin = FALSE, userid;
     char *peopdesc = NULL, *sidedesc, *str;
     char descbuf[80], ctrlbuf[80], buf[BUFSIZE];
     Side *side2, *side3;
@@ -1894,6 +1909,17 @@ oneliner(Side *side, VP *vp, int sx, int sy)
 	strcat(tmpbuf, descbuf);
 	strcat(tmpbuf, ")");
     }
+    if (user_defined()
+	&& ((userid = user_at(x, y)) != NOUSER)) {
+	user = find_unit(userid);
+	strcat(tmpbuf, " (used by ");
+	if (user->name)
+	  strcat(tmpbuf, user->name);
+	else
+	  sprintf(tmpbuf+strlen(tmpbuf), "%s u#%d",
+		  u_type_name(user->type), user->id);
+	strcat(tmpbuf, ")");
+    }
     if (vp->draw_ai && side_has_ai(side)) {
 	strcat(tmpbuf, " ");
 	strcat(tmpbuf, ai_at_desig(side, x, y));
@@ -2102,6 +2128,8 @@ int ch, *dir1p, *dir2p, *modp;
    cell/other unit, prepare a "most appropriate" action. */
 /* (should share diff cell and same cell interaction code) */
 
+extern void toggle_user_at(Unit *unit, int x, int y);
+
 int
 advance_into_cell(Side *side, Unit *unit, int x, int y, Unit *other)
 {
@@ -2116,6 +2144,12 @@ advance_into_cell(Side *side, Unit *unit, int x, int y, Unit *other)
     z = unit->z;
     if (x != unit->x || y != unit->y) {
 	if (unit->act && unit->plan) { /* (should be more sophist test?) */
+	    if (!mobile(unit->type)
+		&& u_advanced(unit->type)
+		&& distance(unit->x, unit->y, x, y) <= unit->reach) {
+		toggle_user_at(unit, x, y);
+		return TRUE;
+	    }
 	    if (distance(unit->x, unit->y, x, y) == 1) {
 		/* Destination is adjacent to us. */
 		/* First, handle the case where no unit was clicked on. */
@@ -2269,6 +2303,47 @@ advance_into_cell(Side *side, Unit *unit, int x, int y, Unit *other)
     return TRUE;
 }
 
+void
+toggle_user_at(Unit *unit, int x, int y)
+{
+    Side *side2;
+
+    /* Return if landuse is undefined. */
+    if (!user_defined())
+      return;
+    /* Return if the cell is not visible to this side. */
+    if (!terrain_visible(unit->side, x, y))
+      return;
+    /* Return if the cell is used by another unit. */
+    if (user_at(x, y) != NOUSER && user_at(x, y) != unit->id)
+      return;
+    /* Return if using maxcells and we are trying to add one more. */
+    if (unit->usedcells >= unit->maxcells && user_at(x, y) != unit->id)
+      return;
+    /* Return if independents or untrusted side has a unit in the cell. */
+    if (unit_at(x, y) != NULL) {
+	Unit *unit2;
+	for_all_stack(x, y, unit2)
+	  if (!trusted_side(unit->side, unit2->side))
+	    return;
+    }
+    /* Toggle landuse by unit either on or off for the cell. */
+    if (user_at(x, y) == NOUSER) {
+	set_user_at(x, y, unit->id);
+	unit->usedcells += 1;
+    } else if (user_at(x, y) == unit->id) {
+	set_user_at(x, y, NOUSER);
+	unit->usedcells -= 1;
+    }
+    for_all_sides(side2) {
+	if (side2->see_all
+	    || side_sees_unit(side2, unit) 
+	    || side_tracking_unit(side2, unit)) {
+	    update_cell_display(side2, x, y, UPDATE_ALWAYS);
+	}
+    }
+}
+
 /* Given a unit and amounts of supplies desired to transfer, move as
    much of them as possible into the unit's transport. */
 
@@ -2332,9 +2407,7 @@ short *amts, *rslts;
    transport into the unit. */
 
 int
-take_supplies(unit, amts, rslts)
-Unit *unit;
-short *amts, *rslts;
+take_supplies(Unit *unit, short *amts, short *rslts)
 {
     int m, want, actual, neededsome;
     Unit *unit2;
@@ -2350,40 +2423,42 @@ short *amts, *rslts;
 	    neededsome = TRUE;
 	    unit2 = unit->transport;
 	    if (in_play(unit2) && completed(unit2)) {
-	    	
-	    	/* Bug fix. Units inside a transport inside a base or city were
-	    	previously unable to resupply directly from the base. Now the 
-	    	occ checks if the transport is inside a base and then resupplies 
-	    	directly from it. Moreover, the 'stingy' 50% left principle is 
-	    	applied only to units that need m themselves. Other units are 
-	    	free to give away all their supply. */
+	    	/* Bug fix. Units inside a transport inside a base or
+	    	   city were previously unable to resupply directly
+	    	   from the base. Now the occ checks if the transport
+	    	   is inside a base and then resupplies directly from
+	    	   it. Moreover, the 'stingy' 50% left principle is
+	    	   applied only to units that need m themselves. Other
+	    	   units are free to give away all their supply. */
 	    
-	         Unit	*base = unit2->transport;
-		int	request = want;				
+		Unit *base = unit2->transport;
+		int request = want;				
 
-		/* First take as much supplies as possible from the transport's 
-		transport (the base) if it exists. */
+		/* First take as much supplies as possible from the
+		   transport's transport (the base) if it exists. */
 		actual = 0;
-	         if (in_play(base) && completed(base)) {
-			/* Be stingy if base is low (but only if it also needs m). */
-			if (2 * base->supply[m] < um_storage_x(base->type, m)
-			    && (um_base_consumption(base->type, m) > 0 
-			         || um_consumption_per_move(base->type, m) > 0)) {		    			
-				request = max(1, want/2);
-			}
-			actual += transfer_supply(base, unit, m, request);
-			want -= actual;
+		if (in_play(base) && completed(base)) {
+		  /* Be stingy if base is low (but only if it also needs m). */
+		  if (2 * base->supply[m] < um_storage_x(base->type, m)
+		      && (um_base_consumption(base->type, m) > 0 
+			  || um_consumption_per_move(base->type, m) > 0)) {		    			
+		    request = max(1, want/2);
+		  }
+		  actual += transfer_supply(base, unit, m, request);
+		  want -= actual;
 		}
-		/* If still unsatisfied, also take supplies from transport. */
+		/* If still unsatisfied, also take supplies from
+                   transport. */
 		if (want) {
-			request = want;
-			/* Be stingy if transport is low (but only if it also needs m). */
-			if (2 * unit2->supply[m] < um_storage_x(unit2->type, m)
-			    && (um_base_consumption(unit2->type, m) > 0 
-			         || um_consumption_per_move(unit2->type, m) > 0)) {		    			
-				request = max(1, want/2);
-			}
-			actual += transfer_supply(unit2, unit, m, want);
+		  request = want;
+		  /* Be stingy if transport is low (but only if it
+                     also needs m). */
+		  if (2 * unit2->supply[m] < um_storage_x(unit2->type, m)
+		      && (um_base_consumption(unit2->type, m) > 0 
+			  || um_consumption_per_move(unit2->type, m) > 0)) {
+		    request = max(1, want/2);
+		  }
+		  actual += transfer_supply(unit2, unit, m, want);
 		}
 		if (rslts)
 		  rslts[m] = actual;
