@@ -331,8 +331,7 @@ int flagacp;
 }
 
 void
-init_unit_extras(unit)
-Unit *unit;
+init_unit_extras(Unit *unit)
 {
     if (unit->extras == NULL) {
 	unit->extras = (UnitExtras *) xmalloc(sizeof(UnitExtras));
@@ -352,9 +351,7 @@ Unit *unit;
 /* Changing a unit's type has many effects. */
 
 void
-change_unit_type(unit, newtype, reason)
-Unit *unit;
-int newtype, reason;
+change_unit_type(Unit *unit, int newtype, int reason)
 {
     int oldtype = unit->type, oldhp = unit->hp;
     PastUnit *pastunit;
@@ -385,10 +382,11 @@ int newtype, reason;
     /* We might have to change sides as a result. */
     /* (should modify per-side counts) */
     if (!type_allowed_on_side(newtype, unit->side)) {
-    	if (type_allowed_on_side(newtype, NULL)) {
+    	if (type_allowed_on_side(newtype, indepside)) {
 	    /* Unit becomes independent. */
-	    change_unit_side(unit, NULL, reason, NULL);
+	    change_unit_side(unit, indepside, reason, NULL);
 	} else {
+	    /* (what to do? search for an allowed side?) */
 	    run_warning("Leaving unit on disallowed side");
 	}
     }
@@ -417,8 +415,7 @@ int newtype, reason;
 }
 
 int
-max_builds(u)
-int u;
+max_builds(int u)
 {
     int u2;
 
@@ -430,9 +427,7 @@ int u;
 }
 
 int
-side_owns_occupant(side, unit)
-Side *side;
-Unit *unit;
+side_owns_occupant(Side *side, Unit *unit)
 {
     int suboccs = FALSE;
     Unit *occ;
@@ -1328,10 +1323,7 @@ Unit *unit, *occ;
    to another.  This is a common internal routine, so no messages here. */
 
 void
-change_unit_side(unit, newside, reason, agent)
-Unit *unit, *agent;
-Side *newside;
-int reason;
+change_unit_side(Unit *unit, Side *newside, int reason, Unit *agent)
 {
     int ux = unit->x, uy = unit->y;
     Side *oldside = unit->side;
@@ -1381,9 +1373,7 @@ int reason;
    on the given side. */
 
 int
-unit_allowed_on_side(unit, side)
-Unit *unit;
-Side *side;
+unit_allowed_on_side(Unit *unit, Side *side)
 {
     int u;
 
@@ -1424,8 +1414,7 @@ new_unit_allowed_on_side(int u, Side *side)
 }
 
 int
-test_class_membership(leaf)
-Obj *leaf;
+test_class_membership(Obj *leaf)
 {
     char *sclass;
 
@@ -1445,32 +1434,29 @@ Obj *leaf;
     }
 }
 
-/* Inability to build a unit should not preclude its capture etc. Separate calls to
-has_advance_to_build have therefore been introduced everywhere in the code. */
+/* Inability to build a unit should not preclude its capture
+   etc. Separate calls to has_advance_to_build have therefore been
+   introduced everywhere in the code. */
 
 int
-type_allowed_on_side(u, side)
-int u;
-Side *side;
+type_allowed_on_side(int u, Side *side)
 {
+    int u2;
+
     if (side->uavail == NULL) {
-
-	int u2;
-
 	side->uavail = (short *) xmalloc(numutypes * sizeof(short));
 	for_all_unit_types(u2) {
 	    tmpside_classtest = side;
 	    side->uavail[u2] =
-	      eval_boolean_expression(u_possible_sides(u2), test_class_membership, TRUE);
+	      eval_boolean_expression(u_possible_sides(u2),
+				      test_class_membership, TRUE);
 	}
     }
     return side->uavail[u];
 }
 
 int
-type_ever_available(u, side)
-int u;
-Side *side;
+type_ever_available(int u, Side *side)
 {
     int u2;
     Unit *unit;
@@ -1494,9 +1480,7 @@ Side *side;
 /* TRUE if side has advance to build this unit. */
 
 int
-has_advance_to_build(side, u)
-Side *side;
-int u;
+has_advance_to_build(Side *side, int u)
 {
     int a;
 
@@ -1505,12 +1489,15 @@ int u;
 	    return FALSE;
 	}
     }
+    /* Account for an obsoleting advance. */
+    a = u_obsolete(u);
+    if (a != NONATYPE && has_advance(side, a))
+      return FALSE;
     return TRUE;
 }
 
 int
-num_sides_allowed(u)
-int u;
+num_sides_allowed(int u)
 {
     int rslt;
     Side *side;
