@@ -1,5 +1,5 @@
 /* Definitions relating to units in Xconq.
-   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -289,10 +289,12 @@ typedef struct a_actorstate {
 #define has_pending_action(unit)  \
   ((unit)->act && (unit)->act->nextaction.type != ACTION_NONE)
 
-/* This is true if the unit still has acp left to act with. */
+/* This is true if the unit still has acp left to act with.  This used
+   to test acp > acp-min, but that allows units to move around with
+   negative acp, and doesn't work as well in game designs. */
 
-#define has_acp_left(unit) \
-  ((unit)->act && (unit)->act->acp > u_acp_min((unit)->type))
+#define has_acp_left(unit)  \
+  ((unit)->act && (unit)->act->acp > 0)
 
 /* All the definitions that govern planning. */
 
