@@ -1358,8 +1358,7 @@ write_doctrines()
 /* Write declarations of all the sides. */
 
 static void
-write_sides(module)
-Module *module;
+write_sides(Module *module)
 {
     Side *side;
 
@@ -1403,8 +1402,7 @@ Module *module;
 /* Write random properties of a side. */
 
 static void
-write_side_properties(side)
-Side *side;
+write_side_properties(Side *side)
 {
     int i, u, u2, anyudoctrines;
 
@@ -1564,8 +1562,7 @@ Side *side;
 }
 
 static void 
-write_standing_orders(side)
-Side *side;
+write_standing_orders(Side *side)
 {
     int u, u1, numtypes, numargs;
     char *str = NULL;
@@ -1633,121 +1630,103 @@ Side *side;
 /* Write about what has been seen in the area. */
 
 static int
-fn_terrain_view(x, y)
-int x, y;
+fn_terrain_view(int x, int y)
 {
     return terrain_view(tmpside, x, y);
 }
 
 static int
-fn_terrain_view_date(x, y)
-int x, y;
+fn_terrain_view_date(int x, int y)
 {
     return terrain_view_date(tmpside, x, y);
 }
 
 static int
-fn_aux_terrain_view(x, y)
-int x, y;
+fn_aux_terrain_view(int x, int y)
 {
     return aux_terrain_view(tmpside, x, y, tmpttype);
 }
 
 static int
-fn_aux_terrain_view_date(x, y)
-int x, y;
+fn_aux_terrain_view_date(int x, int y)
 {
     return aux_terrain_view_date(tmpside, x, y, tmpttype);
 }
 
 static int
-fn_unit_view(x, y)
-int x, y;
+fn_unit_view(int x, int y)
 {
     return unit_view(tmpside, x, y);
 }
 
 static int
-fn_unit_view_date(x, y)
-int x, y;
+fn_unit_view_date(int x, int y)
 {
     return unit_view_date(tmpside, x, y);
 }
 
 static int
-fn_material_view(x, y)
-int x, y;
+fn_material_view(int x, int y)
 {
     return material_view(tmpside, x, y, tmpmtype);
 }
 
 static int
-fn_material_view_date(x, y)
-int x, y;
+fn_material_view_date(int x, int y)
 {
     return material_view_date(tmpside, x, y, tmpmtype);
 }
 
 static int
-fn_temp_view(x, y)
-int x, y;
+fn_temp_view(int x, int y)
 {
     return temperature_view(tmpside, x, y);
 }
 
 static int
-fn_temp_view_date(x, y)
-int x, y;
+fn_temp_view_date(int x, int y)
 {
     return temperature_view_date(tmpside, x, y);
 }
 
 static int
-fn_cloud_view(x, y)
-int x, y;
+fn_cloud_view(int x, int y)
 {
     return cloud_view(tmpside, x, y);
 }
 
 static int
-fn_cloud_bottom_view(x, y)
-int x, y;
+fn_cloud_bottom_view(int x, int y)
 {
     return cloud_bottom_view(tmpside, x, y);
 }
 
 static int
-fn_cloud_height_view(x, y)
-int x, y;
+fn_cloud_height_view(int x, int y)
 {
     return cloud_height_view(tmpside, x, y);
 }
 
 static int
-fn_cloud_view_date(x, y)
-int x, y;
+fn_cloud_view_date(int x, int y)
 {
     return cloud_view_date(tmpside, x, y);
 }
 
 static int
-fn_wind_view(x, y)
-int x, y;
+fn_wind_view(int x, int y)
 {
     return wind_view(tmpside, x, y);
 }
 
 static int
-fn_wind_view_date(x, y)
-int x, y;
+fn_wind_view_date(int x, int y)
 {
     return wind_view_date(tmpside, x, y);
 }
 
 static void
-write_side_view(side, compress)
-     Side *side;
-     int compress;
+write_side_view(Side *side, int compress)
 {
     int t, m;
 
@@ -1812,8 +1791,7 @@ write_side_view(side, compress)
 }
 
 static void
-write_one_side_view_layer(propkey, fn)
-int propkey, (*fn)(int x, int y);
+write_one_side_view_layer(int propkey, int (*fn)(int x, int y))
 {
     newline_form();
     start_form(key(K_SIDE));
@@ -1830,7 +1808,7 @@ int propkey, (*fn)(int x, int y);
 }
 
 static void
-write_players()
+write_players(void)
 {
     Side *side;
 
@@ -1845,8 +1823,7 @@ write_players()
 }
 
 static void
-write_player(player)
-Player *player;
+write_player(Player *player)
 {
     start_form(key(K_PLAYER));
     add_num_to_form(player->id);
@@ -1862,7 +1839,7 @@ Player *player;
 }
 
 static void
-write_agreements()
+write_agreements(void)
 {
     Agreement *ag;
 
@@ -1872,8 +1849,7 @@ write_agreements()
 }
 
 static void
-write_agreement(agreement)
-Agreement *agreement;
+write_agreement(Agreement *agreement)
 {
     start_form(key(K_AGREEMENT));
     add_num_to_form(agreement->id);
@@ -1900,8 +1876,7 @@ Agreement *agreement;
 /* Write the unit section of a game module. */
 
 static void
-write_units(module)
-Module *module;
+write_units(Module *module)
 {
     int x0, y0, x, y, numtowrite;
     Unit *unit;
@@ -2008,8 +1983,7 @@ Module *module;
 /* Write random properties, but only if they have non-default values. */
 
 static void
-write_unit_properties(unit)
-Unit *unit;
+write_unit_properties(Unit *unit)
 {
     write_num_prop(key(K_NB), unit->number, 0, FALSE, FALSE);
     write_num_prop(key(K_HP), unit->hp, u_hp(unit->type), FALSE, FALSE);
@@ -2027,27 +2001,26 @@ Unit *unit;
     /* (should do appear x,y also here?) */
     write_num_prop(key(K_DISAPPEAR), unit_disappear_turn(unit), -1, FALSE, FALSE);
     write_lisp_prop(key(K_X), unit_hook(unit), lispnil, FALSE, TRUE, FALSE);
-	/* Advanced unit support. */
-	if (u_advanced(unit->type)) {
-		write_num_prop(key(K_SIZE), unit->size, 0, FALSE, FALSE);		
-		write_num_prop(key(K_REACH), unit->reach, 2, FALSE, FALSE);		
-		write_num_prop(key(K_USEDCELLS), unit->usedcells, 0, FALSE, FALSE);		
-		write_num_prop(key(K_MAXCELLS), unit->maxcells, 1, FALSE, FALSE);		
-		write_num_prop(key(K_CURADVANCE), unit->curadvance, -1, FALSE, FALSE);		
-		write_num_prop(key(K_POPULATION), unit->population, 1, FALSE, FALSE);		
-    		write_mtype_value_list(key(K_PRODUCTION), unit->production, 0, FALSE);
-		write_num_prop(key(K_AUTOPLAN), unit->autoplan, FALSE, FALSE, FALSE);		
-		write_num_prop(key(K_CP_STASH), unit->cp_stash, FALSE, FALSE, FALSE);		
-		write_num_prop(key(K_AUTOBUILD), unit->autobuild, FALSE, FALSE, FALSE);		
-		write_num_prop(key(K_AUTORESEARCH), unit->autoresearch, FALSE, FALSE, FALSE);		
-	}
+    /* Advanced unit support. */
+    if (!u_advanced(unit->type))
+      return;
+    write_num_prop(key(K_SIZE), unit->size, 0, FALSE, FALSE);	
+    write_num_prop(key(K_REACH), unit->reach, 0, FALSE, FALSE);		
+    write_num_prop(key(K_USEDCELLS), unit->usedcells, 0, FALSE, FALSE);
+    write_num_prop(key(K_MAXCELLS), unit->maxcells, 1, FALSE, FALSE);
+    write_num_prop(key(K_CURADVANCE), unit->curadvance, -1, FALSE, FALSE);
+    write_num_prop(key(K_POPULATION), unit->population, 1, FALSE, FALSE);
+    write_mtype_value_list(key(K_PRODUCTION), unit->production, 0, FALSE);
+    write_num_prop(key(K_AUTOPLAN), unit->autoplan, FALSE, FALSE, FALSE);
+    write_num_prop(key(K_CP_STASH), unit->cp_stash, FALSE, FALSE, FALSE);
+    write_num_prop(key(K_AUTOBUILD), unit->autobuild, FALSE, FALSE, FALSE);
+    write_num_prop(key(K_AUTORESEARCH), unit->autoresearch, FALSE, FALSE, FALSE);
 }
 
 /* Write out the unit's current actor state. */
 
 static void
-write_unit_act(unit)
-Unit *unit;
+write_unit_act(Unit *unit)
 {
     int acp = u_acp(unit->type);
     ActorState *act = unit->act;
@@ -2078,9 +2051,7 @@ Unit *unit;
 }
 
 static void
-write_action(action, id)
-Action *action;
-int id;
+write_action(Action *action, int id)
 {
     int atype, i, slen;
 
@@ -2097,8 +2068,7 @@ int id;
 /* Write out the unit's current plan. */
 
 static void
-write_unit_plan(unit)
-Unit *unit;
+write_unit_plan(Unit *unit)
 {
     Task *task;
     Plan *plan = unit->plan;
@@ -2143,8 +2113,7 @@ Unit *unit;
 }
 
 static void
-write_task(task)
-Task *task;
+write_task(Task *task)
 {
     int i, numargs;
     char *argtypes;
@@ -2164,9 +2133,7 @@ Task *task;
 }
 
 static void
-write_goal(goal, keyword)
-Goal *goal;
-int keyword;
+write_goal(Goal *goal, int keyword)
 {
     int i, numargs;
     char *argtypes;
@@ -2186,7 +2153,7 @@ int keyword;
 /* Write all the historical events recorded so far. */
 
 static void
-write_history()
+write_history(void)
 {
     PastUnit *pastunit;
     HistEvent *hevt;
@@ -2205,8 +2172,7 @@ write_history()
 }
 
 static void
-write_past_unit(pastunit)
-PastUnit *pastunit;
+write_past_unit(PastUnit *pastunit)
 {
     start_form(key(K_EXU));
     add_num_to_form(pastunit->id);
@@ -2222,8 +2188,7 @@ PastUnit *pastunit;
 }
 
 static void
-write_historical_event(hevt)
-HistEvent *hevt;
+write_historical_event(HistEvent *hevt)
 {
     int i;
     char *descs;
