@@ -14,38 +14,69 @@ any later version.  See the file COPYING.  */
 void prealloc_debug(void);
 #include "lisp.h"
 #include "imf.h"
-extern int emblem_position PARAMS ((Image *uimg, char *ename, Image *eimg, int sw, int sh,
-				    int *exxp, int *eyyp, int *ewp, int *ehp));
+
+#include <stddef.h>
+#include <stdarg.h>
+#include <stdlib.h>
 
 #ifdef THINK_C
 #include <MacHeaders>
-#else /* probably MPW */
+#include <Sound.h>
+#endif /* THINK_C */
+
+#if 1 /*def MPW*/
 #ifndef __MWERKS__ /* actually, "old headers" */
 #include <Values.h>
 #endif
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#include <MacWindows.h>
+#else
 #include <Types.h>
-#include <Memory.h>
+#include <Windows.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <QuickDraw.h>
 #include <Fonts.h>
 #include <Events.h>
-#include <Windows.h>
 #include <Menus.h>
+#include <TextEdit.h>
 #include <Dialogs.h>
+#ifndef NEW_HEADERS
 #include <Desk.h>
+#endif
+#include <Scrap.h>
 #include <ToolUtils.h>
+#ifdef NEW_HEADERS
+#include <MacMemory.h>
+#else
+#include <Memory.h>
+#endif
 #include <SegLoad.h>
 #include <Files.h>
 #include <Folders.h>
 #include <OSUtils.h>
+#ifndef __MWERKS__
 #include <OSEvents.h>
+#endif
 #include <DiskInit.h>
 #include <Packages.h>
 #include <Traps.h>
+#include <Lists.h>
 #include <StandardFile.h>
-#endif
+#include <Sound.h>
+#include <Devices.h>
+#endif /* MPW */
 
+#include <QDOffscreen.h>
+
+#ifdef NEW_HEADERS
+#include <Gestalt.h>
+#else
+#include <GestaltEqu.h>
+#endif
 #include <PPCToolbox.h>
+#include <AppleEvents.h>
 
 /* This is a way - perhaps not the best way - to distinguish "old headers"
    from "universal headers". */
@@ -65,6 +96,8 @@ extern int emblem_position PARAMS ((Image *uimg, char *ename, Image *eimg, int s
 
 #include "macimf.h"
 
+LibraryPath *xconq_libs;
+
 extern void syntax_error(Obj *x, char *msg);
 
 /* Semi-private routines from macimf.c. */
@@ -2058,4 +2091,3 @@ __ttyname(long fd)
 }
 
 #endif /* __MWERKS__ */
-
