@@ -46,6 +46,8 @@ static void init_fonts(void);
 static void init_rects(void);
 static void init_ae(void);
 static void handle_event(EventRecord *event);
+static int handle_keystroke(EventRecord *event);
+static void drag_window(WindowPtr win, EventRecord *event);
 static pascal OSErr do_ae_open_application(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_open_documents(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_print_documents(AppleEvent *message, AppleEvent *reply, long refcon);
@@ -507,13 +509,13 @@ get_files()
 void
 event_loop()
 {
-	int done = FALSE;
-	Boolean gotevent;
-	Point mouse;
+	int 			done = FALSE;
+	Boolean		gotevent;
+	Point 		mouse;
 	EventRecord	event;
-	RgnHandle cursorRgn;
-	short itemhit;
-	DialogPtr dialog;
+	RgnHandle 		cursorRgn;
+	short 		itemhit;
+	DialogPtr 		dialog;
 
 	/* Figure out if the WaitNextEvent Trap is available. */
 	useWNE = (NGetTrapAddress(0x60, ToolTrap) != NGetTrapAddress(0x9f, ToolTrap));
@@ -521,43 +523,43 @@ event_loop()
 	cursorRgn = NewRgn();
 	/* Loop (almost) forever. */
 	while (!eventloopdone) {
+		/* Move here since we must call adjust_cursor at least once. Thus, we can 
+		avoid the second call below by always calling it here. */
+		get_global_mouse(&mouse);
+		adjust_cursor(mouse, cursorRgn);
 		/* Use WaitNextEvent if it is available, otherwise GetNextEvent. */
 		if (useWNE) {
-			get_global_mouse(&mouse);
-			adjust_cursor(mouse, cursorRgn);
 			gotevent = WaitNextEvent(everyEvent, &event, 0L, cursorRgn);
 		} else {
 			SystemTask();
 			gotevent = GetNextEvent(everyEvent, &event);
 		}
-		/* First test if the event is for playersetupwin, the only modeless dialog left. */
-		if (topWindow 
-		     && playersetupwin == topWindow 
-		     && IsDialogEvent(&event)) {
-			/* Process events we want to handle specially, such as keystrokes. */
-		  	handle_player_setup_event(&event);
-			/* Stop if the dialog just was closed, or DialogSelect will crash! */
-			if (playersetupwin == NULL)
+		/* Deal with keystrokes first since their handling now is the same 
+		for modeless dialogs and normal windows. */
+		if (event.what == keyDown || event.what == autoKey) {
+			if (handle_keystroke(&event))
 			    continue;
-			/* Process events that the Dialog Manager handles, such as clicks on buttons. */
-			if (DialogSelect(&event, &dialog, &itemhit)) {
+		}
+		/* Then handle non-keystroke modeless dialog events. */
+		if (IsDialogEvent(&event) && DialogSelect(&event, &dialog, &itemhit)) {
 				if (dialog == playersetupwin) {
-					mouse = event.where;
-					SetPort(playersetupwin);
-					GlobalToLocal(&mouse);
-					if (hit_player_setup_dialog(itemhit, mouse))
-					  launch_game_2();
+				hit_player_setup_dialog(itemhit, mouse);
+			} else if (topFloat && unit_closeup_from_window(topFloat)) {
+				hit_closeup_dialog(topFloat, itemhit, mouse);
 				}
 				/* If this was something like a key or click event, but wasn't handled
-				   by the dialog, we're done and can go wait for the next event.
-				   Otherwise, for null events, fall through to the usual handling. */
+			   by the dialog, we're done and can go wait for the next event. Otherwise, 
+			   for null events, fall through to the usual handling. */
 				if (gotevent)
 				  continue;
 			}
-		}
 		if (gotevent) {
+
+#if 0			/* Redundant, adjust_cursor is now always called above. */
+		
 			/* Make sure we have the right cursor before handling the event. */
 			adjust_cursor(event.where, cursorRgn);
+#endif
 			handle_event(&event);
 		} else if (connection_method > 0 && numremotewaiting > 0) {
 			receive_data(0);
@@ -569,11 +571,9 @@ event_loop()
 			run_local_ai(1, 20);
 			net_run_game(1);
 			maybe_select_next_unit();
-
 #if 0
 			run_local_ai(2, 20);
 #endif
-
 			/* If the game ended, force various changes in interaction. */
 			if (endofgame)
 			  set_end_of_game_interaction_modes();
@@ -592,6 +592,78 @@ event_loop()
 	}
 }
 
+/* This function returns TRUE for keyboard events that get handled and
+FALSE for those who should fall through to the dialog manager. */
+
+int
+handle_keystroke(EventRecord *event)
+{
+	char	charCode = event->message & charCodeMask;
+	char	keyCode = (event->message & keyCodeMask) >> 8;
+	int	handled = FALSE;
+	UnitCloseup *closeup;
+	List	*list;
+	
+	/* Menu key commands. */
+	if (event->modifiers & cmdKey) {
+		adjust_menus();
+		do_menu_command(MenuKey(charCode));
+		return TRUE;
+	}
+	/* Alt-w closes topFloat. */
+	if (charCode == '') {
+		close_window(topFloat);
+		return TRUE;
+	} 
+	/* Key commands for the player setup dialog. */ 
+	if (FrontWindow() == playersetupwin) {
+		handle_player_setup_event(event);
+		return TRUE;
+	}		
+	/* Key commands for modeless dialogs etc. */	
+	if (topFloat) {
+		closeup = unit_closeup_from_window(topFloat);
+		if (closeup) {
+			int result = do_key_down_closeup(closeup, keyCode);
+
+			/* We are done. Return TRUE to prevent further
+			handling by the dialog manager. */
+			if (result == DONE) {
+				return TRUE;
+			/* This was a key that should be handled by the
+			dialog manager. Return FALSE. */
+			} else if (result == TRUE) {
+				return FALSE;
+			/* The key returned FALSE and should therefore
+			be processed further below. */
+			}
+		} else if (topFloat == helpwin) {
+			handled = do_key_down_help(charCode);		
+		}
+	}
+	/* Key commands for normal windows. */
+	if (!handled && topWindow) {
+		list = list_from_window(topWindow);
+		if (list) {
+			handled = do_key_down_list(list, charCode);
+		} else	if (topWindow == constructionwin) {
+			handled = do_key_down_construction(charCode);
+		} else if (topWindow == commandwin) {
+			handled = do_key_down_command(charCode);
+		}
+	}
+	/* Special use of the numeric keypad and arrow keys for 
+	scrolling of maps. */
+	if (!handled && (frontmap || worldmap)) {
+		handled = handle_numeric_keypad(event);
+	}
+	/* All other keys are interpreted as keyboard commands. */
+	if (!handled) {
+		do_keyboard_command(charCode);
+	}
+	return TRUE;
+}
+
 void
 get_global_mouse(Point *mouse)
 {
@@ -633,6 +705,14 @@ adjust_cursor(Point mouse, RgnHandle region)
 	Map *map;
 	GrafPtr oldport;
 
+	/* Work around of Appearance manager bug. Since clicks in the collapse box fails
+	to generate an application level event, and since the system fails to update dialogs
+	that are uncollapsed, it is necessary to force dialog updates in some other way. */
+	if (topFloat && IsDialog(topFloat) 
+	    && PtInRgn(mouse, ((WindowPeek) topFloat)->strucRgn) == TRUE
+	    && PtInRgn(mouse, ((WindowPeek) topFloat)->contRgn) == FALSE) {
+		update_window(topFloat);
+	}
 	map = frontmap;
 	if (map != NULL) {
 		GetPort(&oldport);
@@ -645,8 +725,10 @@ adjust_cursor(Point mouse, RgnHandle region)
 			return;
 		}
 
-		if (mouse.h > map->conw && mouse.h < map->window->portRect.right - map->sbarwid
-			&& mouse.v > map->toph && mouse.v < map->window->portRect.bottom - map->sbarwid) {
+		if (mouse.h > map->conw 
+		    && mouse.v > map->toph 
+		    && mouse.h < map->window->portRect.right - map->sbarwid
+		    && mouse.v < map->window->portRect.bottom - map->sbarwid) {
 
 			if (mouse.h < map->conw + 40) {
 				current_cursor = leftcursor;				
@@ -773,14 +855,13 @@ void
 handle_event(EventRecord *event)
 {
 	short part, err;
-	WindowPtr win;
-	char key;
+	WindowPtr win, win2;
 	Point pnt;
 
 	switch (event->what) {
 		case mouseDown:
 			/* See if the click happened in a special part of the screen. */
-			part = FindWindow(event->where, &win);
+			part = MacFindWindow(event->where, &win);
 			switch (part) {
 				case inMenuBar:
 					adjust_menus();
@@ -810,8 +891,7 @@ handle_event(EventRecord *event)
 					}
 					break;
 				case inDrag:
-					/* Standard drag behavior, no tricks necessary. */
-					DragTheWindow(win, event);
+					drag_window(win, event);
 					break;
 				case inGrow:
 					grow_window(win, event->where);
@@ -824,36 +904,21 @@ handle_event(EventRecord *event)
 					if (TrackGoAway(win, event->where))
 					  close_window(win);
 					break;
+				case inCollapseBox:
+					/* Uncollapsed dialogs don't get redrawn by the system. */ 
+					if (IsDialog(win)) {
+					    EraseRect(&win->portRect);
+					    force_update(win);
+					}
+					break;					
 			}
 			break;
+#if 0					/* Now handled elsewhere. */
 		case keyDown:
 		case autoKey:
-			/* First handle the special use of the numeric keypad for offscreen scrolling:
-			break if a keypad key was hit to prevent reading of its charCode below. */
-			if (handle_numeric_keypad(event))
-			  break;
-			key = event->message & charCodeMask;
-			/* Check for menukey equivalents. */
-			if (event->modifiers & cmdKey) {
-				if (event->what == keyDown) {
-					adjust_menus();
-					do_menu_command(MenuKey(key));
-				}
-			/* '' is the charCode for option + w. */
-			} else if (key == '') {
-				close_window(topFloat);
-			} else if (event->what == keyDown) {
-				int handled = FALSE;
-
-				/* Random keypress, interpret it. */
-				if (topWindow && topWindow == constructionwin)
-				  handled = do_key_down_construction(key);
-				if (topWindow && topWindow == commandwin)
-				  handled = do_key_down_command(key);
-				if (!handled)
-				  do_keyboard_command(key);
-			}
+			handle_keystroke(event);
 			break;
+#endif
 		case activateEvt:
 			activate_window((WindowPtr) event->message, event->modifiers & activeFlag);
 			break;
@@ -882,7 +947,7 @@ handle_event(EventRecord *event)
 					ActivateWindow(topWindow, !inbackground);
 					/* The floats should also be activated/deactivated! */
 					for_all_windows(win) {
-						if (IsFloating(win)) {
+						if (IsFloating(win) || IsDialog(win)) {
 							ActivateWindow(win, !inbackground);
 						}							
 					}
@@ -905,6 +970,14 @@ handle_event(EventRecord *event)
 #endif
 }
 
+void
+drag_window(WindowPtr win, EventRecord *event)
+{
+	DragTheWindow(win, event);
+	/* Necessary to redraw some modeless dialogs. */
+	update_all_visible_dialogs();
+}
+
 /* Handle window growing by mindlessly tracking via GrowWindow,
    then passing the chosen size to specific window resize handlers
    or else doing the generic resize. */
@@ -936,6 +1009,8 @@ grow_window(WindowPtr win, Point where)
 			grow_scores(winh, winv);
 		}
 		SetPort(oldport);
+		/* Necessary to redraw some modeless dialogs. */
+		update_all_visible_dialogs();
 	}
 }
 
@@ -969,6 +1044,8 @@ zoom_window(WindowPtr win, Point where, int part)
 			InvalRect(&win->portRect);
 		}
 		SetPort(oldport);
+		/* Necessary to redraw some modeless dialogs. */
+		update_all_visible_dialogs();
 	}
 }
 
@@ -993,7 +1070,8 @@ close_window(WindowPtr win)
 		HideTheWindow(win);
 		/* At least for now, don't actually dispose of the window. */
 	}
-	update_all_map_windows();
+	/* Necessary to redraw some modeless dialogs. */
+	update_all_visible_dialogs();
 }
 
 /* This just dispatches to the appropriate window handler. */
@@ -1724,7 +1802,7 @@ update_side_display(Side *side, Side *side2, int rightnow)
 				/* A side must have been added recently; redraw
 				   the entire window this time. */
 				if (eimages[side_number(side2)] == NULL)
-				  init_emblem_images(side2);
+				  init_emblem_images();
 				draw_game();
 			}
 		}
@@ -1955,7 +2033,7 @@ update_fire_into_display(Side *side, Unit *unit, int x2, int y2, int z2, int m,
 void
 update_unit_display(Side *side, Unit *unit, int rightnow)
 {
-	UnitCloseup *unitcloseup;
+	UnitCloseup *unitcloseup = find_unit_closeup(unit);
 
 	if (active_display(side) && unit != NULL) {
 		update_unit_in_maps(unit);
@@ -1963,9 +2041,9 @@ update_unit_display(Side *side, Unit *unit, int rightnow)
 			update_cell_display(side, unit->x, unit->y, UPDATE_ALWAYS);
 		}
 		update_unit_in_lists(unit);
-		if ((unitcloseup = find_unit_closeup(unit)) != NULL
-		    && 1 /* window is visible */) {
-			draw_unit_closeup(unitcloseup);
+		if (unitcloseup
+		    && ((WindowPeek) (unitcloseup->window))->visible) {
+			update_window(unitcloseup->window);
 		}
 		if (unit->side != NULL && unit->act != NULL) {
 			update_side_display(side, unit->side, rightnow);
@@ -1991,8 +2069,9 @@ update_unit_acp_display(Side *side, Unit *unit, int rightnow)
 #endif
 		update_unit_in_lists(unit);
 		unitcloseup = find_unit_closeup(unit);
-		if (unitcloseup != NULL && (((WindowPeek) (unitcloseup->window))->visible)) {
-			draw_unit_closeup(unitcloseup);
+		if (unitcloseup
+		    && ((WindowPeek) (unitcloseup->window))->visible) {
+			update_window(unitcloseup->window);
 		}
 	}
 }
@@ -2629,9 +2708,10 @@ force_update(WindowPtr win)
 
  		update_gworld(map);
 	} else {
-		/* For non-maps, erase and invalidate everything. */
-		EraseRect(&win->portRect);
+		/* For non-maps invalidate and update the window. Don't EraseRect
+		since it messes up the modeless dialogs. */
 		InvalRect(&win->portRect);
+		update_window(win);
 	}
 	SetPort(oldport);
 }
