@@ -861,6 +861,7 @@ extern void grow_list(List *list, int w, int h);
 extern void zoom_list(List *list, int part);
 extern void adjust_list_decor(List *list);
 extern void do_mouse_down_list(List *list, Point mouse, int mods);
+extern int do_key_down_list(List *list, char key);
 extern void set_list_sorting(List *list, enum sortkeys newkey, int mi);
 extern void toggle_list_completed(List *list);
 extern void toggle_list_incomplete(List *list);
@@ -888,19 +889,18 @@ extern int open_game_dialog(void);
 extern int open_game_from_fsspec(FSSpec *fsspec);
 extern int open_game_from_name_and_volume(Str255 name, int vrefnum);
 extern void handle_player_setup_event(EventRecord *event);
-extern int hit_player_setup_dialog(int itemhit, Point mouse);
+extern void hit_player_setup_dialog(int itemhit, Point mouse);
+extern void hit_closeup_dialog(WindowPtr win, int itemhit, Point mouse);
+extern int do_key_down_closeup(UnitCloseup *closeup, char key);
 extern int launch_game(void);
 extern void launch_game_2(void);
 extern void check_for_missing_images(void);
 extern void open_progress_dialog(void);
 extern void close_progress_dialog(void);
 extern void init_display(void);
-extern void init_terrain_images(void);
-extern void interp_named_color(char *name, enum grays *grayvar, RGBColor *colorvar);
 extern void init_unit_images(void);
-extern void init_all_emblem_images(void);
-extern void reinit_all_emblem_images(void);
-extern void init_emblem_images(Side *side2);
+extern void init_emblem_images(void);
+extern void init_terrain_images(void);
 extern void init_menus(void);
 extern void add_window_menu_item(char *name, WindowPtr win);
 extern void remove_window_menu_item(WindowPtr win);
@@ -1035,6 +1035,7 @@ extern void set_help_content(HelpNode *helpnode);
 extern void draw_help(void);
 extern void adjust_help_scrollbar(void);
 extern void activate_help(int activate);
+extern int do_key_down_help(char key);
 extern void do_mouse_down_help(Point mouse, int mods);
 extern void grow_help(int h, int v);
 extern void zoom_help(int part);
@@ -1081,8 +1082,10 @@ extern void show_command_window(void);
 extern void invert_map(Map *map, int x, int y, int r);
 extern Map *front_map(void);
 
+extern void update_all_dialogs(void);
 extern void update_all_windows(void);
 extern void update_all_map_windows(void);
+extern void update_all_visible_dialogs(void);
 extern void update_all_visible_windows(void);
 extern void update_all_visible_map_windows(void);
 
@@ -1124,10 +1127,6 @@ extern void set_terrain_color(Str255 tname);	/* Feature color Dialog for terrain
 extern void build_colors_menu(void);			/* Build the Colors menu */
 extern void build_sides_menu(void);			/* Build popup menu for prefs */
 extern void build_features_menu(void);		/* Build popup menu for prefs */
-extern void init_unit_cicns(void);			/* Intitialize cicns by a dummy PlotCIcon call */
-extern void init_emblem_cicns(void);			/* Intitialize cicns ny a dummy PlotCIcon call */
-extern void dispose_unit_cicns(void);			/* Trash every unit cicn in use */
-extern void dispose_emblem_cicns(void);		/* Trash every emblem cicn in use */
 
 extern void dissect_colorscheme(int e);	 	/* Dissect a colorscheme into sideColorNames */
 
