@@ -10,6 +10,7 @@ any later version.  See the file COPYING.  */
 
 #define OkButton 1
 #define CancelButton 2
+#define HelpButton 3
 
 /* The menu bar. */
 
@@ -70,7 +71,7 @@ any later version.  See the file COPYING.  */
 #define mFeatPopup 708
 
 #define mBuildPopup 710
-#define mAdvancePopup 711
+#define mResearchPopup 711
 
 #define mAdvanceAchieved 713
 
@@ -762,7 +763,6 @@ any later version.  See the file COPYING.  */
 
 #define dCity 803
 
-#define diCityMap 3
 #define diCityMatPanel 4
 #define diCityFacPanel 5
 #define diCityOccPanel 6
@@ -789,6 +789,8 @@ any later version.  See the file COPYING.  */
 
 #define diCityIcon 93
 #define diCityClickText 94
+#define diCityBuildTimes 97
+#define diCityMap 98
 
 #define dAdvance 804
 
@@ -844,6 +846,8 @@ any later version.  See the file COPYING.  */
 
 #define diWorldMapOK 1
 
+#define dCloseup 810
+
 #define MAXPROJMENUS 9		/* Max number of project menus (excluding Idle units) and associated items. */
 
 /* Alerts. */
@@ -939,6 +943,7 @@ any later version.  See the file COPYING.  */
 #define cPrevButton 133
 #define cNextButton 134
 #define cBackButton 135
+#define cDevelopButton 136
 
 #define cFeatureAddButton 401
 #define cFeatureRemoveButton 402
