@@ -818,38 +818,29 @@ handle_numeric_keypad(EventRecord *event)
 {
 	int	pxh, pxw;
 	Map *map;
-	char key;
-
-	List *list;
+	char	key;
 
 	/* Get the worldmap if alt key was down, else get the front map. */
 	if (event->modifiers & optionKey)
 		map = worldmap;
 	else	map = frontmap;
 
-	/* Check keys by keyCode instead of charCode to identify numeric keypad events */
+	/* Use keyCode instead of charCode to distinguish numeric keypad keys. */
 	key = (event->message & keyCodeMask) >> 8;
 
-	/* Return FALSE if this is not a numeric keypad key (or if it is an arrow key on an antique Mac) */
-
+	/* Return FALSE if this is not a numeric keypad key (or if it is an arrow 
+	key on an antique Mac) */
 	if (key < 0x41
 		|| (key > 0x5C && key < 0x7B)
 		|| key == 0x42
 		|| key == 0x46
+		|| key == 0x47
 		|| key == 0x48
+		|| key == 0x4C
 		|| key == 0x4D)
 
 	  return FALSE;
 
-	/* Keyboard scrolling of list window. */
-
-	/* First check if the front window is a list and scroll it if we pressed a normal arrow key. */
-	if (((list = list_from_window(topWindow)) != NULL) && map != worldmap
-	     && (key == 0x7B || key == 0x7C || key == 0x7D || key == 0x7E)) {
-		scroll_list_using_key(list, key);
-		return TRUE;
-	}
-
 	/* Return TRUE and BEEP if the front window is not a map window */
 	if (!map) {
 		beep(); 
@@ -904,16 +895,6 @@ handle_numeric_keypad(EventRecord *event)
 
 		if (key == 0x43) { rotate_angle_up(map); return TRUE; }		/* Keypad * */
 		if (key == 0x4B) { rotate_angle_down(map); return TRUE; }		/* Keypad / */
-
-		if (Debug || DebugG || DebugM) {
-			/* Take a snapshot of the gworld */
-			if (key == 0x47) {
-				copy_from_gworld(map, map->contentrect); 		/* Keypad CLEAR */
-				return TRUE;
-			}
-			/* Enter the debugger */
-			if (key == 0x4C) {Debugger(); return TRUE; }			/* Keypad ENTER */
-		}
 	}
 	return FALSE;	/* Default FALSE permits normal use of unassigned numeric keypad keys */
 }
@@ -1296,3 +1277,25 @@ update_all_visible_map_windows(void)
 	}
 }
 
+void
+update_all_dialogs(void)
+{
+	WindowPtr win;
+	
+	for_all_windows(win) {
+		if (((WindowPeek) win)->windowKind == dialogKind)
+			update_window(win);
+	}
+}
+
+void
+update_all_visible_dialogs(void)
+{
+	WindowPtr win;
+	
+	for_all_windows(win) {
+		if (((WindowPeek) win)->windowKind == dialogKind
+		    && ((WindowPeek) win)->visible)
+			update_window(win);
+	}
+}
