@@ -49,6 +49,7 @@ static void mac_interp_image(ImageFamily *imf, Image *img, int force);
 static void mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int force);
 static void mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump);
 static void mac_copy_bytes(char *data, int numbytes, Handle desthandle, int macrowbytes, int rowbytes);
+static void mac_init_cicn_1(Image *img);
 static CTabHandle interp_ctab(Image *img);
 static CTabHandle synth_ctab(void);
 static void convert_ppat(Image *img, PixPatHandle colrpat);
@@ -65,6 +66,10 @@ init_mac_image(Image *img)
 	for (j = 0; j < 8; ++j)
 	  SET_IMG_PAT (macimg, j, '\F');
 	macimg->generic = img;
+//	
+	macimg->colricon = nil;
+	macimg->cicn_inited = FALSE;
+	
 	return macimg;
 }
 
@@ -526,7 +531,7 @@ mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int fo
 		colrdone = maskdone = TRUE;
 	}
 	/* Finally, a color icon covers all other cases. */
-	if (hasColorQD && w <= 64 && h <= 64 && !img->istile
+	if (hasColorQD && w <= 88 && h <= 96 && !img->istile
 	    && (subimg->rawcolrdata != NULL
 	        || (macimg->monosicn == nil
 	            && macimg->masksicn == nil
@@ -536,9 +541,7 @@ mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int fo
 		colrdone = TRUE;
 	}
 	/* This will mostly happen for images that are too large. */
-	/* Special-case 88x96 because the image library does include
-	   some, for use by other interfaces. */
-	if (!monodone && !colrdone && !(w == 88 && h == 96)) {
+	if (!monodone && !colrdone) {
 		run_warning("%dx%d image of \"%s\" could not be interpreted", w, h, imf->name);
 	}
 }
@@ -588,8 +591,8 @@ mac_create_cicn(Image *img)
 	int pixelsize, rowbytes, macrowbytes;
 	MacImage *macimg;
 	Handle sicnhandle, iconhandle, datahandle, monohandle, maskhandle;
-    CIconHandle cicnhandle;
-    PixMapHandle pmhandle;
+	CIconHandle cicnhandle;
+	PixMapHandle pmhandle;
 
 	w = img->w;  h = img->h;
 	actualw = img->actualw;  actualh = img->actualh;
@@ -604,7 +607,9 @@ mac_create_cicn(Image *img)
 	macimg = get_mac_image(img);
 	/* Make a full color icon. */
 	/* Allocate enough space for the icon and its data/mask bitmaps. */
-	cicnhandle = (CIconHandle) NewHandle(sizeof(CIcon) + 2 * actualh * macbitrowbytes);
+	/* Use xmalloc to avoid heap gymnastics. */
+	cicnhandle = (CIconHandle) NewEmptyHandle();
+	*cicnhandle = (CIcon *) xmalloc(sizeof(CIcon) + 2 * actualh * macbitrowbytes);
 	if (cicnhandle == nil)
 	  run_error("out of memory");
 	HLock((Handle) cicnhandle);
@@ -632,7 +637,9 @@ mac_create_cicn(Image *img)
 	(*cicnhandle)->iconMask.rowBytes = macbitrowbytes;
 	(*cicnhandle)->iconMask.baseAddr = NULL;
 	numbytes = actualh * macrowbytes;
-	datahandle = NewHandle(numbytes);
+	/* Use xmalloc to avoid heap gymnastics. */
+	datahandle = NewEmptyHandle();
+	*datahandle = xmalloc(numbytes);
 	(*cicnhandle)->iconData = datahandle;
 	/* Fill up the datahandle with the color data, or else use mono/mask data
 	   if the color data is missing. */
@@ -768,26 +775,10 @@ mac_copy_bytes(char *data, int numbytes, Handle desthandle, int macrowbytes, int
     HUnlock(desthandle);
 }
 
-/* This function makes dummy calls to PlotCIcon.  The dummy call is absolutely
-   essential if one wishes to speed up and/or modify plotting by direct access
-   of the cicn PixMap and BitMap, as is now done in draw_unit_image and draw_side_emblem. 
-   The reason for this is that PlotCIcon not only plots the cicn, but also sets up
-   a pointer to the PixMap data at offset zero. If this has not been done at least
-   once, CopyBits cannot handle the cicn. Moreover, since this is a pointer and not
-   a handle, it gets trashed whenever the cicn or PixMap data is moved in memory.
-   This is why a dummy call to PlotCIcon is needed before every access of the PixMap.
-   To avoid this, the cicn, PixMap and CTable are now locked in memory right after
-   the first (and only) PlotCIcon call. To prevent fragmentation, the handles are
-   also moved to the top of the heap zone before this is done. */
-
-void mac_init_cicn_1(Image *img);
-
 void
 mac_init_cicn(Image *img)
 {
 	int subi;
-	CIconHandle cicnhandle;
-	MacImage *macimg;
 
 	if (img->numsubimages > 0) {
 		for (subi = 0; subi < img->numsubimages; ++subi) {
@@ -798,34 +789,26 @@ mac_init_cicn(Image *img)
 	}
 }
 
+	/* This function makes dummy calls to PlotCIcon.  This is absolutely
+	essential if one wishes to speed up and/or modify plotting by direct access
+	of the cicn PixMap and BitMap, as is now done everywhere in the code. 
+	The reason for this is that PlotCIcon not only plots the cicn, but also sets up
+	a pointer to the PixMap data at offset zero. If this has not been done at least
+	once, CopyBits cannot handle the cicn. */
+
 void
 mac_init_cicn_1(Image *img)
 {
-	CIconHandle cicnhandle;
-	MacImage *macimg;
-
-	macimg = (MacImage *) img->hook;
+	MacImage *macimg = (MacImage *) img->hook;
 	
 	/* First make sure we have a colricon! */
 	if (macimg->colricon == NULL) {
 		macimg->colricon = (Handle) mac_create_cicn(img);
 		macimg->cicn_inited = FALSE;
 	}
-	/* Then init the cicn if necessary. */
+	/* Init the PixMap pointer by a dummy PlotCIcon call. */
 	if (macimg->cicn_inited == FALSE) {
-		cicnhandle = (CIconHandle) macimg->colricon;
-
-		/* Hang 'em High! */
-		HLockHi((Handle) cicnhandle);
-		HLockHi((Handle) (*cicnhandle)->iconData);
-		HLockHi((Handle) (*cicnhandle)->iconPMap.pmTable);
-
-		/* Initialize PixMap pointer by this dummy call. */
-		PlotCIcon(NULL, cicnhandle);
-
-		/* Lock down stuff that got unlocked by PlotCIcon. */
-		HLock((Handle) (*cicnhandle)->iconData);
-		HLock((Handle) cicnhandle);
+		PlotCIcon(NULL, (CIconHandle) macimg->colricon);
 		macimg->cicn_inited = TRUE;
 	}
 }
@@ -849,7 +832,9 @@ interp_ctab(Image *img)
 	if (palette == lispnil && img->rawpalette == NULL)
 	  return nil;
 	len = (palette != lispnil ? length(palette) : img->numcolors);
-	ctabhandle = (CTabHandle) NewHandle(8 + len * 8);
+	/* Use xmalloc to avoid heap gymnastics. */
+	ctabhandle = (CTabHandle) NewEmptyHandle();
+	*ctabhandle = (ColorTable *) xmalloc(8 + len * 8);
 	HLock((Handle) ctabhandle);
 	(*ctabhandle)->ctFlags = 0;
 	(*ctabhandle)->ctSeed = GetCTSeed();
