@@ -7,9 +7,11 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING. */
 
 #include "conq.h"
+#include "macconq.h"
+
 extern void send_version PARAMS ((int rid));
 extern void debugging_string PARAMS ((char *buf));
-#include "macconq.h"
+extern void mac_init_cicn(Image *img);
 
 extern char *connection_method_name;
 
@@ -55,6 +57,9 @@ static int player_setup_dialog(void);
 static void set_player_setup_button_states(void);
 static pascal void draw_progress(WindowPtr win, short ditem);
 static void add_default_emblem_image(ImageFamily *imf, int e, char *name);
+static void dispose_terrain_images(void);
+static void dispose_emblem_images(void);
+static void dispose_unit_images(void);
 
 /* Global variables. */
 
@@ -1910,10 +1915,11 @@ player_setup_dialog()
 	playerh = 22;
 	playerbaseline = 14;
 	/* Always start out with the first player selected. */
-	selectedplayer = 0;
+	selectedplayer = 1;
 	set_player_setup_button_states();
 	draw_default_button(playersetupwin, diPlayerSetupOK);
 	/* This is a normal window dressed out as dialog. Handle accordingly. */
+	MakeDialogFloat(playersetupwin);
 	SelectTheWindow(playersetupwin);
 	return TRUE;
 }
@@ -1983,24 +1989,42 @@ handle_player_setup_event(EventRecord *event)
 			if (event->modifiers & cmdKey) {
 				if (event->what == keyDown) {
 					if (ch == '.') {
-						/* (should) Cancel out of player setup. */
-						beep();
+						mouse.h = mouse.v = 0;
+						hit_player_setup_dialog(diPlayerSetupCancel, mouse);
 					}
 				}
 			} else {
 				if (event->what == keyDown) {
-					if (ch == 3 || ch == 13) {
 						/* The mouse position doesn't actually get used here. */
 						mouse.h = mouse.v = 0;
-						if (hit_player_setup_dialog(diPlayerSetupOK, mouse))
-		  				  launch_game_2();
-					}
+					/* Return key. */
+					if (ch == 3 || ch == 13) {
+						hit_player_setup_dialog(diPlayerSetupOK, mouse);
+					/* Escape key. */
+					} else if (ch == 27) {
+						hit_player_setup_dialog(diPlayerSetupCancel, mouse);
 					/* Shortcuts to tweak the advantage. */
-					else if (ch == '+') {
+					} else if (ch == '+') {
 						adjust_advantage(selected_player(), selected_side(), 1);
 					} else if (ch == '-') {
 						adjust_advantage(selected_player(), selected_side(), -1);
+					/* Up arrow. */
+					} else if (ch == 0x1E) {
+						if (selectedplayer == 1)
+							select_player(numplayers - 1);
+						else	select_player(selectedplayer - 1);
+						set_player_setup_button_states();
+					/* Down arrow. */
+					} else if (ch == 0x1F) {
+						if (selectedplayer == numplayers - 1)
+							select_player(1);
+						else	select_player(selectedplayer + 1);
+						set_player_setup_button_states();
 					}
+					/* Make sure advantage and player changes are shown. */
+					set_player_setup_button_states();
+					DrawDialog(playersetupwin);
+					draw_default_button(playersetupwin, diPlayerSetupOK);
 				}
 			}
 			break;
@@ -2011,18 +2035,24 @@ handle_player_setup_event(EventRecord *event)
 	}
 }
 
-int
-hit_player_setup_dialog(int ditem, Point pt)
+void
+hit_player_setup_dialog(int ditem, Point mouse)
 {
-	int playersok = FALSE, done = FALSE, changedselected = FALSE, changedall = FALSE;
-	int willhost = FALSE;
+	int playersok = FALSE, done = FALSE, changedselected = FALSE, changedall = FALSE, willhost = FALSE;
 	Player *player, *tmpplayer;
+	Handle itemhandle;
+	short itemtype;
+	Rect itemrect;
 	Side *tmpside;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
+	GrafPtr oldport;
 
 	tmpplayer = selected_player();
 	tmpside = selected_side();
 
+	GetPort(&oldport);
+	SetPort(playersetupwin);
+	GlobalToLocal(&mouse);
+
 	switch (ditem) {
 		case diPlayerSetupOK:
 			playersok = TRUE;
@@ -2041,8 +2071,7 @@ hit_player_setup_dialog(int ditem, Point pt)
 			add_side_and_player();
 			/* Select the newly-created side/player pair. */
 			select_player(numsides - 1);
-			init_emblem_images(tmpside);
-			reinit_all_emblem_images();
+			init_emblem_images();
 			changedall = TRUE;
 			break;
 		case diPlayerSetupRemove:
@@ -2059,7 +2088,7 @@ hit_player_setup_dialog(int ditem, Point pt)
 			tmpside->pluralnoun = NULL;
 			tmpside->adjective = NULL;
 			make_up_side_name(tmpside);
-			reinit_all_emblem_images();
+			init_emblem_images();
 			changedselected = TRUE;
 			break;
 		case diPlayerSetupAI:
@@ -2097,14 +2126,14 @@ hit_player_setup_dialog(int ditem, Point pt)
 			break;
 		case diPlayerSetupList:
 			GetDItem(playersetupwin, diPlayerSetupList, &itemtype, &itemhandle, &itemrect);
-			select_player((pt.v - itemrect.top) / playerh + 1);
+			select_player((mouse.v - itemrect.top) / playerh + 1);
 			if (tmpplayer != selected_player())
 			  changedselected = TRUE;
 			break;
 		case diPlayerSetupAdvantage:
 			GetDItem(playersetupwin, diPlayerSetupAdvantage, &itemtype, &itemhandle, &itemrect);
 			if (tmpplayer != NULL) {
-				if (pt.v - itemrect.top < ((itemrect.bottom - itemrect.top) / 2)) {
+				if (mouse.v - itemrect.top < ((itemrect.bottom - itemrect.top) / 2)) {
 					changedselected = adjust_advantage(tmpplayer, tmpside, 1);
 				} else {
 					changedselected = adjust_advantage(tmpplayer, tmpside, -1);
@@ -2139,7 +2168,11 @@ hit_player_setup_dialog(int ditem, Point pt)
 			}
 		}
 	}
-	return playersok;
+	SetPort(oldport);
+	/* Call launch_game_2 directly instead of returning playersok 
+	to the caller, which then calls launch_game_2. */ 
+	if (playersok)
+		launch_game_2();
 }
 
 void
@@ -2252,18 +2285,13 @@ launch_game()
 		/* Cancelled from variants dialog, go back to wherever. */
 		return FALSE;
 	}
-	/* Set up the images we'll be needing. */
-	init_terrain_images();
 	init_side_colors();
 	/* Trial side/player assignment, will create the default player if nececessary. */
 	make_trial_assignments();
-	/* Get the emblems so they can be displayed in the player dialog. */
-	init_all_emblem_images();
-	init_emblem_cicns();
-	/* It is important that unit cicns are loaded after the emblems since functions
-	that reinit cicns assume that emblems always are located above the unit cicns. */
+	/* Set up the images we'll be needing. */
+	init_terrain_images();
+	init_emblem_images();
 	init_unit_images();
-
 	/* Ask for modifications to the trial assignment. */
 	return player_setup_dialog();
 }
@@ -2612,14 +2640,16 @@ init_display()
 	Dprintf("Opened Mac display!\n");
 }
 
-/* Allocate and fill in images for all the unit types. */
-
 void
 init_unit_images()
 {
 	int u;
 	ImageFamily *imf;
 
+	/* First trash the old images if they exist. */
+	if (uimages) {
+		dispose_unit_images();
+	}
 	uimages = (ImageFamily **) xmalloc(numutypes * sizeof(ImageFamily *));
 
 	for_all_unit_types(u) {
@@ -2628,10 +2658,27 @@ init_unit_images()
 		  describe_imf(dside, "unit type", u_type_name(u), imf);
 		uimages[u] = imf;
 	}
-	init_unit_cicns();
 }
 
-/* Set up the collection of terrain image patterns and colors. */
+void
+init_emblem_images()
+{
+	Side *side2;
+	int s2;
+	
+	/* First trash the old images if they exist. */
+	if (eimages) {
+		dispose_emblem_images();
+	}
+	/* Allocate enough space to account for both initial sides, and for ones that
+       may be added later. */
+	eimages = (ImageFamily **) xmalloc((g_sides_max() + 1) * sizeof(ImageFamily *));
+
+	for_all_sides_plus_indep(side2) {
+		s2 = side_number(side2);
+		eimages[s2] = get_emblem_images(dside, side2);
+	}
+}
 
 void
 init_terrain_images()
@@ -2644,7 +2691,11 @@ init_terrain_images()
 	CTabHandle ctabhandle;
 	ColorSpec *ctdata;
 
-	/* First load terrain images. */
+	/* First trash the old images if they exist. */
+	if (timages) {
+		dispose_terrain_images();
+	}
+	/* Then load terrain images. */
 	timages = (ImageFamily **) xmalloc(numttypes * sizeof(ImageFamily *));
 	for_all_terrain_types(t) {
 		imf = get_terrain_type_images(dside, t);
@@ -2652,7 +2703,6 @@ init_terrain_images()
 		  describe_imf(dside, "terrain type", t_type_name(t), imf);
 		timages[t] = imf;
 	}
-
 	/* Then optionally load terrain colors. */
 	if (hasColorQD) {
 
@@ -2690,131 +2740,8 @@ init_terrain_images()
 	get_unseen_images(dside);
 }
 
-/* Given a color name, get actual RGB values for it.  As a special case for b/w
-   screens, the five shades of gray also get set here. */
-
 void
-interp_named_color(char *name, enum grays *grayvar, RGBColor *colorvar)
-{
-	Image *tmpimg;
-
-	if (strcmp(name, "black") == 0) {
-		*grayvar = blackgray;
-		colorvar->red = colorvar->green = colorvar->blue = 0;
-	} else if (strcmp(name, "dark gray") == 0) {
-		*grayvar = darkgray;
-		colorvar->red = colorvar->green = colorvar->blue = 16384;
-	} else if (strcmp(name, "gray") == 0) {
-		*grayvar = mediumgray;
-		colorvar->red = colorvar->green = colorvar->blue = 32768;
-	} else if (strcmp(name, "light gray") == 0) {
-		*grayvar = lightgray;
-		colorvar->red = colorvar->green = colorvar->blue = 49000;
-	} else if (strcmp(name, "white") == 0) {
-		*grayvar = whitegray;
-		colorvar->red = colorvar->green = colorvar->blue = 65535;
-	} else if (0 /* hasColorQD */) {
-#if 0
-		tmpcolor.name = name;
-		mac_load_image_color(&tmpcolor);
-		colorvar->red = tmpcolor.r;
-		colorvar->green = tmpcolor.g;
-		colorvar->blue = tmpcolor.b;
-#endif
-	}
-}
-
-/* Load/create all the images of each side's emblem. */
-
-void
-init_all_emblem_images()
-{
-	Side *side2;
-
-	/* Allocate enough space to account for both initial sides, and for ones that
-       may be added later. */
-	eimages = (ImageFamily **) xmalloc((g_sides_max() + 1) * sizeof(ImageFamily *));
-
-	for_all_sides_plus_indep(side2) {
-		init_emblem_images(side2);
-	}
-}
-
-void
-reinit_all_emblem_images()
-{
-	dispose_emblem_cicns();
-
-	/* The emblem cicns are locked in place above the unit cicns, which are
-	also locked. To reinit the emblem cicns in their correct position we must 
-	therefore first dispose of the unit cicns and then reinit them after we are
-	done with the emblem cicns. */
-
-	/* Also free the emblem images. */
-	free(eimages);
-
-	/* Dispose of and free unit images. */
-	dispose_unit_cicns();
-	free(uimages);
-
-	/* This isn't great, because all the collected data goes away, and we have to
-	   reread and reallocate everything.  It would be smarter just to reinit the
-	   Mac-specific parts alone. */
-	init_all_emblem_images();
-	init_emblem_cicns();
-
-	/* Finally load the unit cicns again below the new emblems. */
-	init_unit_images();
-}
-
-/* Set up the emblems for a particular side.  This is used both to do all sides
-   initially and whenever a side changes emblems. */
-
-void
-init_emblem_images(Side *side2)
-{
-	int s2 = side_number(side2);
-
-	eimages[s2] = get_emblem_images(dside, side2);
-}
-
-/* The purpose of these 4 functions is to ensure the current cicn set is properly
-   initialized through a dummy PlotCIcon call.  */
-
-/* Init the unit cicns and lock them at the top of the heap zone. */
-
-void mac_init_cicn(Image *img);
-
-void
-init_unit_cicns()
-{
-	int u;
-	Image *img;
-	
-	for_all_unit_types(u) {
-		for_all_images(uimages[u], img) {
-			if (img && img->hook) {
-				mac_init_cicn(img);
-			}
-		}
-
-		/* The purpose of these dummy calls is to avoid later calls to
-		mac_init_cicn during the game which will mess up the heap. 
-		In particular, calls to best_image during offscreen drawing 
-		will put locked cicn handles all over the heap, since the gworld 
-		pixmap is locked in place at drawing time. */
-
-		/* Trigger add_shrunken_image once and for all. */
-		img = best_image(uimages[u], 1, 1);
-		/* Trigger add_magnified_image once and for all. */
-		img = best_image(uimages[u], 128, 128);
-	}
-}
-
-/* Trash the unit cicns. */
-
-void
-dispose_unit_cicns()
+dispose_unit_images()
 {
 	int u;
 	CIconHandle cicnhandle;
@@ -2836,40 +2763,11 @@ dispose_unit_cicns()
 			}
 		}
 	}
+	free(uimages);
 }
 
-/* Init the emblem cicns and lock them at the top of the heap zone. */
-
 void
-init_emblem_cicns()
-{
-	Side *side;
-	Image *img;
-	
-	for_all_sides_plus_indep(side) {
-		for_all_images(eimages[side_number(side)], img) {
-			if (img && img->hook) {
-				mac_init_cicn(img);
-			}
-		}
-
-		/* The purpose of these dummy calls is to avoid later calls to
-		mac_init_cicn during the game which will mess up the heap. 
-		In particular, calls to best_image during offscreen drawing 
-		will put locked cicn handles all over the heap, since the gworld 
-		pixmap is locked in place at drawing time. */
-
-		/* Trigger add_shrunken_image once and for all. */
-		img = best_image(eimages[side_number(side)], 1, 1);
-		/* Trigger add_magnified_image once and for all. */
-		img = best_image(eimages[side_number(side)], 128, 128);
-	}
-}
-
-/* Trash the emblem cicns, thus clearing up the high part of the heap. */
-
-void
-dispose_emblem_cicns()
+dispose_emblem_images()
 {
 	Side *side;
 	CIconHandle cicnhandle;
@@ -2891,4 +2789,31 @@ dispose_emblem_cicns()
 			}
 		}
 	}
+	free(eimages);
+}
+
+void
+dispose_terrain_images()
+{
+	int t;
+	CIconHandle cicnhandle;
+	MacImage *macimg;
+	Image *img;
+	
+	for_all_terrain_types(t) {
+		for_all_images(timages[t], img) {
+			if (img && img->hook) {
+				macimg = (MacImage *) img->hook;
+				if (macimg->colricon != NULL) {
+					cicnhandle = (CIconHandle) macimg->colricon;
+					DisposHandle((Handle) (*cicnhandle)->iconPMap.pmTable);
+					DisposHandle((Handle) (*cicnhandle)->iconData);
+					DisposHandle((Handle) cicnhandle);
+					macimg->colricon = nil;
+					macimg->cicn_inited = FALSE;
+				}
+			}
+		}
+	}
+	free(timages);
 }
