@@ -29,8 +29,7 @@ any later version.  See the file COPYING.  */
 #define MAXWINDOWS 100
 
 extern void enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit);
-extern void global_advance_dialog(Side *side, Unit *unit, int advance);
-extern void city_dialog(Map *map, Unit *unit);
+extern void side_research_dialog(Side *side, Unit *unit, int advance);
 extern void do_closeup_mi(void);				/* Needed in macmouse.c. */			
 
 extern int do_down_command(void);
@@ -44,7 +43,6 @@ extern int do_one_dir_move(Unit *unit);
 extern int do_one_dir_multiple_move(Unit *unit);
 extern int do_one_set_name(Unit *unit);
 
-static void do_city_dialog_mi(void);
 static void resign_the_game(int forced);
 static void te_cut(TEHandle hdl);
 static void te_copy(TEHandle hdl);
@@ -664,9 +662,6 @@ do_menu_command(long which)
 				case miPlayCloseup:
 					do_closeup_mi();
 					break;
-				case miPlayCityDialog:
-					do_city_dialog_mi();
-					break;
 				case miPlayMove:
 					query_position_modally(MOVE_TO_MODAL);
 					break;
@@ -812,7 +807,7 @@ do_menu_command(long which)
 					show_command_window();
 					break;
 				case miWindowsResearch:
-					global_advance_dialog(dside, NULL, -1);
+					side_research_dialog(dside, NULL, -1);
 					break;
 				case miWindowsScores:
 					show_scores_window();
@@ -1637,22 +1632,6 @@ apply_to_all_selected(int (*fn)(Unit *unit), int beepfailure)
 	  beep();
 }
 
-static void
-do_city_dialog_mi()
-{
-	int i;
-	Unit *unit;
-
-	if (frontmap) {
-		for (i = 0; i < frontmap->numselections; ++i) {
-			if ((unit = frontmap->selections[i]) != NULL 
-			       && u_advanced(unit->type)) {
-				city_dialog(frontmap, unit);				
-			}
-		}
-	}
-}
-
 void		/* Now declared extern. */
 do_closeup_mi()
 {
