@@ -16,7 +16,6 @@ any later version.  See the file COPYING.  */
 extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
 
 extern void set_position_modally(void);
-extern void city_dialog(Map *map, Unit *unit);
 extern void do_closeup_mi(void);
 
 static int selrect;
@@ -841,20 +840,16 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		}
 	} else if (mods & controlKey) {
 		m_nearest_unit(map, h, v, &unit);
-		/* Open city dialog or closeup if clicking on a unit that we may examine. */
+		/* Open closeup if clicking on a unit that we may examine. */
 		if (side_sees_unit(dside, unit)) {
 
 			UnitCloseup *unitcloseup;
 
-			if (u_advanced(unit->type)) {
-				city_dialog(map, unit);
-			} else { 	
 				if ((unitcloseup = find_unit_closeup(unit)) != NULL) {
 					SelectTheWindow(unitcloseup->window);
 				} else {
 					create_unit_closeup(unit);
 				}
-			}
 		/* Else focus on clicked spot and select closest own unit. */
 		} else {
 			long	unitdist, mindist = 32000;
