@@ -1169,15 +1169,8 @@ set_preferences()
 	DisposeDialog(win);
 	update_all_map_windows();
 	/* Reflect some kinds of changes immediately. */
-	if (pref_color_unit_images != oldcolor) {
-		/* Get rid of old images and collect new ones. */
-		/* (should be more effective and GC the images themselves too) */
- 		dispose_unit_cicns();
-		/* dispose_unit_cicns accesses uimages, so don't free until after that call. */
- 		free(uimages);
-	  	init_unit_images();
-	}
-
+	if (pref_color_unit_images != oldcolor)
+	  init_unit_images();
 	if (fonts_changed)
 	  recalc_spacing();
 	if (pref_color_unit_images != oldcolor
