@@ -906,7 +906,7 @@ draw_connections(Map *map, int x, int y, int c)
 {
 	int xw = wrapx(x), dir, sx, sy, bitmask = 0, over;
 	
-	if (!(map->see_all || terrain_visible(dside, xw, y)) || !any_connections_at(xw, y, c))
+	if (!(map->see_all || terrain_visible(dside, xw, y)) || !any_connections_at(xw, y, c) || !tt_drawable(c, terrain_at(xw, y)))
 	  return;
 	for_all_directions(dir) {
 		if (connection_at(xw, y, dir, c)) {
