@@ -11,6 +11,14 @@ extern char *curdatestr;
 extern char *goal_desc(char *buf, Goal *goal);
 extern int update_total_hist_lines(Side *side);
 extern int build_hist_contents(Side *side, int n, HistEvent **histcontents, int numvis);
+extern int has_advance_to_research(Side *side, int atype);
+extern void side_research_dialog(Side *side, Unit *unit, int advance);			
+
+static int get_selected_construction_advance();
+static void update_unit_list_for_advance(int a);
+static void select_advance_in_construction_window(int a);
+static void update_construction_advance_list();
+static void update_advance_list_for_unit(Unit *unit);
 
 #define unit_doctrine(unit)  \
   (unit->side->udoctrine[unit->type])
@@ -63,6 +71,7 @@ WindowPtr constructionwin = nil;
 
 static ListHandle construction_unit_list = nil;
 static ListHandle construction_type_list = nil;
+static ListHandle construction_advance_list = nil;
 
 static int maxtypewid;
 static int constructmargin = 5;
@@ -71,16 +80,21 @@ static int constructtop = 32;
 /* This is the vector of units that can do construction or development. */
 
 static UnitVector *constructor_units = NULL;
+static UnitVector *research_units = NULL;
 
 static int numposstypes;
 static int *possibletypes = NULL;
 
+static int numpossadvances;
+static int *possibleadvances = NULL;
+
 int currunlength;
 
 int editedrunlength = -1;
 
 /* (this is referenced from the end-turn command, so can't make static) */
 ControlHandle constructbutton;
+ControlHandle researchbutton;
 static ControlHandle developbutton;
 
 TEHandle run_length_text = nil;
@@ -88,18 +102,7 @@ TEHandle run_length_text = nil;
 static Rect runlengthrect;
 static Rect unitlistrect;
 static Rect typelistrect;
-
-/* Globals for unit closeup windows. */
-
-int lastunitcloseuph = -1, lastunitcloseupv = -1;
-
-static int m_per_row = 2;
-
-static char *nummrows;
-
-int closeup_spacing = 16;
-
-int closeupwinwid = 240;
+static Rect advancelistrect;
 
 /* Globals for the history window. */
 
@@ -580,7 +583,8 @@ create_construction_window()
 		constructionwin = GetNewWindow(wConstruction, NULL, NULL);
 	}
 	constructbutton = GetNewControl(cConstructButton, constructionwin);
-	developbutton = GetNewControl(cResearchButton, constructionwin);
+	developbutton = GetNewControl(cDevelopButton, constructionwin);
+	researchbutton = GetNewControl(cResearchButton, constructionwin);
 	SetPort(constructionwin);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
@@ -606,6 +610,15 @@ create_construction_window()
 	construction_type_list =
 		LNew(&tmprect, &listrect, cellsize, 128, constructionwin,
 			 FALSE, FALSE, FALSE, TRUE);
+	/* Now set up the list of advances. */
+	tmprect = advancelistrect;
+	tmprect.right -= sbarwid;
+	SetRect(&listrect, 0, 0, 1, 0);
+	/* (should calc this from the desired font) */
+	SetPt(&cellsize, 300, 12);
+	construction_advance_list =
+		LNew(&tmprect, &listrect, cellsize, 128, constructionwin,
+			 FALSE, FALSE, FALSE, TRUE);
 	init_construction_lists();
 	if (1 /* position construction at bottom of main screen */) {
 		get_main_screen_size(NULL, &mainheight);
@@ -622,7 +635,7 @@ create_construction_window()
 void
 init_construction_lists()
 {
-	int u;
+	int a, u;
 	Unit *unit;
 	Cell tmpcell;
 
@@ -658,6 +671,26 @@ init_construction_lists()
 		}
 	}
 	LDoDraw(1, construction_type_list);
+
+	/* Update the list of advances. */
+	LDoDraw(0, construction_advance_list);
+	LDelRow(0, 0, construction_advance_list);
+	SetPt(&tmpcell, 0, 0);
+	if (possibleadvances == NULL && numatypes > 0)
+	  possibleadvances = (int *) xmalloc(numatypes * sizeof(int));
+	numpossadvances = 0;
+	for_all_advance_types(a) {
+		if (has_advance_to_research(dside, a)
+		     &! has_advance(dside, a)) {
+			LAddRow(1, tmpcell.v, construction_advance_list);
+			researchible_desc(spbuf, get_selected_construction_unit(), a);
+			LSetCell(spbuf, strlen(spbuf), tmpcell, construction_advance_list);
+			++tmpcell.v;
+			possibleadvances[numpossadvances++] = a;
+		}
+	}
+	LDoDraw(1, construction_advance_list);
+
 	adjust_construction_controls();
 }
 
@@ -700,6 +733,10 @@ draw_construction()
 	tmprect = typelistrect;
 	InsetRect(&tmprect, -1, -1);
 	FrameRect(&tmprect);
+	LUpdate(constructionwin->visRgn, construction_advance_list);
+	tmprect = advancelistrect;
+	InsetRect(&tmprect, -1, -1);
+	FrameRect(&tmprect);
 	/* Maybe show the construct button as the default. */
 	draw_construction_default();
 }
@@ -730,7 +767,7 @@ draw_construction_default()
 void
 calc_construction_rects()
 {
-	int wid, hgt, divide;
+	int wid, hgt, divide1, divide2;
 	Rect tmprect;
 
 	/* (should compute based on num chars needed, plus font size) */
@@ -741,15 +778,19 @@ calc_construction_rects()
 	runlengthrect.right -= 20;  runlengthrect.bottom = 25;
 	wid = tmprect.right - tmprect.left - sbarwid;
 	hgt = tmprect.bottom - tmprect.top - sbarwid;
-	if (wid / 2 > maxtypewid) {
-		divide = wid - maxtypewid;
+	if (wid / 3 > maxtypewid) {
+		divide1 = wid - 2 * maxtypewid;
+		divide2 = wid - maxtypewid;
 	} else {
-		divide = wid / 2;
+		divide1 = wid / 3;
+		divide2 = 2 * wid / 3;
 	}
-	SetRect(&unitlistrect, 0, constructtop, divide, hgt);
+	SetRect(&unitlistrect, 0, constructtop, divide1, hgt);
 	InsetRect(&unitlistrect, constructmargin, constructmargin);
-	SetRect(&typelistrect, divide, constructtop, wid, hgt);
+	SetRect(&typelistrect, divide1, constructtop, divide2, hgt);
 	InsetRect(&typelistrect, constructmargin, constructmargin);
+	SetRect(&advancelistrect, divide2, constructtop, wid, hgt);
+	InsetRect(&advancelistrect, constructmargin, constructmargin);
 }
 
 void
@@ -761,6 +802,7 @@ activate_construction(int activate)
 	  TEDeactivate(run_length_text);
 	LActivate(activate, construction_unit_list);
 	LActivate(activate, construction_type_list);
+	LActivate(activate, construction_advance_list);
 }
 
 Unit *
@@ -794,6 +836,20 @@ get_selected_construction_type()
 	return NONUTYPE;
 }
 
+int
+get_selected_construction_advance()
+{
+	Point tmpcell;
+
+	SetPt(&tmpcell, 0, 0);
+	if (LGetSelect(TRUE, &tmpcell, construction_advance_list)) {				
+		if (tmpcell.v < numpossadvances) {
+			return possibleadvances[tmpcell.v];
+		}
+	}
+	return NONATYPE;
+}
+
 void
 scroll_to_selected_construction_unit()
 {
@@ -816,7 +872,7 @@ do_mouse_down_construction(Point mouse, int mods)
 {
 	ControlHandle control;
 	short part;
-	int u;
+	int a, u;
 	Unit *unit;
 	extern int modal_construction;
 	extern WindowPtr window_behind_construction;
@@ -827,7 +883,7 @@ do_mouse_down_construction(Point mouse, int mods)
 		if (unit != NULL) {
 			u = get_selected_construction_type();
 			if (u != NONUTYPE) {
-				net_push_build_task(unit, u, currunlength);
+				net_push_build_task(unit, u, currunlength, 0, 0);
 				update_construction_unit_list(unit);
 				if (modal_construction && window_behind_construction != nil) {
 					SelectTheWindow(window_behind_construction);
@@ -854,6 +910,22 @@ do_mouse_down_construction(Point mouse, int mods)
 				return;
 			}
 		}
+	} else if (control == researchbutton) {
+		unit = get_selected_construction_unit();
+		if (unit != NULL) {
+			a = get_selected_construction_advance();
+			if (a != NONATYPE) {
+				net_set_unit_curadvance(unit->side, unit, a);
+				update_construction_unit_list(unit);
+				if (modal_construction && window_behind_construction != nil) {
+					SelectTheWindow(window_behind_construction);
+					update_window(window_behind_construction);
+				}
+				window_behind_construction = NULL;
+				modal_construction = FALSE;
+				return;
+			}
+		}
 	} else if (PtInRect(mouse, &runlengthrect)) {
 		TEClick(mouse, mods, run_length_text);
 		/* (should switch this to be current item) */
@@ -861,10 +933,15 @@ do_mouse_down_construction(Point mouse, int mods)
 		LClick(mouse, mods, construction_unit_list);
 		/* Update the type list to show what could be built and in how long. */
 		update_type_list_for_unit(get_selected_construction_unit());
+		update_advance_list_for_unit(get_selected_construction_unit());
 	} else if (PtInRect(mouse, &typelistrect)) {
 		LClick(mouse, mods, construction_type_list);
 		/* Update the unit list to show what could build the type */
 		update_unit_list_for_type(get_selected_construction_type());
+	} else if (PtInRect(mouse, &advancelistrect)) {
+		LClick(mouse, mods, construction_advance_list);
+		/* Update the unit list to show what could build the type */
+		update_unit_list_for_advance(get_selected_construction_advance());
 	} else {
 		/* Click was not in any useful part of the window. */ 
 	}
@@ -948,6 +1025,22 @@ select_type_in_construction_window(int u)
 	update_unit_list_for_type(get_selected_construction_type());
 }
 
+void
+select_advance_in_construction_window(int a)
+{
+	int i;
+	Point tmpcell;
+
+	for (i = 0; i < numpossadvances; ++i) {
+		SetPt(&tmpcell, 0, i);
+		LSetSelect((a == possibleadvances[i]), tmpcell, construction_advance_list);
+		LAutoScroll(construction_advance_list);
+	}
+	if (a == NONATYPE)
+	  return;
+	update_unit_list_for_advance(get_selected_construction_advance());
+}
+
 /* Given a unit (which may be any unit), update the list of constructing units. */
 
 void
@@ -965,7 +1058,7 @@ update_construction_unit_list(Unit *unit)
 		if (unit == (constructor_units->units)[i].unit) {
 			SetPt(&tmpcell, 0, i);
 			if (is_active(unit)
-				&& can_build_or_help(unit)
+				&& (can_build_or_help(unit) || can_research(unit))
 				&& side_controls_unit(dside, unit)) {
 				construction_desc(spbuf, unit, u);
 				LSetCell(spbuf, strlen(spbuf), tmpcell, construction_unit_list);
@@ -986,7 +1079,7 @@ maybe_add_unit_to_construction_list(Unit *unit)
 	Point tmpcell;
 
 	if (is_active(unit)
-		&& can_build_or_help(unit)
+		&& (can_build_or_help(unit) || can_research(unit))
 		&& side_controls_unit(dside, unit)) {
 		/* Add this unit to the vector of constructing units. */
 		constructor_units = add_unit_to_vector(constructor_units, unit, 0);
@@ -1022,6 +1115,29 @@ update_unit_list_for_type(int u)
 	adjust_construction_controls();
 }
 
+void
+update_unit_list_for_advance(int a)
+{
+	int i;
+	Point tmpcell;
+	Unit *unit;
+
+	for (i = 0; i < constructor_units->numunits; ++i) {
+		unit = (constructor_units->units)[i].unit;
+		if (unit != NULL) {
+			SetPt(&tmpcell, 0, i);
+			if (is_active(unit) && unit->side == dside) {
+				research_desc(spbuf, unit, a);
+				LSetCell(spbuf, strlen(spbuf), tmpcell, construction_unit_list);
+			} else {
+/*				LDelRow(1, tmpcell.v, construction_unit_list); */
+				LSetCell("", 0, tmpcell, construction_unit_list);
+			}
+		}
+	}
+	adjust_construction_controls();
+}
+
 void
 update_construction_type_list()
 {
@@ -1047,13 +1163,38 @@ update_type_list_for_unit(Unit *unit)
 	adjust_construction_controls();
 }
 
+void
+update_construction_advance_list()
+{
+	int a;
+
+	if (constructionwin == nil)
+	  return;
+	a = get_selected_construction_advance();
+	update_advance_list_for_unit(get_selected_construction_unit());
+}
+
+void
+update_advance_list_for_unit(Unit *unit)
+{
+	int i;
+	Point tmpcell;
+
+	for (i = 0; i < numpossadvances; ++i) {
+		researchible_desc(spbuf, get_selected_construction_unit(), possibleadvances[i]);
+		SetPt(&tmpcell, 0, i);
+		LSetCell(spbuf, strlen(spbuf), tmpcell, construction_advance_list);
+	}
+	adjust_construction_controls();
+}
+
 /* Enable/disable controls according to whether the selected list elements can
    do construction activities. */
 
 void
 adjust_construction_controls()
 {
-	int u, canconstruct = FALSE, candevelop = FALSE, len;
+	int a, u, canconstruct = FALSE, candevelop = FALSE, canresearch = FALSE, len;
 	Unit *unit;
 
 	unit = get_selected_construction_unit();
@@ -1067,9 +1208,18 @@ adjust_construction_controls()
 			if (uu_acp_to_develop(unit->type, u) > 0)
 			     candevelop = TRUE;
 		}
+		a = get_selected_construction_advance();
+		if (a != NONATYPE) {
+			if (u_advanced(unit->type)
+			    && has_advance_to_research(dside, a)
+		     	    &! has_advance(dside, a)) {
+				canresearch = TRUE;
+			}
+		}
 	}
 	HiliteControl(constructbutton, (canconstruct ? 0 : 255));
 	HiliteControl(developbutton, (candevelop ? 0 : 255));
+	HiliteControl(researchbutton, (canresearch ? 0 : 255));
 	draw_construction_default();
 	/* If there is doctrine on construction run length, use it to seed the
 	   run length in the dialog. */
@@ -1159,6 +1309,359 @@ adjust_construction_items()
 		  construction_type_list);
 	SetPt(&cellsize, max(300, listwid), 12);
 	LCellSize(cellsize, construction_type_list);
+	/* Move the advance list (is this the approved way to do it?) */
+	(*construction_advance_list)->rView.left = advancelistrect.left;
+	listwid = advancelistrect.right - advancelistrect.left - sbarwid;
+	LSize(listwid, advancelistrect.bottom - advancelistrect.top,
+		  construction_advance_list);
+	SetPt(&cellsize, max(300, listwid), 12);
+	LCellSize(cellsize, construction_advance_list);
+}
+
+/*
+	Research dialog for all advanced units belonging to a side. Unit is the city that just made a
+	discovery and advance the discovery itself. Set to NULL and (-1) if no discovery was made. 
+	This is the old global_advance_dialog which was renamed and moved here from macadv.c.
+*/
+
+void
+side_research_dialog(Side *side, Unit *unit, int advance)
+{
+	short 		ditem, mitem, menu, num, text, status, check, done = FALSE, disabled, i, m, s;
+	char			cname[32], sname[32], buf[BUFSIZE];
+	int			oldprojects[MAXPROJMENUS + 1];
+	int			participants[MAXATYPES];	
+	Str255 		pname, pnumber;
+	int			numprojects = 0;
+	int			maxprojects;
+	int			loafers = 0;	
+	Handle 		itemhandle; 
+	MenuHandle	doneMenu;
+	MenuHandle	popMenu;
+	GrafPtr 		oldport;
+	Unit 			*unit2;
+ 	DialogPtr		win;
+	
+	/* Crash bug fix. */
+	if (!side)
+	  return;
+	memset(oldprojects, 0, (MAXPROJMENUS + 1) * sizeof(int));
+	memset(participants, 0, MAXATYPES * sizeof(int));
+
+	/* Always give the own side, designers and debuggers full control. */
+	if ((side && side == dside) || Debug || DebugG || DebugM || is_designer(dside)) {
+		disabled = FALSE;
+	/* Allow peeking at trusted side but with controls disabled. */
+	} else if (trusted_side(dside, side)) {
+		disabled = TRUE;
+	/* Allow peeking at enemy if it is permitted, else return. */ 
+	} else if (g_peek_at_enemy() == TRUE) {
+		disabled = TRUE;
+	} else return;
+
+	/* Open the dialog. */
+	win = GetNewDialog(dGlobalAdvance, NULL, (DialogPtr) -1);
+	GetPort(&oldport);	
+	SetPort(win);
+
+	/* Hide everything except main text. */
+	HideDItem(win, diGlobalMadScientist);
+	HideDItem(win, diGlobalBusyScientist);
+	HideDItem(win, diGlobalIdleScientist);
+	for (i = diGlobalIdleMenu; i < diGlobalIdleMenu + 4 * (MAXPROJMENUS + 1); i++)
+		HideDItem(win, i);
+	for (i = diGlobalIdleCheck; i <= diGlobalIdleCheck + MAXPROJMENUS; i++)
+		HideDItem(win, i);
+	HideDItem(win, diGlobalMoreNote);
+	
+	/* Count loafers and project participants. */
+	for_all_side_units(side, unit2) {
+		if (!u_advanced(unit2->type))
+			continue;
+		if (unit2->curadvance != NOADVANCE && unit2->curadvance != advance)
+			participants[unit2->curadvance] += 1;				
+		else loafers += 1;
+	}
+	/* Count the number of active projects. */
+	for_all_advance_types(s) {
+		if (participants[s] > 0) {
+			numprojects += 1;
+			oldprojects[numprojects] = s;
+		}
+	}
+	maxprojects = min(numprojects, MAXPROJMENUS);
+
+	/* Load achieved advances into done menu. */
+	doneMenu = GetMenu(mAdvanceAchieved);
+	for_all_advance_types(s) {
+		if (side->advance[s] == DONE) {
+			c2p(a_type_name(s), pname);
+			AppendMenu(doneMenu, pname);
+		}
+	}
+	/* Load available advances into popup menu. */
+	popMenu = GetMenu(mResearchPopup);
+	for_all_advance_types(s) {
+		if (has_advance_to_research(side, s)
+		    /* Skip advances already done. */
+		    && side->advance[s] != DONE) {
+			c2p(a_type_name(s), pname);
+			AppendMenu(popMenu, pname);
+		}
+	}
+	m = CountMItems(popMenu);
+
+	/* Show mad scientist and relevant text if new discovery was signaled. */
+	if (advance != NOADVANCE) {
+		ShowDItem(win, diGlobalMadScientist);
+		if (side == dside)
+			strcpy(buf, "Your");
+		else	strcpy(buf, side_adjective(side));
+		strcat(buf, " scientists in ");
+		strcat(buf, unit->name);
+		strcat(buf, " discover ");
+		strcat(buf,  a_type_name(advance));
+		strcat(buf, "!\r\r");
+		strcat(buf, "Pick new advance to research:");
+	/* Else show busy scientist and report ongoing research. */
+	} else {
+		ShowDItem(win, diGlobalBusyScientist);
+		if (side == dside)
+			strcpy(buf, "Your");
+		else	strcpy(buf, side_adjective(side));
+			strcat(buf, " scientists are busy with the following projects:");
+		if (loafers)
+			strcat(buf, "\r\rNote : some of them are idle. You should put them to work!");
+	}
+	/* Show the main text. */
+	c2p(buf, pname);
+	GetDItem(win, diGlobalMainText, NULL, &itemhandle, NULL);
+	SetIText(itemhandle, pname);
+
+	/* Maybe show warning about more projects than shown. */
+	if (numprojects > MAXPROJMENUS)
+		ShowDItem(win, diGlobalMoreNote);
+
+	if (loafers) {
+		/* Show first menu (idle units) only if needed. */
+		GetDItem(win, diGlobalIdleNum, NULL, &itemhandle, NULL);
+		NumToString(loafers, pnumber);
+		SetIText(itemhandle, pnumber);
+		ShowDItem(win, diGlobalIdleNum);
+		ShowDItem(win, diGlobalIdleText);
+		GetDItem(win, diGlobalIdleMenu, NULL, &itemhandle, NULL);
+		ShowDItem(win, diGlobalIdleMenu);
+		/* Don't allow other sides to use the menu. */
+		if (disabled)
+			HiliteControl((ControlHandle) itemhandle, 255); 
+		/* Also show first checkbox (idle units) if needed. */
+		GetDItem(win, diGlobalIdleCheck, NULL, &itemhandle, NULL);
+		SetCtlValue((ControlHandle) itemhandle, TRUE);
+		for_all_side_units(side, unit2) {
+			if (!u_advanced(unit2->type))
+				continue;
+			if (unit2->curadvance != NOADVANCE && unit2->curadvance != advance)
+				continue;
+			if (unit2->autoresearch != TRUE)
+				SetCtlValue((ControlHandle) itemhandle, FALSE);
+		}
+		ShowDItem(win, diGlobalIdleCheck);
+		/* Don't allow other sides to use the checkbox. */
+		if (disabled)
+			HiliteControl((ControlHandle) itemhandle, 255); 
+	}
+	/* Show project popup menus and associated text items. */
+	num = diGlobalIdleNum;
+	status = diGlobalIdleStatus;
+	check = diGlobalIdleCheck;
+	text = diGlobalIdleText;
+	menu = diGlobalIdleMenu;
+	for (s = 1; s <= maxprojects; s++) {
+		/* Show the number of citites on each project. */
+		GetDItem(win, ++num, NULL, &itemhandle, NULL);
+		NumToString(participants[oldprojects[s]], pnumber);
+		SetIText(itemhandle, pnumber);
+		ShowDItem(win, num);
+		/* Show the "are currently researching" text. */
+		ShowDItem(win, ++text);
+		/* Show the popup menu with the advance name. */
+		GetDItem(win, ++menu, NULL, &itemhandle, NULL);
+		SetCtlMax((ControlHandle) itemhandle, m);	/* Important! */
+		for (i = 1; i <= m; i++) {
+			GetItem(popMenu, i, pname);
+			p2c(pname, cname);
+			if (strcmp(a_type_name(oldprojects[s]), cname) != 0)
+				continue;
+			SetCtlValue((ControlHandle) itemhandle, i);
+		}
+		ShowDItem(win, menu);
+		/* Don't allow other sides to use the menu. */
+		if (disabled)
+			HiliteControl((ControlHandle) itemhandle, 255); 
+		/* Show the status info on the advence. */
+		GetDItem(win, ++status, NULL, &itemhandle, NULL);
+		NumToString(side->advance[oldprojects[s]], pnumber);
+		p2c(pnumber, cname);
+		strcat(cname, " / ");
+		NumToString(a_rp(oldprojects[s]), pnumber);
+		p2c(pnumber, sname);
+		strcat(cname, sname);
+		c2p(cname, pnumber);
+		SetIText(itemhandle, pnumber);
+		ShowDItem(win, status);
+		/* Show the auto checkbox. */
+		GetDItem(win, ++check, NULL, &itemhandle, NULL);
+		SetCtlValue((ControlHandle) itemhandle, TRUE);
+		for_all_side_units(side, unit2) {
+			if (!u_advanced(unit2->type))
+				continue;
+			if (unit2->curadvance != oldprojects[s])
+				continue;
+			if (unit2->autoresearch != TRUE)
+				SetCtlValue((ControlHandle) itemhandle, FALSE);
+		}
+		ShowDItem(win, check);
+		/* Don't allow other sides to use the checkbox. */
+		if (disabled)
+			HiliteControl((ControlHandle) itemhandle, 255); 
+	}
+
+	/* Finally handle the All units checkbox. */
+	GetDItem(win, diGlobalAllCheck, NULL, &itemhandle, NULL);
+	/* First assume autoresearch is ON for all units. */
+	SetCtlValue((ControlHandle) itemhandle, TRUE);
+	for_all_side_units(side, unit2) {
+		if (!u_advanced(unit2->type))
+			continue;
+		/*Uncheck the box if at least one unit is not on autoresearch. */ 
+		if (unit2->autoresearch != TRUE) {		
+			SetCtlValue((ControlHandle) itemhandle, FALSE);
+			break;
+		}
+	}
+
+	/* Don't allow other sides to use the checkbox. */
+	if (disabled)
+		HiliteControl((ControlHandle) itemhandle, 255); 
+
+	ShowWindow(win);
+	while (!done) {
+		SetCursor(&QD(arrow));
+		draw_default_button(win, OkButton);
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+			case 	OkButton:
+				/* Don't allow others to change anything. */
+				if (disabled) {
+					done = TRUE;
+					break;
+				}					
+				/* First deal with any idle units. */
+				if (loafers) {
+					GetDItem(win, diGlobalIdleMenu, NULL, &itemhandle, NULL);
+					mitem = GetCtlValue((ControlHandle) itemhandle);
+					GetItem(popMenu, mitem, pname);
+					p2c(pname, sname);
+					/* A new task was chosen for the idle units. */
+					if (strcmp("Idle", sname) != 0) {
+						for_all_side_units(side, unit2) {
+							if (!u_advanced(unit2->type))
+								continue;
+							/* Set all Idle units and those who worked on the just acquired advance 
+							to do selected research instead. */
+							if (unit2->curadvance == NOADVANCE || unit2->curadvance == advance)
+								unit2->curadvance = atype_from_name(sname);
+						}
+					} else {
+						for_all_side_units(side, unit2) {
+							if (!u_advanced(unit2->type))
+								continue;
+							/* Set all units who worked on the just acquired advance to Idle. */
+							if (unit2->curadvance == advance)
+								unit2->curadvance = NOADVANCE;
+						}						
+					}
+					/* Then deal with the first auto checkbox. */
+					GetDItem(win, diGlobalIdleCheck, NULL, &itemhandle, NULL);
+					check = GetCtlValue((ControlHandle) itemhandle);
+					for_all_side_units(side, unit2) {
+						if (!u_advanced(unit2->type))
+							continue;
+						if (strcmp("Idle", sname) == 0) {
+							if (unit2->curadvance == NOADVANCE || unit2->curadvance == advance) {
+								unit2->autoresearch = check;
+							} else continue;
+						} else {
+							if (unit2->curadvance == atype_from_name(sname)) {
+								unit2->autoresearch = check;
+							} else	 continue;
+						}
+					}
+				}
+				/* Then deal with ongoing research projects. */
+				for (i = 1; i <= maxprojects; i++) {
+					GetDItem(win, diGlobalIdleMenu + i, NULL, &itemhandle, NULL);
+					mitem = GetCtlValue((ControlHandle) itemhandle);
+					GetItem(popMenu, mitem, pname);
+					p2c(pname, sname);
+					/* Only proceed if we changed the project. */
+					if (strcmp(a_type_name(oldprojects[i]), sname) != 0) {
+						for_all_side_units(side, unit2) {
+							if (!u_advanced(unit2->type))
+								continue;
+							/* Change units involved in the old project to new goal. */
+							if (unit2->curadvance == oldprojects[i]) {
+								if (strcmp("Idle", sname) == 0)
+									unit2->curadvance = NOADVANCE;
+								else 	unit2->curadvance = atype_from_name(sname);			
+							}
+						}
+					}
+					/* Finally deal with auto checkboxes. */
+					GetDItem(win, diGlobalIdleCheck + i, NULL, &itemhandle, NULL);
+					check = GetCtlValue((ControlHandle) itemhandle);
+					for_all_side_units(side, unit2) {
+						if (!u_advanced(unit2->type))
+							continue;
+						if (unit2->curadvance != oldprojects[i])
+							continue;
+						unit2->autoresearch = check;
+					}
+				}
+				done = TRUE;
+				break;
+
+			case 	CancelButton:
+				done = TRUE;
+				break;
+
+			default:
+				/* Handle auto checkboxes. */
+				for (i = 0; i <= maxprojects;  i++) {
+					if (ditem == diGlobalIdleCheck + i) {
+						GetDItem(win, ditem, NULL, &itemhandle, NULL);
+						SetCtlValue((ControlHandle) itemhandle, 
+							!GetCtlValue((ControlHandle) itemhandle));
+					}
+				}
+				/* Handle All units checkbox. Toggle all other checkboxes. */
+				if (ditem == diGlobalAllCheck) {
+					GetDItem(win, ditem, NULL, &itemhandle, NULL);
+					SetCtlValue((ControlHandle) itemhandle, 
+						!GetCtlValue((ControlHandle) itemhandle));
+					check = GetCtlValue((ControlHandle) itemhandle);
+					for (i = 0; i <= maxprojects;  i++) {
+						GetDItem(win, diGlobalIdleCheck + i, NULL, &itemhandle, NULL);
+						SetCtlValue((ControlHandle) itemhandle, check);
+					}
+				}
+				break;
+		}
+	}
+	/* Close the dialog. */
+	DisposeDialog(win);
+	/* Restore old port. */
+	SetPort(oldport);
 }
 
 /* The side renaming dialog includes places for all the different name-related
@@ -1203,15 +1706,11 @@ side_rename_dialog(Side *side)
 				net_set_side_adjective(dside, side, get_side_rename_string(win, diSideRenameAdjective));
 				net_set_side_emblemname(dside, side, get_side_rename_string(win, diSideRenameEmblemName));
 				net_set_side_colorscheme(dside, side, get_side_rename_string(win, diSideRenameColorScheme));
-
-				/* Trash and reload all emblem cicns at the top of the heap zone,
-				   so as to avoid fragmentation */				
-				reinit_all_emblem_images();
 				current_colorscheme[side_number(side)] = copy_string(side->colorscheme);
 				dissect_colorscheme(side_number(side));
+				init_emblem_images();
 				ui_update_state();
 				save_preferences();
-				
 				/* Tweak the side menu. */
 				update_side_menu(dside);
 				/* Force redisplay of everything that might use any side names. */
@@ -1227,7 +1726,7 @@ side_rename_dialog(Side *side)
 				net_set_side_pluralnoun(dside, side, NULL);
 				net_set_side_adjective(dside, side, NULL);
 				make_up_side_name(side);
-				init_emblem_images(side);
+				init_emblem_images();
 				changed = TRUE;
 				break;
 		}
@@ -1312,302 +1811,6 @@ unit_rename_dialog(Unit *unit)
 	return TRUE;
 }
 
-/* Unit closeups. */
-
-UnitCloseup *
-find_unit_closeup(Unit *unit)
-{
-	UnitCloseup *unitcloseup;
-
-	for_all_unit_closeups(unitcloseup) {
-		if (unitcloseup->unit == unit
-			&& unitcloseup->window
-			&& ((WindowPeek) unitcloseup->window)->visible)
-		  return unitcloseup;
-	}
-	return NULL;
-}
-
-void
-create_unit_closeup(Unit *unit)
-{
-	int u, w, h;
-	WindowPtr win;
-	UnitCloseup *unitcloseup = (UnitCloseup *) xmalloc(sizeof(UnitCloseup));
-
-	if (!active_display(dside) || unit == NULL)
-	  return;
-	DGprintf("Creating a closeup of %s\n", unit_desig(unit));
-	u = unit->type;
-	unitcloseup->unit = unit;
-	if (hasColorQD) {
-		win = GetNewCWindow(wFloatFixed, NULL, NULL);
-	} else {
-		win = GetNewWindow(wFloatFixed, NULL, NULL);
-	}
-	unitcloseup->window = win;
-
-	/* Put it adjacent to the clicked or list-selected unit. */
-	if (frontmap) {
-		GrafPtr oldport;
-		Point upoint;
-		int sx, sy;
-						
-		GetPort(&oldport);
-		SetPort(frontmap->window);
-		xform(frontmap, unit->x + 2, unit->y - 2, &sx, &sy);
-		upoint.h = sx; upoint.v = sy;
-		LocalToGlobal(&upoint);
-		MoveWindow(win, upoint.h, upoint.v, true);
-		SetPort(oldport);
-	} else {
-		/* This only happens if no maps are open and a closeup is chosen from a list. */
-		stagger_window(unitcloseup->window, &lastunitcloseuph, &lastunitcloseupv);
-	}
-	preferred_closeup_size(u, &w, &h);
-	SizeWindow(win, w, h, 1);
-	sprintf(spbuf, "Closeup on %s", short_unit_handle(unit));
-	add_window_menu_item(spbuf, win);
-	unitcloseup->next = unitcloseuplist;
-	unitcloseuplist = unitcloseup;
-	/* We're now ready to show this closeup to the world. */
-	MakeFloat(win);
-	SelectTheWindow(win);
-}
-
-/* Compute the right size for a unit's closeup.  More complicated units
-   with more state get bigger closeups. */
-
-void
-preferred_closeup_size(int u, int *widp, int *hgtp)
-{
-	int wid = closeupwinwid, hgt, m, u2, count;
-
-	closeup_spacing = small_line_spacing + 1;
-	/* Add 4 pixels at bottom to make room for 3rd task. */
-	hgt = 4 + 32 + 4 + 3 * closeup_spacing + 4;
-	if (nummrows == NULL) {
-		/* Compute and cache the space needed to display each unit's supply. */
-		nummrows = xmalloc(numutypes);
-		for_all_unit_types(u2) {
-			nummrows[u2] = count = 0;
-			for_all_material_types(m) {
-				if (um_storage_x(u2, m) > 0)
-		  		  ++count;
-		  	}
-		  	if (count > 0) {
-		  		nummrows[u2] = max(1, (count + (m_per_row - 1)) / m_per_row);
-		  	}
-		}
-	}
-	hgt += nummrows[u] * closeup_spacing;
-	if (u_acp(u) > 0)
-	  hgt += 5 * closeup_spacing;
-	/* (should make some room for tooling state) */
-	*widp = wid;  *hgtp = hgt;
-}
-
-/* Draw all the fields and displays in a unit closeup. */
-
-void
-draw_unit_closeup(UnitCloseup *unitcloseup)
-{
-	int u, m, sx = 4, sy, count, mrow, sx2, sy2;
-	char infobuf[BUFSIZE];
-	Rect tmprect;
-	GrafPtr oldport;
-	WindowPtr win = unitcloseup->window;
-	Unit *unit = unitcloseup->unit;
-	Map	*map;
-
-	update_cell_display(dside, unit->x, unit->y, UPDATE_ALWAYS);
-	if (!active_display(dside))
-	  return;
-	/* We now permit peeking at enemy units. */
-	if (!in_play(unit)) { 
-		remove_window_menu_item(win);
-		destroy_unit_closeup(unitcloseup);
-		HideTheWindow(win);
-		return;
-	}
-	GetPort(&oldport);
-	SetPort(win);
-	/* Use the small font. */
-	TextFont(small_font_id);
-	TextSize(small_font_size);
-	EraseRect(&win->portRect);
-	u = unit->type;
-	/* Draw the unit's image. */
-	SetRect(&tmprect, sx, sx, sx + 32, sx + 32); 
-	EraseRect(&tmprect);
-	draw_unit_image(win, tmprect.left, tmprect.top,
-					tmprect.right - tmprect.left, tmprect.bottom - tmprect.top,
-					u, side_number(unit->side), !completed(unit), TRUE);
-
-	/* Draw unit size on top of advanced unit. */
-	if (u_advanced(unit->type) && default_drawsizes) 
-		draw_unit_size(unit, win, tmprect.left, tmprect.top,
-						tmprect.right - tmprect.left, tmprect.bottom - tmprect.top);
-
-	/* Draw the unit's name. */
-	name_or_number(unit, infobuf);
-	MoveTo(44, 4 + closeup_spacing);
-	DrawText(infobuf, 0, strlen(infobuf));
-	if (Debug || DebugG || DebugM) {
-		sprintf(infobuf, " %d ", unit->id);
-		MoveTo(closeupwinwid - 30, 15);
-		DrawText(infobuf, 0, strlen(infobuf));
-	}
-	/* Draw the unit's side and type. */
-	side_and_type_name(infobuf, dside, u, unit->side);
-	MoveTo(44, 4 + closeup_spacing * 2);
-	DrawText(infobuf, 0, strlen(infobuf));
-	/* Draw the unit's location. */
-	location_desc(infobuf, dside, unit, unit->type, unit->x, unit->y);
-	sy = 4 + 32 + 4 + closeup_spacing;
-	MoveTo(sx, sy);
-	DrawText(infobuf, 0, strlen(infobuf));
-	/* Very briefly list the numbers and types of the occupants. */
-	occupants_desc(infobuf, unit);
-	sy += closeup_spacing;
-	MoveTo(sx, sy);
-	DrawText(infobuf, 0, strlen(infobuf));
-	/* Draw the unit's hit points. */
-	hp_desc(infobuf, unit, TRUE);
-	strcat(infobuf, "   ");
-	acp_desc(infobuf + strlen(infobuf), unit, TRUE);
-	sy += closeup_spacing;
-	MoveTo(sx, sy);
-	DrawText(infobuf, 0, strlen(infobuf));
-	/* Draw the unit's supplies. */
-	mrow = 0;
-	while (supply_desc(infobuf, unit, mrow)) {
-		sy += closeup_spacing;
-		MoveTo(sx, sy);
-		DrawText(infobuf, 0, strlen(infobuf));
-		++mrow;
-	}
-	/* Draw the unit's plan, if it has one. */
-	if (unit->plan) {
-
-		Task *task;
-		Plan *plan = unit->plan;
-		
-		/* plan_desc now also includes goal_desc on the same line. To avoid 
-		repetition only the first half of plan_desc is therefore included below. */ 
-		strcpy(infobuf, "Plan: ");
-		strcat(infobuf, plantypenames[plan->type]);
-		if (plan->waitingfortasks)
-		  strcat(infobuf, " Waiting");
-		if (plan->asleep)
-		  strcat(infobuf, " Asleep");
-		if (plan->reserve)
-		  strcat(infobuf, " Reserve");
-		if (plan->delayed)
-		  strcat(infobuf, " Delay");
-		if (!plan->aicontrol)
-		  strcat(infobuf, " NoAI");
-		if (plan->supply_is_low)
-		  strcat(infobuf, " SupplyLow");
-		if (plan->tasks) {
-			int i = 0;
-			for_all_tasks(plan, task)
-				++i;
-			tprintf(infobuf, " (%d task%s)", i, (i == 1 ? "" : "s"));
-		} 
-		sy += closeup_spacing;
-		MoveTo(sx, sy);
-		DrawText(infobuf, 0, strlen(infobuf));
-    	if (plan->maingoal) {
-			strcpy(infobuf, "Goal: ");
-	    		goal_desc(tmpbuf, plan->maingoal);
-			strncat(infobuf, tmpbuf, strlen(tmpbuf));
-			sy += closeup_spacing;			
-			MoveTo(sx, sy);
-			DrawText(infobuf, 0, strlen(infobuf));
-    	}
-    	if (plan->formation) {
-			strcpy(infobuf, "Formation: ");
-    			goal_desc(infobuf+strlen(infobuf), plan->formation);
-			sy += closeup_spacing;			
-			MoveTo(sx, sy);
-			DrawText(infobuf, 0, strlen(infobuf));
-    	}
-		if (plan->tasks) {
-			int i = 0;
-	    		for_all_tasks(plan, task) {
-				++i;
-				sprintf(infobuf, "Task %d: ", i);
-				task_desc(tmpbuf, unit->side, unit, task);			
-				strcat(infobuf, tmpbuf);
-				sy += closeup_spacing;			
-				MoveTo(sx, sy);
-				DrawText(infobuf, 0, strlen(infobuf));
-			}
-		}
-	}
-	SetPort(oldport);
-}
-
-
-UnitCloseup *
-unit_closeup_from_window(WindowPtr win)
-{
-	UnitCloseup *unitcloseup;
-	
-	for_all_unit_closeups(unitcloseup) {
-		if (unitcloseup->window == win)
-		  return unitcloseup;
-	}
-	return NULL;
-}
-
-int
-do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int mods)
-{
-	ControlHandle control;
-	short part;
-	WindowPtr window = unitcloseup->window;
-
-	part = FindControl(mouse, window, &control);
-	if (0 /* some control */) {
-	} else {
-	
-		Unit	*unit;
-		Map	*map;
-
-		unit = unitcloseup->unit;
-		for_all_maps(map) {
-			unselect_all(map);
-			select_unit_on_map(map, unit);
-		}
-		update_cell_display(dside, unit->x, unit->y, UPDATE_ALWAYS);
-		return TRUE;
-	}
-}
-
-void
-destroy_unit_closeup(UnitCloseup *unitcloseup)
-{
-	UnitCloseup *unitcloseup2;
-	Unit	*unit;
-	Map	*map;
-
-	unit = unitcloseup->unit;
-	update_cell_display(dside, unit->x, unit->y, UPDATE_ALWAYS);
-	if (unitcloseuplist == unitcloseup) {
-		unitcloseuplist = unitcloseup->next;
-	} else {
-		for_all_unit_closeups(unitcloseup2) {
-			if (unitcloseup2->next == unitcloseup) {
-				unitcloseup2->next = unitcloseup->next;
-			}
-		}
-	}
-	/* (should destroy substructs) */
-	free(unitcloseup);
-}
-
 /* History window. */
 
 /* This is the top-level access to bring up the history window, can be called
