@@ -1,5 +1,5 @@
 /* Commands for the tcl/tk interface to Xconq.
-   Copyright (C) 1998, 1999 Stanley T. Shebs.
+   Copyright (C) 1998-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -83,13 +83,15 @@ do_add_terrain(Side *side)
 	}
     }
     if (numtypes == 0) {
-	cmd_error(side, "%s cannot add or alter terrain!", unit_handle(side, unit));
+	cmd_error(side, "%s cannot add or alter terrain!",
+		  unit_handle(side, unit));
     } else if (numtypes == 1) {
 	map->argunitid = unit->id;
 	do_add_terrain_2(side, map, tfirst);
     } else {
 	map->argunitid = unit->id;
-	ask_terrain_type(side, map, "Type to add:", possibles, aux_add_terrain);
+	ask_terrain_type(side, map, "Type to add:", possibles,
+			 aux_add_terrain);
     }
 }
 
@@ -118,10 +120,7 @@ aux_add_terrain(Side *side, Map *map, int cancel)
 /* This is like do_add_terrain, but with a terrain type given. */
 
 static void
-do_add_terrain_2(side, map, t)
-Side *side;
-Map *map;
-int t;
+do_add_terrain_2(Side *side, Map *map, int t)
 {
     char abuf[100];
 
@@ -153,17 +152,24 @@ aux_add_terrain_2(Side *side, Map *map, int cancel)
 		    break;
 		  case bordersubtype:
 		  case connectionsubtype:
+		    if (x == unit->x && y == unit->y) {
+			beep(side);
+			notify(side, "Pick an adjacent location.");
+			map->modalhandler = aux_add_terrain_2;
+		    }
 		    dir = closest_dir(x - unit->x, y - unit->y);
-		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y, dir, map->tmpt);
+		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y,
+						dir, map->tmpt);
 		    break;
 		  case coatingsubtype:
-		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y, 1, map->tmpt);
+		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y,
+						1, map->tmpt);
 		    break;
 		}
 	    }
 	}
     } else {
-	beep(side);     
+	beep(side);
 	map->modalhandler = aux_add_terrain_2;
     }
 }
@@ -240,7 +246,9 @@ do_build(Side *side)
     }
     numtypes = 0;
     for_all_unit_types(u2) {
-	if (uu_acp_to_create(u, u2) > 0 && type_allowed_on_side(u, side)) {
+	if (uu_acp_to_create(u, u2) > 0
+	    && type_allowed_on_side(u2, side)
+	    && has_advance_to_build(side, u2)) {
 	    possibles[u2] = TRUE;
 	    ++numtypes;
 	    ufirst = u2;
@@ -332,11 +340,13 @@ aux_build_2(Side *side, Map *map, int cancel)
     if (grok_position(side, map, &x, &y, NULL)) {
 	unit = find_unit(map->argunitid);
 	if (unit == NULL) {
-	    cmd_error(side, "Unit disappeared!");
+	    cmd_error(side, "Unit (#%d) disappeared!", map->argunitid);
 	} else if (!inside_area(x, y)) {
-	    cmd_error(side, "Cannot build here.");
+	    cmd_error(side, "%s cannot build on the edge of the world.",
+		      unit_handle(side, unit));
 	} else {
 	    impl_build(side, unit, map->inptype, x, y, map->prefixarg);
+	    map->inptype = NONUTYPE;
 	}
     } else {
         beep(side);
@@ -353,7 +363,7 @@ impl_build(Side *side, Unit *unit, int u2, int x, int y, int n)
       n = 99;
     notify(side, "%s will build %d %s.",
 	   unit_handle(side, unit), n, u_type_name(u2));
-    net_set_build_task(unit, u2, n);
+    net_set_build_task(unit, u2, n, x, y);
     /* We may need to move to get to where we're supposed to build. */
     if (distance(x, y, unit->x, unit->y) > uu_create_range(unit->type, u2)) {
 	net_push_move_to_task(unit, x, y, uu_create_range(unit->type, u2));
