@@ -487,11 +487,13 @@ tk_make_color_pixmap(Tk_Window rootwin, ImageFamily *imf, Image *img)
 	      pix = tkimg->cmap[rc]->pixel;
 	    if (use_images) {
 		if (imdepth == 16) {
-#if 0
+#if 1
 		    *(((short *) ximbuf) + (img->w * r + c)) = pix;
 #endif
-		    ximbuf[(img->w * r + c) * 2 + 0] = pix & 0xff;
-		    ximbuf[(img->w * r + c) * 2 + 1] = pix >> 8;
+#if 0
+		    ximbuf[(img->w * r + c) * 2 + 1] = pix & 0xff;
+		    ximbuf[(img->w * r + c) * 2 + 0] = pix >> 8;
+#endif
 		} else {
 		    XPutPixel(xim, c, r, pix);
 		}
@@ -725,7 +727,7 @@ blacken_masked_area(ImageFamily *imf, Image *img)
 	}
     }
     if (black < 0
-	&& (1 << img->pixelsize) >= img->numcolors
+	&& (1 << img->pixelsize) == img->numcolors
 	&& img->pixelsize < 8) {
 	char *newdata, *nrp;
 	int newpsize, nrmask, nri;
@@ -767,6 +769,7 @@ blacken_masked_area(ImageFamily *imf, Image *img)
 	    ++nrp;
 	  }
 	}
+	img->orig_pixelsize = img->pixelsize;
 	img->pixelsize = newpsize;
 	img->rawcolrdata = newdata;
     }
