@@ -34,6 +34,7 @@ static Tk_Cursor make_cursor(Tk_Window tkwin, char *cursbits, char *maskbits, in
 static void init_bitmaps(void);
 
 static void init_unit_images(void);
+static void init_material_images(void);
 static void init_terrain_images(void);
 static void init_emblem_images(void);
 static void report_missing_images(void);
@@ -193,7 +194,6 @@ init_display(void)
       dpyname = getenv("DISPLAY");
 
     /* Set up things shared by all the windows. */
-    dside->ui->dflt_color_terr_images = TRUE;
     dside->ui->dflt_color_embl_images = TRUE;
     dside->ui->default_latlong_interval = 600;
     for (p = 0; p < NUMPOWERS; ++p) {
@@ -221,7 +221,9 @@ init_display(void)
     if (DebugG == 2)
       XSynchronize(Tk_Display(tmp_root_window), True);
 
+    init_ui_chars();
     init_unit_images();
+    init_material_images();
     init_terrain_images();
     init_emblem_images();
     init_other_images();
@@ -765,6 +767,81 @@ init_unit_images(void)
     }
 }
 
+extern ImageFamily *get_material_type_images(Side *side, int m);
+
+/* (should be in ui.c) */
+
+ImageFamily *
+get_material_type_images(Side *side, int m)
+{
+    char *name;
+    ImageFamily *imf;
+
+    if (!empty_string(m_image_name(m)))
+      name = m_image_name(m);
+    else
+      name = m_type_name(m);
+    imf = get_generic_images(name);
+#if 0
+    if (imf != NULL && imf->numsizes == 0) {
+	imf->ersatz = TRUE;
+	imf = add_default_material_image(imf, m);
+    }
+#endif
+    record_imf_get(imf);
+    return imf;
+}
+
+static void
+init_material_images(void)
+{
+    int m, p;
+    ImageFamily *imf;
+    Image *img, *mimg;
+    TkImage *tkimg;
+
+    dside->ui->mimages =
+      (ImageFamily **) xmalloc(nummtypes * sizeof(ImageFamily *));
+    for_all_material_types(m) {
+	imf = get_material_type_images(dside, m);
+	if (DebugG)
+	  describe_imf(dside, "material type", m_type_name(m), imf);
+	dside->ui->mimages[m] = imf;
+	/* Precalculate which images to use at which magnifications. */
+	for (p = 0; p < NUMPOWERS; ++p) {
+	    mimg = best_image(dside->ui->mimages[m], hws[p], hhs[p]);
+	    dside->ui->bestmimages[p][m] = mimg;
+#if 0
+	    if (mimg != NULL) {
+		tkimg = (TkImage *) timg->hook;
+		if (tkimg != NULL) {
+		    if (!dside->ui->monochrome && tkimg->colr != None)
+		      dside->ui->terrpics[p][t] = tkimg->colr;
+		    else
+		      dside->ui->terrpics[p][t] = tkimg->mono;
+		}
+	    }
+#endif
+	}
+	/* Look for a solid color. */
+	mimg = NULL;
+	for_all_images(dside->ui->mimages[m], img) {
+	    if (img->w == 1 && img->h == 1) {
+		mimg = img;
+		break;
+	    }
+	}
+	if (mimg != NULL && mimg->w == 1 && mimg->h == 1) {
+	    if (mimg->hook) {
+		tkimg = (TkImage *) mimg->hook;
+		/* Save the color if found. */
+		if (tkimg->solid)
+		  dside->ui->material_color[m] = tkimg->solid;
+	    }
+	}
+    }
+}
+
 /* Collect images for all the terrain types. */
 
 static void init_shades(int t);
@@ -774,7 +851,7 @@ init_terrain_images(void)
 {
     int t, p;
     ImageFamily *imf;
-    Image *img, *timg;
+    Image *img, *timg, *subimg;
     TkImage *tkimg;
 
     dside->ui->timages =
@@ -789,13 +866,14 @@ init_terrain_images(void)
 	    timg = best_image(dside->ui->timages[t], hws[p], hhs[p]);
 	    dside->ui->besttimages[p][t] = timg;
 	    if (timg != NULL) {
-		tkimg = (TkImage *) timg->hook;
+		subimg = timg;
+		/* If we have multiple subimages, pick the first as
+                   representative. */
+		if (timg->numsubimages > 0 && timg->subimages)
+		  subimg = timg->subimages[0];
+		tkimg = (TkImage *) subimg->hook;
 		if (tkimg != NULL) {
-		    if (!dside->ui->monochrome
-			/* if somebody changes their mind about using
-                           the color images, this will lose */
-			&& dside->ui->dflt_color_terr_images
-			&& tkimg->colr != None)
+		    if (!dside->ui->monochrome && tkimg->colr != None)
 		      dside->ui->terrpics[p][t] = tkimg->colr;
 		    else
 		      dside->ui->terrpics[p][t] = tkimg->mono;
