@@ -21,9 +21,6 @@ extern char *outdirname;
 
 static void usage(void);
 
-int my_rid;
-int numremotes;
-
 main(int argc, char *argv[])
 {
     char *arg, *mask_color_name = NULL;
