@@ -314,7 +314,7 @@ int u2, n;
       n = 99;
     notify(side, "%s will build %d %s.",
 	   unit_handle(side, unit), n, u_type_name(u2));
-    net_push_build_task(unit, u2, n);
+    net_push_build_task(unit, u2, n, 0, 0);
 }
 
 void
