@@ -230,7 +230,9 @@ void
 init_ui(side)
 Side *side;
 {
-    if (side_wants_display(side)) {
+	/* This is a hack. Should figure out why the indepside
+	 * is asking for a display! */
+    if (side_wants_display(side) && side != indepside) {
         side->ui = (UI *) xmalloc(sizeof(UI));
 	Dprintf("One UI is %d bytes.\n", sizeof(UI));
     } else {
