@@ -1,5 +1,5 @@
 /* Map handling for the X11 interface to Xconq.
-   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999, 2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -1472,8 +1472,7 @@ void (*handler)(Side *side, Map *map, int cancelled);
     for_all_terrain_types(t) {
 	if (possibles == NULL || possibles[t]) {
 	    map->tvec[numtypes] = t;
-	    map->tstr[numtypes] =
-	      (!empty_string(t_char(t)) ? t_char(t)[0] : (t - 'a'));
+	    map->tstr[numtypes] = terrchars[t];
 	    ++numtypes;
 	}
     }
