@@ -135,7 +135,11 @@ XtIntervalId *id;
     run_local_ai(1, 20);
     /* Run the kernel itself. */
     rslt += net_run_game(1);
+
+#if 0
     run_local_ai(2, 20);
+#endif
+
     /* Check for any input from remotes. */
     if (my_rid > 0) {
 	flush_outgoing_queue();
@@ -412,9 +416,7 @@ int rightnow;
 	    }
 	}
 	if (endofgame && !side->ui->told_outcome) {
-	    side->may_set_see_all = TRUE;
-	    if (!all_see_all) {
-		side->see_all = TRUE;
+	    if (!side->see_all) {
 		for_all_maps(side, map) {
 		    update_controls(side, map);
 		    draw_map(side, map);
